@@ -31,15 +31,16 @@ class TestNodepoolIntegration(BaseTestCase):
         super(TestNodepoolIntegration, self).setUp()
 
         self.statsd = None
-        self.zk = zuul.zk.ZooKeeper(enable_cache=True)
-        self.addCleanup(self.zk.disconnect)
-        self.zk.connect('localhost:2181')
+        self.zk_client = zuul.zk.ZooKeeperClient('localhost:2181')
+        self.addCleanup(self.zk_client.disconnect)
+        self.zk_client.connect()
         self.hostname = socket.gethostname()
 
         self.provisioned_requests = []
         # This class implements the scheduler methods zuul.nodepool
         # needs, so we pass 'self' as the scheduler.
-        self.nodepool = zuul.nodepool.Nodepool(self)
+        self.nodepool = zuul.nodepool.Nodepool(
+            self.zk_client, self.hostname, self.statsd, self)
 
     def waitForRequests(self):
         # Wait until all requests are complete.
@@ -104,8 +105,8 @@ class TestNodepoolIntegration(BaseTestCase):
         job.nodeset = nodeset
         self.fake_nodepool.paused = True
         request = self.nodepool.requestNodes(None, job, 0)
-        self.zk.client.stop()
-        self.zk.client.start()
+        self.zk_client.client.stop()
+        self.zk_client.client.start()
         self.fake_nodepool.paused = False
         self.waitForRequests()
         self.assertEqual(len(self.provisioned_requests), 1)
