@@ -16,17 +16,20 @@ from configparser import ConfigParser
 import fixtures
 import logging
 import textwrap
+import testtools
 
-from zuul.configloader import AuthorizationRuleParser
+from zuul import model
+from zuul.configloader import AuthorizationRuleParser, safe_load_yaml
 
 from tests.base import ZuulTestCase
+from zuul.model import SourceContext
 
 
 class TenantParserTestCase(ZuulTestCase):
     create_project_keys = True
 
     CONFIG_SET = set(['pipeline', 'job', 'semaphore', 'project',
-                      'project-template', 'nodeset', 'secret'])
+                      'project-template', 'nodeset', 'secret', 'queue'])
     UNTRUSTED_SET = CONFIG_SET - set(['pipeline'])
 
     def setupAllProjectKeys(self, config: ConfigParser):
@@ -75,6 +78,41 @@ class TestTenantSimple(TenantParserTestCase):
         self.assertEqual(job[0].variant_description, "")
         self.assertEqual(job[1].variant_description, "stable")
 
+    def test_merge_anchor(self):
+        to_parse = textwrap.dedent(
+            """
+            - job:
+                name: job1
+                vars: &docker_vars
+                  registry: 'registry.example.org'
+
+            - job:
+                name: job2
+                vars:
+                  <<: &buildenv_vars
+                    image_name: foo
+                  <<: *docker_vars
+
+            - job:
+                name: job3
+                vars:
+                  <<: *buildenv_vars
+                  <<: *docker_vars
+            """)
+        tenant = self.scheds.first.sched.abide.tenants.get('tenant-one')
+        project = tenant.config_projects[0]
+        source_context = SourceContext(project, 'master', 'zuul.yaml', True)
+
+        data = safe_load_yaml(to_parse, source_context)
+        self.assertEqual(len(data), 3)
+        job_vars = [i['job']['vars'] for i in data]
+        # Test that merging worked
+        self.assertEqual(job_vars, [
+            {'registry': 'registry.example.org'},
+            {'registry': 'registry.example.org', 'image_name': 'foo'},
+            {'registry': 'registry.example.org', 'image_name': 'foo'},
+        ])
+
 
 class TestTenantOverride(TenantParserTestCase):
     tenant_config_file = 'config/tenant-parser/override.yaml'
@@ -365,6 +403,24 @@ class TestTenantConfigBranches(ZuulTestCase):
         # Now job must be defined on stable branch
         self._validate_job(common_job, 'stable')
 
+        # Now try to break the config in common-config on stable
+        in_repo_conf = textwrap.dedent(
+            """
+            - job:
+                name: base
+                parent: non-existing
+            """)
+        file_dict = {'zuul.yaml': in_repo_conf}
+        A = self.fake_gerrit.addFakeChange('common-config', 'stable', 'A',
+                                           files=file_dict)
+        self.fake_gerrit.addEvent(A.getPatchsetCreatedEvent(1))
+        self.waitUntilSettled()
+
+        # No job should have run due to the change introducing a config error
+        self.assertHistory([])
+        self.assertTrue(A.reported)
+        self.assertTrue('Job non-existing not defined' in A.messages[0])
+
 
 class TestSplitConfig(ZuulTestCase):
     tenant_config_file = 'config/split-config/main.yaml'
@@ -397,6 +453,11 @@ class TestSplitConfig(ZuulTestCase):
         self.assertIn('project1-project2-integration',
                       project1_config[0].pipelines['check'].job_list.jobs)
 
+        # This check ensures the .zuul.ignore flag file is working in
+        # the config directory.
+        self.assertEquals(
+            len(tenant.layout.loading_errors), 0)
+
     def test_dynamic_split_config(self):
         in_repo_conf = textwrap.dedent(
             """
@@ -428,17 +489,25 @@ class TestSplitConfig(ZuulTestCase):
                                                files=file_dict)
             self.fake_gerrit.addEvent(A.getPatchsetCreatedEvent(1))
             self.waitUntilSettled()
+            return A
 
         log_fixture = self.useFixture(
             fixtures.FakeLogger(level=logging.WARNING))
 
         log_fixture._output.truncate(0)
-        add_file("common-config", "zuul.yaml")
-        self.assertIn("Multiple configuration", log_fixture.output)
+        A = add_file("common-config", "zuul.yaml")
+        self.assertIn("Configuration in common-config/zuul.d/jobs.yaml@master "
+                      "ignored because project-branch is already configured",
+                      log_fixture.output)
+        self.assertIn("Configuration in common-config/zuul.d/jobs.yaml@master "
+                      "ignored because project-branch is already configured",
+                      A.messages[0])
 
         log_fixture._output.truncate(0)
         add_file("org/project1", ".zuul.yaml")
-        self.assertIn("Multiple configuration", log_fixture.output)
+        self.assertIn("Configuration in org/project1/.zuul.d/gate.yaml@master "
+                      "ignored because project-branch is already configured",
+                      log_fixture.output)
 
 
 class TestConfigConflict(ZuulTestCase):
@@ -453,6 +522,62 @@ class TestConfigConflict(ZuulTestCase):
             jobs)
 
 
+class TestUnparsedConfigCache(ZuulTestCase):
+    tenant_config_file = 'config/single-tenant/main.yaml'
+
+    def test_config_caching(self):
+        cache = self.scheds.first.sched. unparsed_config_cache
+        tenant = self.scheds.first.sched.abide.tenants["tenant-one"]
+
+        common_cache = cache.getFilesCache("review.example.com/common-config",
+                                           "master")
+        tpc = tenant.project_configs["review.example.com/common-config"]
+        self.assertTrue(common_cache.isValidFor(tpc, cache_ltime=-1))
+        self.assertEqual(len(common_cache), 1)
+        self.assertIn("zuul.yaml", common_cache)
+        self.assertTrue(len(common_cache["zuul.yaml"]) > 0)
+
+        project_cache = cache.getFilesCache("review.example.com/org/project",
+                                            "master")
+        # Cache of org/project should be valid but empty (no in-repo config)
+        tpc = tenant.project_configs["review.example.com/org/project"]
+        self.assertTrue(project_cache.isValidFor(tpc, cache_ltime=-1))
+        self.assertEqual(len(project_cache), 0)
+
+    def test_cache_use(self):
+        sched = self.scheds.first.sched
+        # Stop cleanup thread so it's not removing projects from the cache
+        # during the test.
+        sched.cleanup_stop.set()
+        sched.cleanup_thread.join()
+        tenant = sched.abide.tenants['tenant-one']
+        _, project = tenant.getProject('org/project2')
+
+        # Get the current ltime from Zookeeper and run a full reconfiguration,
+        # so that we know all items in the cache have a larger ltime.
+        ltime = self.getCurrentLtime()
+        self.scheds.first.fullReconfigure()
+
+        # Clear the unparsed branch cache so all projects (except for
+        # org/project2) are retrieved from the cache in Zookeeper.
+        sched.abide.unparsed_project_branch_cache.clear()
+        self.gearman_server.jobs_history.clear()
+
+        # Create a tenant reconfiguration event with a known ltime that is
+        # smaller than the ltime of the items in the cache.
+        event = model.TenantReconfigureEvent(
+            tenant.name, project.canonical_name, branch_name=None)
+        event.zuul_event_ltime = ltime
+        sched.management_events.put(event, needs_result=False)
+        self.waitUntilSettled()
+
+        # As the cache should be valid, we only expect a cat job for
+        # org/project2
+        cat_jobs = [job for job in self.gearman_server.jobs_history
+                    if job.name == b"merger:cat"]
+        self.assertEqual(len(cat_jobs), 1)
+
+
 class TestAuthorizationRuleParser(ZuulTestCase):
     tenant_config_file = 'config/tenant-parser/authorizations.yaml'
 
@@ -695,3 +820,15 @@ class TestTenantExtra(TenantParserTestCase):
             dict(name='project2-job', result='SUCCESS', changes='2,1'),
             dict(name='project2-extra-file2', result='SUCCESS', changes='2,1'),
         ], ordered=False)
+
+
+class TestTenantDuplicate(TenantParserTestCase):
+    tenant_config_file = 'config/tenant-parser/duplicate.yaml'
+
+    def setUp(self):
+        with testtools.ExpectedException(Exception, 'Duplicate configuration'):
+            super().setUp()
+
+    def test_tenant_dupe(self):
+        # The magic is in setUp
+        pass
