@@ -55,7 +55,7 @@ class TestGerritToGithubCRD(ZuulTestCase):
         self.assertEqual(A.data['status'], 'NEW')
         self.assertFalse(B.is_merged)
 
-        for connection in self.connections.connections.values():
+        for connection in self.scheds.first.connections.connections.values():
             connection.maintainCache([])
 
         self.executor_server.hold_jobs_in_build = True
@@ -517,7 +517,7 @@ class TestGithubToGerritCRD(ZuulTestCase):
         self.assertFalse(A.is_merged)
         self.assertEqual(B.data['status'], 'NEW')
 
-        for connection in self.connections.connections.values():
+        for connection in self.scheds.first.connections.connections.values():
             connection.maintainCache([])
 
         self.executor_server.hold_jobs_in_build = True
