@@ -106,8 +106,8 @@ class TestExecutorRepos(ZuulTestCase):
         A.addApproval('Code-Review', 2)
         B.addApproval('Code-Review', 2)
         self.fake_gerrit.addEvent(A.addApproval('Approved', 1))
+        self.waitUntilSettled()
         self.fake_gerrit.addEvent(B.addApproval('Approved', 1))
-
         self.waitUntilSettled()
 
         self.assertEqual(2, len(self.builds), "Two builds are running")
@@ -145,9 +145,10 @@ class TestExecutorRepos(ZuulTestCase):
         B.addApproval('Code-Review', 2)
         C.addApproval('Code-Review', 2)
         self.fake_gerrit.addEvent(A.addApproval('Approved', 1))
+        self.waitUntilSettled()
         self.fake_gerrit.addEvent(B.addApproval('Approved', 1))
+        self.waitUntilSettled()
         self.fake_gerrit.addEvent(C.addApproval('Approved', 1))
-
         self.waitUntilSettled()
 
         self.assertEqual(3, len(self.builds), "Three builds are running")
@@ -205,10 +206,12 @@ class TestExecutorRepos(ZuulTestCase):
         C.addApproval('Code-Review', 2)
         D.addApproval('Code-Review', 2)
         self.fake_gerrit.addEvent(A.addApproval('Approved', 1))
+        self.waitUntilSettled()
         self.fake_gerrit.addEvent(B.addApproval('Approved', 1))
+        self.waitUntilSettled()
         self.fake_gerrit.addEvent(C.addApproval('Approved', 1))
+        self.waitUntilSettled()
         self.fake_gerrit.addEvent(D.addApproval('Approved', 1))
-
         self.waitUntilSettled()
 
         self.assertEqual(4, len(self.builds), "Four builds are running")
@@ -382,7 +385,7 @@ class TestExecutorRepos(ZuulTestCase):
         # one to make sure that our post job runs with the one that we
         # intended rather than simply the current repo state.
         B = self.fake_gerrit.addFakeChange('org/project1', 'master', 'B',
-                                           parent='refs/changes/1/1/1')
+                                           parent='refs/changes/01/1/1')
         B.setMerged()
         B_commit = str(upstream[p1].commit('master'))
         self.log.debug("B commit: %s" % B_commit)
@@ -459,6 +462,42 @@ class TestAnsibleJob(ZuulTestCase):
                                           })[0]['host_keys']
         self.assertEqual(keys[0], '[localhost]:22022 fake-host-key')
 
+        # Test with no host keys
+        node['host_keys'] = []
+        host = self.test_job.getHostList({'nodes': [node],
+                                          'host_vars': {},
+                                          'vars': {},
+                                          'groups': [],
+                                          })[0]
+        self.assertEqual(host['host_keys'], [])
+        self.assertEqual(
+            host['host_vars']['ansible_ssh_common_args'],
+            '-o StrictHostKeyChecking=false')
+
+    def test_getHostList_shell_type(self):
+        # Test without shell type set
+        node = {'name': 'fake-host',
+                'host_keys': ['fake-host-key'],
+                'interface_ip': 'localhost'}
+        host = self.test_job.getHostList({'nodes': [node],
+                                          'host_vars': {},
+                                          'vars': {},
+                                          'groups': [],
+                                          })[0]
+        self.assertNotIn('ansible_shell_type', host['host_vars'])
+
+        # Test with custom shell type set.
+        node['shell_type'] = 'cmd'
+        host = self.test_job.getHostList({'nodes': [node],
+                                          'host_vars': {},
+                                          'vars': {},
+                                          'groups': [],
+                                          })[0]
+        self.assertIn('ansible_shell_type', host['host_vars'])
+        self.assertEqual(
+            host['host_vars']['ansible_shell_type'],
+            'cmd')
+
 
 class TestExecutorHostname(ZuulTestCase):
     config_file = 'zuul-executor-hostname.conf'
@@ -901,7 +940,14 @@ class TestExecutorExtraPackages(AnsibleZuulTestCase):
         for version in ansible_manager._supported_versions:
             command = [ansible_manager.getAnsibleCommand(version, 'pip'),
                        'uninstall', '-y', self.test_package]
-            subprocess.run(command)
+            # We want to error if the uninstall fails as the test below
+            # relies on the package not being installed to be properly
+            # exercised.
+            s = subprocess.run(command,
+                               stdout=subprocess.PIPE,
+                               stderr=subprocess.STDOUT)
+            self.log.info(s.stdout)
+            self.assertEqual(s.returncode, 0)
 
     @mock.patch('zuul.lib.ansible.ManagedAnsible.extra_packages',
                 new_callable=mock.PropertyMock)
