@@ -228,6 +228,31 @@ class TestGerritWeb(ZuulTestCase):
 
         self.assertEqual([], tested_change_ids)
 
+    def test_recheck(self):
+        A = self.fake_gerrit.addFakeChange('org/project', 'master', 'A')
+        self.fake_gerrit.addEvent(A.getPatchsetCreatedEvent(1))
+        self.waitUntilSettled()
+        self.assertEqual(3, len(self.history))
+
+        self.fake_gerrit.addEvent(A.getChangeCommentEvent(1))
+        self.waitUntilSettled()
+        self.assertEqual(3, len(self.history))
+
+        self.fake_gerrit.addEvent(A.getChangeCommentEvent(1,
+                                  'recheck'))
+        self.waitUntilSettled()
+        self.assertEqual(6, len(self.history))
+
+        self.fake_gerrit.addEvent(A.getChangeCommentEvent(1,
+                                  patchsetcomment='recheck'))
+        self.waitUntilSettled()
+        self.assertEqual(9, len(self.history))
+
+        self.fake_gerrit.addEvent(A.getChangeCommentEvent(1,
+                                  patchsetcomment='do not recheck'))
+        self.waitUntilSettled()
+        self.assertEqual(9, len(self.history))
+
 
 class TestFileComments(AnsibleZuulTestCase):
     config_file = 'zuul-gerrit-web.conf'
@@ -246,10 +271,25 @@ class TestFileComments(AnsibleZuulTestCase):
                          'SUCCESS')
         self.assertEqual(self.getJobFromHistory('file-comments-error').result,
                          'SUCCESS')
-        self.assertEqual(len(A.comments), 4)
-        comments = sorted(A.comments, key=lambda x: x['line'])
+        self.assertEqual(len(A.comments), 6)
+        comments = sorted(A.comments, key=lambda x: (x['file'], x['line']))
         self.assertEqual(
             comments[0],
+            {
+                'file': 'otherfile.txt',
+                'line': 21,
+                'message': 'This is a much longer message.\n\n'
+                'With multiple paragraphs.\n',
+                'reviewer': {
+                    'email': 'zuul@example.com',
+                    'name': 'Zuul',
+                    'username': 'jenkins'
+                },
+            },
+        )
+
+        self.assertEqual(
+            comments[1],
             {
                 "file": "path/to/file.py",
                 "line": 2,
@@ -261,24 +301,56 @@ class TestFileComments(AnsibleZuulTestCase):
                 },
             },
         )
-        self.assertEqual(comments[1],
-                         {'file': 'otherfile.txt',
-                          'line': 21,
-                          'message': 'This is a much longer message.\n\n'
-                          'With multiple paragraphs.\n',
-                          'reviewer': {'email': 'zuul@example.com',
-                                       'name': 'Zuul',
-                                       'username': 'jenkins'}}
+
+        self.assertEqual(
+            comments[2],
+            {
+                "file": "path/to/file.py",
+                "line": 21,
+                "message": (
+                    "A second zuul return value using the same file should not"
+                    "\noverride the first result, but both should be merged.\n"
+                ),
+                "reviewer": {
+                    "email": "zuul@example.com",
+                    "name": "Zuul",
+                    "username": "jenkins",
+                },
+            },
         )
-        self.assertEqual(comments[2],
-                         {'file': 'path/to/file.py',
-                          'line': 42,
-                          'message': 'line too long',
-                          'reviewer': {'email': 'zuul@example.com',
-                                       'name': 'Zuul',
-                                       'username': 'jenkins'}}
+
+        self.assertEqual(
+            comments[3],
+            {
+                'file': 'path/to/file.py',
+                'line': 42,
+                'message': 'line too long',
+                'reviewer': {
+                    'email': 'zuul@example.com',
+                    'name': 'Zuul',
+                    'username': 'jenkins'
+                },
+            },
         )
-        self.assertEqual(comments[3],
+
+        self.assertEqual(
+            comments[4],
+            {
+                "file": "path/to/file.py",
+                "line": 42,
+                "message": (
+                    "A second comment applied to the same line in the same "
+                    "file\nshould also be added to the result.\n"
+                ),
+                "reviewer": {
+                    "email": "zuul@example.com",
+                    "name": "Zuul",
+                    "username": "jenkins",
+                }
+            }
+        )
+
+        self.assertEqual(comments[5],
                          {'file': 'path/to/file.py',
                           'line': 82,
                           'message': 'line too short',
@@ -306,13 +378,27 @@ class TestChecksApi(ZuulTestCase):
 
         self.assertEqual(A.checks_history[0]['zuul:check']['state'],
                          'NOT_STARTED')
+
         self.assertEqual(A.checks_history[1]['zuul:check']['state'],
                          'SCHEDULED')
+        self.assertEqual(
+            A.checks_history[1]['zuul:check']['url'],
+            'http://zuul.example.com/t/tenant-one/status/change/2,1')
+
         self.assertEqual(A.checks_history[2]['zuul:check']['state'],
                          'RUNNING')
+        self.assertEqual(
+            A.checks_history[2]['zuul:check']['url'],
+            'http://zuul.example.com/t/tenant-one/status/change/2,1')
+
         self.assertEqual(A.checks_history[3]['zuul:check']['state'],
                          'SUCCESSFUL')
+        self.assertTrue(
+            A.checks_history[3]['zuul:check']['url'].startswith(
+                'http://zuul.example.com/t/tenant-one/buildset/'))
+
         self.assertEqual(len(A.checks_history), 4)
+
         self.assertTrue(isinstance(
             A.checks_history[3]['zuul:check']['started'], str))
         self.assertTrue(isinstance(
@@ -458,7 +544,7 @@ class TestChecksApi(ZuulTestCase):
         self.executor_server.hold_jobs_in_build = False
         self.executor_server.release()
         self.waitUntilSettled()
-        print(A.checks_history)
+        self.log.info(A.checks_history)
         self.assertEqual(A.checks_history[3]['zuul:check']['state'],
                          'NOT_STARTED')
         self.assertEqual(A.checks_history[4]['zuul:check']['state'],
@@ -552,3 +638,35 @@ class TestPolling(ZuulTestCase):
         self.assertHistory([
             dict(name='tag-job', result='SUCCESS'),
         ])
+
+
+class TestWrongConnection(ZuulTestCase):
+    config_file = 'zuul-connections-multiple-gerrits.conf'
+    tenant_config_file = 'config/wrong-connection-in-pipeline/main.yaml'
+
+    def test_wrong_connection(self):
+        # Test if the wrong connection is configured in a gate pipeline
+
+        # Our system has two gerrits, and we have configured a gate
+        # pipeline to trigger on the "review_gerrit" connection, but
+        # report (and merge) via "another_gerrit".
+        A = self.fake_review_gerrit.addFakeChange('org/project', 'master', 'A')
+        A.addApproval('Code-Review', 2)
+        self.fake_review_gerrit.addEvent(A.addApproval('Approved', 1))
+
+        self.waitUntilSettled()
+
+        B = self.fake_review_gerrit.addFakeChange('org/project', 'master', 'B')
+        # Let's try this as if the change was merged (say, via another tenant).
+        B.setMerged()
+        B.addApproval('Code-Review', 2)
+        self.fake_review_gerrit.addEvent(B.addApproval('Approved', 1))
+
+        self.waitUntilSettled()
+
+        self.assertEqual(A.reported, 0)
+        self.assertEqual(B.reported, 0)
+        self.assertHistory([
+            dict(name='test-job', result='SUCCESS', changes='1,1'),
+            dict(name='test-job', result='SUCCESS', changes='2,1'),
+        ], ordered=False)
