@@ -12,9 +12,10 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
+import json
 import os
 import re
-from testtools.matchers import MatchesRegex, StartsWith
+from testtools.matchers import MatchesRegex, Not, StartsWith
 import urllib
 import socket
 import time
@@ -24,6 +25,7 @@ from unittest import mock, skip
 import git
 import github3.exceptions
 
+from tests.fakegithub import FakeGithubEnterpriseClient
 from zuul.driver.github.githubconnection import GithubShaCache
 import zuul.rpcclient
 
@@ -348,7 +350,7 @@ class TestGithubDriver(ZuulTestCase):
         # Stop queuing timer triggered jobs so that the assertions
         # below don't race against more jobs being queued.
         self.commitConfigUpdate('org/common-config',
-                                'layouts/basic-github.yaml')
+                                'layouts/no-timer-github.yaml')
         self.scheds.execute(lambda app: app.sched.reconfigure(app.config))
         self.waitUntilSettled()
         # If APScheduler is in mid-event when we remove the job, we
@@ -456,8 +458,9 @@ class TestGithubDriver(ZuulTestCase):
         # We should only have one status for the head sha
         self.assertEqual(1, len(statuses))
         check_status = statuses[0]
-        check_url = ('http://zuul.example.com/status/#%s,%s' %
-                     (A.number, A.head_sha))
+        check_url = (
+            'http://zuul.example.com/t/tenant-one/status/change/%s,%s' %
+            (A.number, A.head_sha))
         self.assertEqual('tenant-one/check', check_status['context'])
         self.assertEqual('check status: pending',
                          check_status['description'])
@@ -472,13 +475,12 @@ class TestGithubDriver(ZuulTestCase):
         statuses = self.fake_github.getCommitStatuses(project, A.head_sha)
         self.assertEqual(2, len(statuses))
         check_status = statuses[0]
-        check_url = ('http://zuul.example.com/status/#%s,%s' %
-                     (A.number, A.head_sha))
+        check_url = 'http://zuul.example.com/t/tenant-one/buildset/'
         self.assertEqual('tenant-one/check', check_status['context'])
         self.assertEqual('check status: success',
                          check_status['description'])
         self.assertEqual('success', check_status['state'])
-        self.assertEqual(check_url, check_status['url'])
+        self.assertThat(check_status['url'], StartsWith(check_url))
         self.assertEqual(1, len(A.comments))
         self.assertThat(A.comments[0],
                         MatchesRegex(r'.*Build succeeded.*', re.DOTALL))
@@ -508,8 +510,7 @@ class TestGithubDriver(ZuulTestCase):
         self.assertEqual('success', report_status['state'])
         self.assertEqual(2, len(A.comments))
 
-        base = 'http://logs.example.com/tenant-one/reporting/%s/%s/' % (
-            A.project, A.number)
+        base = 'http://zuul.example.com/t/tenant-one/buildset/'
 
         # Deconstructing the URL because we don't save the BuildSet UUID
         # anywhere to do a direct comparison and doing regexp matches on a full
@@ -518,9 +519,9 @@ class TestGithubDriver(ZuulTestCase):
         # The first part of the URL matches the easy base string
         self.assertThat(report_status['url'], StartsWith(base))
 
-        # The rest of the URL is a UUID and a trailing slash.
+        # The rest of the URL is a UUID
         self.assertThat(report_status['url'][len(base):],
-                        MatchesRegex(r'^[a-fA-F0-9]{32}\/$'))
+                        MatchesRegex(r'^[a-fA-F0-9]{32}$'))
 
     @simple_layout('layouts/reporting-github.yaml', driver='github')
     def test_truncated_status_description(self):
@@ -567,10 +568,10 @@ class TestGithubDriver(ZuulTestCase):
         self.waitUntilSettled()
 
         # there should only be one report, a status
-        self.assertEqual(1, len(self.fake_github.reports))
+        self.assertEqual(1, len(self.fake_github.github_data.reports))
         # Verify the user/context/state of the status
         status = ('zuul', 'tenant-one/push-reporting', 'pending')
-        self.assertEqual(status, self.fake_github.reports[0][-1])
+        self.assertEqual(status, self.fake_github.github_data.reports[0][-1])
 
         # free the executor, allow the build to finish
         self.executor_server.hold_jobs_in_build = False
@@ -578,10 +579,10 @@ class TestGithubDriver(ZuulTestCase):
         self.waitUntilSettled()
 
         # Now there should be a second report, the success of the build
-        self.assertEqual(2, len(self.fake_github.reports))
+        self.assertEqual(2, len(self.fake_github.github_data.reports))
         # Verify the user/context/state of the status
         status = ('zuul', 'tenant-one/push-reporting', 'success')
-        self.assertEqual(status, self.fake_github.reports[-1][-1])
+        self.assertEqual(status, self.fake_github.github_data.reports[-1][-1])
 
         # now make a PR which should also comment
         self.executor_server.hold_jobs_in_build = True
@@ -591,7 +592,7 @@ class TestGithubDriver(ZuulTestCase):
 
         # Now there should be a four reports, a new comment
         # and status
-        self.assertEqual(4, len(self.fake_github.reports))
+        self.assertEqual(4, len(self.fake_github.github_data.reports))
         self.executor_server.release()
         self.waitUntilSettled()
 
@@ -630,12 +631,18 @@ class TestGithubDriver(ZuulTestCase):
     def test_report_pull_merge(self):
         # pipeline merges the pull request on success
         A = self.fake_github.openFakePullRequest('org/project', 'master',
-                                                 'PR title')
+                                                 'PR title',
+                                                 body='I shouldnt be seen',
+                                                 body_text='PR body')
         self.fake_github.emitEvent(A.getCommentAddedEvent('merge me'))
         self.waitUntilSettled()
         self.assertTrue(A.is_merged)
         self.assertThat(A.merge_message,
-                        MatchesRegex(r'.*PR title.*Reviewed-by.*', re.DOTALL))
+                        MatchesRegex(r'.*PR title\n\nPR body.*', re.DOTALL))
+        self.assertThat(A.merge_message,
+                        Not(MatchesRegex(
+                            r'.*I shouldnt be seen.*',
+                            re.DOTALL)))
         self.assertEqual(len(A.comments), 0)
 
         # pipeline merges the pull request on success after failure
@@ -668,7 +675,49 @@ class TestGithubDriver(ZuulTestCase):
         # Validate that the merge failure comment contains the message github
         # returned
         self.assertEqual(D.comments[0],
-                         'Pull request merge failed: 403 Merge not allowed')
+                         'Pull request merge failed: Merge not allowed '
+                         'because of fake reason')
+
+    @simple_layout('layouts/merging-github.yaml', driver='github')
+    def test_report_pull_merge_message_reviewed_by(self):
+        # pipeline merges the pull request on success
+        A = self.fake_github.openFakePullRequest('org/project', 'master', 'A')
+
+        self.fake_github.emitEvent(A.getCommentAddedEvent('merge me'))
+        self.waitUntilSettled()
+        self.assertTrue(A.is_merged)
+        # assert that no 'Reviewed-By' is in merge commit message
+        self.assertThat(A.merge_message,
+                        Not(MatchesRegex(r'.*Reviewed-By.*', re.DOTALL)))
+
+        B = self.fake_github.openFakePullRequest('org/project', 'master', 'B')
+        B.addReview('derp', 'APPROVED')
+
+        self.fake_github.emitEvent(B.getCommentAddedEvent('merge me'))
+        self.waitUntilSettled()
+        self.assertTrue(B.is_merged)
+        # assert that single 'Reviewed-By' is in merge commit message
+        self.assertThat(B.merge_message,
+                        MatchesRegex(
+                            r'.*Reviewed-by: derp <derp@example.com>.*',
+                            re.DOTALL))
+
+        C = self.fake_github.openFakePullRequest('org/project', 'master', 'C')
+        C.addReview('derp', 'APPROVED')
+        C.addReview('herp', 'COMMENTED')
+
+        self.fake_github.emitEvent(C.getCommentAddedEvent('merge me'))
+        self.waitUntilSettled()
+        self.assertTrue(C.is_merged)
+        # assert that multiple 'Reviewed-By's are in merge commit message
+        self.assertThat(C.merge_message,
+                        MatchesRegex(
+                            r'.*Reviewed-by: derp <derp@example.com>.*',
+                            re.DOTALL))
+        self.assertThat(C.merge_message,
+                        MatchesRegex(
+                            r'.*Reviewed-by: herp <herp@example.com>.*',
+                            re.DOTALL))
 
     @simple_layout('layouts/dependent-github.yaml', driver='github')
     def test_draft_pr(self):
@@ -703,8 +752,9 @@ class TestGithubDriver(ZuulTestCase):
         # We should only have one status for the head sha
         self.assertEqual(1, len(statuses))
         check_status = statuses[0]
-        check_url = ('http://zuul.example.com/status/#%s,%s' %
-                     (A.number, A.head_sha))
+        check_url = (
+            'http://zuul.example.com/t/tenant-one/status/change/%s,%s' %
+            (A.number, A.head_sha))
         self.assertEqual('tenant-one/check', check_status['context'])
         self.assertEqual('check status: pending', check_status['description'])
         self.assertEqual('pending', check_status['state'])
@@ -718,12 +768,11 @@ class TestGithubDriver(ZuulTestCase):
         statuses = self.fake_github.getCommitStatuses(project, A.head_sha)
         self.assertEqual(2, len(statuses))
         check_status = statuses[0]
-        check_url = ('http://zuul.example.com/status/#%s,%s' %
-                     (A.number, A.head_sha))
+        check_url = 'http://zuul.example.com/t/tenant-one/buildset/'
         self.assertEqual('tenant-one/check', check_status['context'])
         self.assertEqual('success', check_status['state'])
         self.assertEqual('check status: success', check_status['description'])
-        self.assertEqual(check_url, check_status['url'])
+        self.assertThat(check_status['url'], StartsWith(check_url))
         self.assertEqual(1, len(A.comments))
         self.assertThat(A.comments[0],
                         MatchesRegex(r'.*Build succeeded.*', re.DOTALL))
@@ -905,13 +954,15 @@ class TestGithubDriver(ZuulTestCase):
                                      expect_reconfigure=False,
                                      old_sha=None, new_sha=None,
                                      modified_files=None,
+                                     removed_files=None,
                                      expected_cat_jobs=None):
         pevent = self.fake_github.getPushEvent(
             project=project,
             ref='refs/heads/%s' % branch,
             old_rev=old_sha,
             new_rev=new_sha,
-            modified_files=modified_files)
+            modified_files=modified_files,
+            removed_files=removed_files)
 
         # record previous tenant reconfiguration time, which may not be set
         old = self.scheds.first.sched.tenant_last_reconfigured\
@@ -962,6 +1013,7 @@ class TestGithubDriver(ZuulTestCase):
         github = self.fake_github.getGithubClient()
         repo = github.repo_from_project(project)
         repo._create_branch(branch)
+        repo._set_branch_protection(branch, False)
 
         self.fake_github.emitEvent(
             self.fake_github.getPushEvent(
@@ -972,22 +1024,24 @@ class TestGithubDriver(ZuulTestCase):
         A = self.fake_github.openFakePullRequest(project, branch, 'A')
         old_sha = A.head_sha
         A.setMerged("merging A")
+        new_sha = random_sha1()
 
         self._test_push_event_reconfigure(project, branch,
                                           expect_reconfigure=True,
                                           old_sha=old_sha,
+                                          new_sha=new_sha,
                                           modified_files=['zuul.yaml'],
                                           expected_cat_jobs=1)
 
-        # Check if deleting that branch will not lead to a reconfiguration as
-        # this branch is not protected
+        # there are no exclude-unprotected-branches in the test class, so a
+        # reconfigure shall occur
         repo._delete_branch(branch)
 
         self._test_push_event_reconfigure(project, branch,
-                                          expect_reconfigure=False,
-                                          old_sha=old_sha,
+                                          expect_reconfigure=True,
+                                          old_sha=new_sha,
                                           new_sha='0' * 40,
-                                          modified_files=[])
+                                          removed_files=['zuul.yaml'])
 
     # TODO(jlk): Make this a more generic test for unknown project
     @skip("Skipped for rewrite of webhook handler")
@@ -1062,14 +1116,17 @@ class TestGithubDriver(ZuulTestCase):
         github = self.fake_github.getGithubClient()
         repo = github.repo_from_project(project)
         repo._create_branch(branch)
+        repo._set_branch_protection(branch, False)
 
         A = self.fake_github.openFakePullRequest(project, branch, 'A')
         old_sha = A.head_sha
         A.setMerged("merging A")
+        new_sha = random_sha1()
 
         self._test_push_event_reconfigure(project, branch,
                                           expect_reconfigure=False,
                                           old_sha=old_sha,
+                                          new_sha=new_sha,
                                           modified_files=['README.md'])
 
         # Check if deleting that branch is ignored as well
@@ -1077,7 +1134,7 @@ class TestGithubDriver(ZuulTestCase):
 
         self._test_push_event_reconfigure(project, branch,
                                           expect_reconfigure=False,
-                                          old_sha=old_sha,
+                                          old_sha=new_sha,
                                           new_sha='0' * 40,
                                           modified_files=['README.md'])
 
@@ -1230,10 +1287,27 @@ class TestGithubDriver(ZuulTestCase):
         self.assertEqual(2, len(self.history))
 
         # now check if the merge was done via rebase
-        merges = [report for report in self.fake_github.reports
+        merges = [report for report in self.fake_github.github_data.reports
                   if report[2] == 'merge']
         assert(len(merges) == 1 and merges[0][3] == 'squash')
 
+    @simple_layout('layouts/basic-github.yaml', driver='github')
+    def test_invalid_event(self):
+        # Regression test to make sure the event forwarder thread continues
+        # running in case the event from the GithubEventProcessor is None.
+        self.fake_github.emitEvent(("pull_request", "invalid"))
+        self.waitUntilSettled()
+
+        A = self.fake_github.openFakePullRequest('org/project', 'master',
+                                                 'A')
+        self.fake_github.emitEvent(A.getPullRequestOpenedEvent())
+        self.waitUntilSettled()
+
+        self.assertEqual('SUCCESS',
+                         self.getJobFromHistory('project-test1').result)
+        self.assertEqual('SUCCESS',
+                         self.getJobFromHistory('project-test2').result)
+
 
 class TestGithubUnprotectedBranches(ZuulTestCase):
     config_file = 'zuul-github-driver.conf'
@@ -1284,7 +1358,7 @@ class TestGithubUnprotectedBranches(ZuulTestCase):
 
         # Enable branch protection on org/project3@stable. We'll use a PR on
         # this branch as a depends-on to validate that the stable branch
-        # which is not protected in org/project3 is not filtered out.
+        # which is not protected in org/project2 is not filtered out.
         repo = github.repo_from_project('org/project3')
         self.create_branch('org/project3', 'stable')
         repo._set_branch_protection('stable', True)
@@ -1462,6 +1536,90 @@ class TestGithubUnprotectedBranches(ZuulTestCase):
 
         self.assertEqual(1, len(self.history))
 
+    def _test_push_event_reconfigure(self, project, branch,
+                                     expect_reconfigure=False,
+                                     old_sha=None, new_sha=None,
+                                     modified_files=None,
+                                     removed_files=None,
+                                     expected_cat_jobs=None):
+        pevent = self.fake_github.getPushEvent(
+            project=project,
+            ref='refs/heads/%s' % branch,
+            old_rev=old_sha,
+            new_rev=new_sha,
+            modified_files=modified_files,
+            removed_files=removed_files)
+
+        # record previous tenant reconfiguration time, which may not be set
+        old = self.scheds.first.sched.tenant_last_reconfigured\
+            .get('tenant-one', 0)
+        self.waitUntilSettled()
+
+        if expected_cat_jobs is not None:
+            # clear the gearman jobs history so we can count the cat jobs
+            # issued during reconfiguration
+            self.gearman_server.jobs_history.clear()
+
+        self.fake_github.emitEvent(pevent)
+        self.waitUntilSettled()
+        new = self.scheds.first.sched.tenant_last_reconfigured\
+            .get('tenant-one', 0)
+
+        if expect_reconfigure:
+            # New timestamp should be greater than the old timestamp
+            self.assertLess(old, new)
+        else:
+            # Timestamps should be equal as no reconfiguration shall happen
+            self.assertEqual(old, new)
+
+        if expected_cat_jobs is not None:
+            # Check the expected number of cat jobs here as the (empty) config
+            # of org/project should be cached.
+            cat_jobs = set([job for job in self.gearman_server.jobs_history
+                           if job.name == b'merger:cat'])
+            self.assertEqual(expected_cat_jobs, len(cat_jobs), cat_jobs)
+
+    def test_push_event_reconfigure_complex_branch(self):
+
+        branch = 'feature/somefeature'
+        project = 'org/project2'
+
+        # prepare an existing branch
+        self.create_branch(project, branch)
+
+        github = self.fake_github.getGithubClient()
+        repo = github.repo_from_project(project)
+        repo._create_branch(branch)
+        repo._set_branch_protection(branch, False)
+
+        self.fake_github.emitEvent(
+            self.fake_github.getPushEvent(
+                project,
+                ref='refs/heads/%s' % branch))
+        self.waitUntilSettled()
+
+        A = self.fake_github.openFakePullRequest(project, branch, 'A')
+        old_sha = A.head_sha
+        A.setMerged("merging A")
+        new_sha = random_sha1()
+
+        # branch is not protected, no reconfiguration even if config file
+        self._test_push_event_reconfigure(project, branch,
+                                          expect_reconfigure=False,
+                                          old_sha=old_sha,
+                                          new_sha=new_sha,
+                                          modified_files=['zuul.yaml'],
+                                          expected_cat_jobs=0)
+
+        # branch is not protected: no reconfiguration
+        repo._delete_branch(branch)
+
+        self._test_push_event_reconfigure(project, branch,
+                                          expect_reconfigure=False,
+                                          old_sha=new_sha,
+                                          new_sha='0' * 40,
+                                          removed_files=['zuul.yaml'])
+
 
 class TestGithubWebhook(ZuulTestCase):
     config_file = 'zuul-github-driver.conf'
@@ -1471,8 +1629,11 @@ class TestGithubWebhook(ZuulTestCase):
 
         # Start the web server
         self.web = self.useFixture(
-            ZuulWebFixture(self.gearman_server.port,
-                           self.config, self.test_root))
+            ZuulWebFixture(self.changes, self.config,
+                           self.additional_event_queues, self.upstream_root,
+                           self.rpcclient, self.poller_events,
+                           self.git_url_with_auth, self.addCleanup,
+                           self.test_root))
 
         host = '127.0.0.1'
         # Wait until web server is started
@@ -1657,6 +1818,28 @@ class TestGithubAppDriver(ZuulGithubAppTestCase):
             check_run["output"]["summary"],
             MatchesRegex(r'.*Starting checks-api-reporting jobs.*', re.DOTALL)
         )
+        # The external id should be a json-string containing all relevant
+        # information to uniquely identify this change.
+        self.assertEqual(
+            json.dumps(
+                {
+                    "tenant": "tenant-one",
+                    "pipeline": "checks-api-reporting",
+                    "change": 1
+                }
+            ),
+            check_run["external_id"],
+        )
+        # A running check run should provide a custom abort action
+        self.assertEqual(1, len(check_run["actions"]))
+        self.assertEqual(
+            {
+                "identifier": "abort",
+                "description": "Abort this check run",
+                "label": "Abort",
+            },
+            check_run["actions"][0],
+        )
 
         # TODO (felix): How can we test if the details_url was set correctly?
         # How can the details_url be configured on the test case?
@@ -1678,6 +1861,8 @@ class TestGithubAppDriver(ZuulGithubAppTestCase):
             MatchesRegex(r'.*Build succeeded.*', re.DOTALL)
         )
         self.assertIsNotNone(check_run["completed_at"])
+        # A completed check run should not provide any custom actions
+        self.assertEqual(0, len(check_run["actions"]))
 
         # Tell gate to merge to test checks requirements
         self.fake_github.emitEvent(A.getCommentAddedEvent('merge me'))
@@ -1744,19 +1929,22 @@ class TestGithubAppDriver(ZuulGithubAppTestCase):
         project = "org/project3"
         github = self.fake_github.getGithubClient(None)
 
+        # Make check run creation fail
+        github._data.fail_check_run_creation = True
+
         # pipeline reports pull request status both on start and success
         self.executor_server.hold_jobs_in_build = True
         A = self.fake_github.openFakePullRequest(project, "master", "A")
         self.fake_github.emitEvent(A.getPullRequestOpenedEvent())
         self.waitUntilSettled()
 
-        # We should have a pending check for the head sha
+        # We should have no pending check for the head sha
         commit = github.repo_from_project(project)._commits.get(A.head_sha)
         check_runs = commit.check_runs()
-        self.assertEqual(1, len(check_runs))
+        self.assertEqual(0, len(check_runs))
 
-        # Delete this check_run to simulate a failed check_run creation
-        commit._check_runs = []
+        # Make check run creation work again
+        github._data.fail_check_run_creation = False
 
         # Now run the build and check if the update of the check_run could
         # still be accomplished.
@@ -1776,6 +1964,8 @@ class TestGithubAppDriver(ZuulGithubAppTestCase):
             MatchesRegex(r'.*Build succeeded.*', re.DOTALL)
         )
         self.assertIsNotNone(check_run["completed_at"])
+        # A completed check run should not provide any custom actions
+        self.assertEqual(0, len(check_run["actions"]))
 
     @simple_layout("layouts/reporting-github.yaml", driver="github")
     def test_update_check_run_missing_permissions(self):
@@ -1804,13 +1994,72 @@ class TestGithubAppDriver(ZuulGithubAppTestCase):
         self.assertEqual(0, len(check_runs))
 
         expected_warning = (
-            "Failed to update check run tenant-one/checks-api-reporting: "
+            "Failed to create check run tenant-one/checks-api-reporting: "
             "403 Resource not accessible by integration"
         )
         self.assertEqual(2, len(A.comments))
         self.assertIn(expected_warning, A.comments[0])
         self.assertIn(expected_warning, A.comments[1])
 
+    @simple_layout("layouts/reporting-github.yaml", driver="github")
+    def test_abort_check_run(self):
+        "Test that we can dequeue a change by aborting the related check run"
+        project = "org/project3"
+
+        self.executor_server.hold_jobs_in_build = True
+        A = self.fake_github.openFakePullRequest(project, "master", "A")
+        self.fake_github.emitEvent(A.getPullRequestOpenedEvent())
+        self.waitUntilSettled()
+
+        # We should have a pending check for the head sha that provides an
+        # abort action.
+        check_runs = self.fake_github.getCommitChecks(project, A.head_sha)
+
+        self.assertEqual(1, len(check_runs))
+        check_run = check_runs[0]
+        self.assertEqual("tenant-one/checks-api-reporting", check_run["name"])
+        self.assertEqual("in_progress", check_run["status"])
+        self.assertEqual(1, len(check_run["actions"]))
+        self.assertEqual("abort", check_run["actions"][0]["identifier"])
+        self.assertEqual(
+            {
+                "tenant": "tenant-one",
+                "pipeline": "checks-api-reporting",
+                "change": 1
+            },
+            json.loads(check_run["external_id"])
+        )
+
+        # Simulate a click on the "Abort" button in Github by faking a webhook
+        # event with our custom abort action.
+        # Handling this event should dequeue the related change
+        self.fake_github.emitEvent(A.getCheckRunAbortEvent(check_run))
+        self.waitUntilSettled()
+
+        tenant = self.scheds.first.sched.abide.tenants.get("tenant-one")
+        check_pipeline = tenant.layout.pipelines["check"]
+        self.assertEqual(0, len(check_pipeline.getAllItems()))
+        self.assertEqual(1, self.countJobResults(self.history, "ABORTED"))
+
+        # The buildset was already dequeued, so there shouldn't be anything to
+        # release.
+        self.executor_server.hold_jobs_in_build = False
+        self.executor_server.release()
+        self.waitUntilSettled()
+
+        # Since the change/buildset was dequeued, the check run should be
+        # reported as cancelled and don't provide any further action.
+        check_runs = self.fake_github.getCommitChecks(project, A.head_sha)
+        self.assertEqual(1, len(check_runs))
+        aborted_check_run = check_runs[0]
+
+        self.assertEqual(
+            "tenant-one/checks-api-reporting", aborted_check_run["name"]
+        )
+        self.assertEqual("completed", aborted_check_run["status"])
+        self.assertEqual("cancelled", aborted_check_run["conclusion"])
+        self.assertEqual(0, len(aborted_check_run["actions"]))
+
 
 class TestCheckRunAnnotations(ZuulGithubAppTestCase, AnsibleZuulTestCase):
     """We need Github app authentication and be able to run Ansible jobs"""
@@ -1921,3 +2170,149 @@ class TestCheckRunAnnotations(ZuulGithubAppTestCase, AnsibleZuulTestCase):
             "start_line": 3,
             "end_line": 3,
         })
+
+    def test_many_file_comments(self):
+        # Test that we only send 50 comments to github
+        project = "org/project"
+        github = self.fake_github.getGithubClient(None)
+
+        # The file must be part of this PR to make the comment function
+        # work. Thus we change it's content to provide some more text.
+        files_dict = {
+            "bigfile": textwrap.dedent(
+                """
+                section one
+                ===========
+
+                here is some text
+                """
+            ),
+        }
+
+        A = self.fake_github.openFakePullRequest(
+            project, "master", "A", files=files_dict
+        )
+        self.fake_github.emitEvent(A.getPullRequestOpenedEvent())
+        self.waitUntilSettled()
+
+        # We should have a pending check for the head sha
+        self.assertIn(
+            A.head_sha, github.repo_from_project(project)._commits.keys())
+        check_runs = self.fake_github.getCommitChecks(project, A.head_sha)
+
+        self.assertEqual(1, len(check_runs))
+        check_run = check_runs[0]
+
+        self.assertEqual("tenant-one/check", check_run["name"])
+        self.assertEqual("completed", check_run["status"])
+        self.assertThat(
+            check_run["output"]["summary"],
+            MatchesRegex(r'.*Build succeeded.*', re.DOTALL)
+        )
+
+        annotations = check_run["output"]["annotations"]
+        self.assertEqual(50, len(annotations))
+
+        # Comments are sorted by uniqueness, so our most unique
+        # comment is first.
+        self.assertEqual(annotations[0], {
+            "path": "bigfile",
+            "annotation_level": "warning",
+            "message": "Insightful comment",
+            "start_line": 2,
+            "end_line": 2,
+        })
+        # This comment appears 3 times.
+        self.assertEqual(annotations[1], {
+            "path": "bigfile",
+            "annotation_level": "warning",
+            "message": "Useful comment",
+            "start_line": 1,
+            "end_line": 1,
+        })
+        # The rest.
+        self.assertEqual(annotations[4], {
+            "path": "bigfile",
+            "annotation_level": "warning",
+            "message": "Annoying comment",
+            "start_line": 1,
+            "end_line": 1,
+        })
+
+
+class TestGithubDriverEnterise(ZuulGithubAppTestCase):
+    config_file = 'zuul-github-driver-enterprise.conf'
+
+    @simple_layout('layouts/merging-github.yaml', driver='github')
+    def test_report_pull_merge(self):
+        github = self.fake_github.getGithubClient()
+        repo = github.repo_from_project('org/project')
+        repo._set_branch_protection(
+            'master', require_review=True)
+
+        # pipeline merges the pull request on success
+        A = self.fake_github.openFakePullRequest('org/project', 'master',
+                                                 'PR title',
+                                                 body='I shouldnt be seen',
+                                                 body_text='PR body')
+
+        self.fake_github.emitEvent(A.getCommentAddedEvent('merge me'))
+        self.waitUntilSettled()
+
+        # Since the PR was not approved it should not be merged
+        self.assertFalse(A.is_merged)
+
+        A.addReview('derp', 'APPROVED')
+        self.fake_github.emitEvent(A.getCommentAddedEvent('merge me'))
+        self.waitUntilSettled()
+
+        # After approval it should be merged
+        self.assertTrue(A.is_merged)
+        self.assertThat(A.merge_message,
+                        MatchesRegex(r'.*PR title\n\nPR body.*', re.DOTALL))
+        self.assertThat(A.merge_message,
+                        Not(MatchesRegex(
+                            r'.*I shouldnt be seen.*',
+                            re.DOTALL)))
+        self.assertEqual(len(A.comments), 0)
+
+
+class TestGithubDriverEnteriseLegacy(ZuulGithubAppTestCase):
+    config_file = 'zuul-github-driver-enterprise.conf'
+
+    def setUp(self):
+        self.old_version = FakeGithubEnterpriseClient.version
+        FakeGithubEnterpriseClient.version = '2.19.0'
+
+        super().setUp()
+
+    def tearDown(self):
+        super().tearDown()
+        FakeGithubEnterpriseClient.version = self.old_version
+
+    @simple_layout('layouts/merging-github.yaml', driver='github')
+    def test_report_pull_merge(self):
+        github = self.fake_github.getGithubClient()
+        repo = github.repo_from_project('org/project')
+        repo._set_branch_protection(
+            'master', require_review=True)
+
+        # pipeline merges the pull request on success
+        A = self.fake_github.openFakePullRequest('org/project', 'master',
+                                                 'PR title',
+                                                 body='I shouldnt be seen',
+                                                 body_text='PR body')
+
+        self.fake_github.emitEvent(A.getCommentAddedEvent('merge me'))
+        self.waitUntilSettled()
+
+        # Note: PR was not approved but old github does not support
+        # reviewDecision so this gets ignored and zuul merges nevertheless
+        self.assertTrue(A.is_merged)
+        self.assertThat(A.merge_message,
+                        MatchesRegex(r'.*PR title\n\nPR body.*', re.DOTALL))
+        self.assertThat(A.merge_message,
+                        Not(MatchesRegex(
+                            r'.*I shouldnt be seen.*',
+                            re.DOTALL)))
+        self.assertEqual(len(A.comments), 0)
