@@ -15,11 +15,12 @@
 
 import time
 
-import zuul.zk
-import zuul.nodepool
 from zuul import model
+import zuul.nodepool
 
-from tests.base import BaseTestCase, ChrootedKazooFixture, FakeNodepool
+from tests.base import BaseTestCase, FakeNodepool
+from zuul.zk import ZooKeeperClient
+from zuul.zk.nodepool import ZooKeeperNodepool
 
 
 class TestNodepool(BaseTestCase):
@@ -30,27 +31,25 @@ class TestNodepool(BaseTestCase):
         super(TestNodepool, self).setUp()
 
         self.statsd = None
-        self.zk_chroot_fixture = self.useFixture(
-            ChrootedKazooFixture(self.id()))
-        self.zk_config = '%s:%s%s' % (
-            self.zk_chroot_fixture.zookeeper_host,
-            self.zk_chroot_fixture.zookeeper_port,
-            self.zk_chroot_fixture.zookeeper_chroot)
+        self.setupZK()
 
-        self.zk = zuul.zk.ZooKeeper(enable_cache=True)
-        self.addCleanup(self.zk.disconnect)
-        self.zk.connect(self.zk_config)
+        self.zk_client = ZooKeeperClient(
+            self.zk_chroot_fixture.zk_hosts,
+            tls_cert=self.zk_chroot_fixture.zookeeper_cert,
+            tls_key=self.zk_chroot_fixture.zookeeper_key,
+            tls_ca=self.zk_chroot_fixture.zookeeper_ca)
+        self.zk_nodepool = ZooKeeperNodepool(self.zk_client)
+        self.addCleanup(self.zk_client.disconnect)
+        self.zk_client.connect()
         self.hostname = 'nodepool-test-hostname'
 
         self.provisioned_requests = []
         # This class implements the scheduler methods zuul.nodepool
         # needs, so we pass 'self' as the scheduler.
-        self.nodepool = zuul.nodepool.Nodepool(self)
+        self.nodepool = zuul.nodepool.Nodepool(
+            self.zk_client, self.hostname, self.statsd, self)
 
-        self.fake_nodepool = FakeNodepool(
-            self.zk_chroot_fixture.zookeeper_host,
-            self.zk_chroot_fixture.zookeeper_port,
-            self.zk_chroot_fixture.zookeeper_chroot)
+        self.fake_nodepool = FakeNodepool(self.zk_chroot_fixture)
         self.addCleanup(self.fake_nodepool.stop)
 
     def waitForRequests(self):
@@ -105,8 +104,8 @@ class TestNodepool(BaseTestCase):
         job.nodeset = nodeset
         self.fake_nodepool.pause()
         request = self.nodepool.requestNodes(None, job, 0)
-        self.zk.client.stop()
-        self.zk.client.start()
+        self.zk_client.client.stop()
+        self.zk_client.client.start()
         self.fake_nodepool.unpause()
         self.waitForRequests()
         self.assertEqual(len(self.provisioned_requests), 1)
@@ -161,7 +160,7 @@ class TestNodepool(BaseTestCase):
         self.assertEqual(len(self.provisioned_requests), 1)
         self.assertEqual(request.state, 'fulfilled')
 
-        self.zk.deleteNodeRequest(request)
+        self.zk_nodepool.deleteNodeRequest(request)
 
         # Accept the nodes
         self.nodepool.acceptNodes(request, request.id)
