@@ -284,6 +284,28 @@ class TestPagureDriver(ZuulTestCase):
             ], ordered=False
         )
 
+    @simple_layout('layouts/files-pagure.yaml', driver='pagure')
+    def test_pull_matched_file_event(self):
+        A = self.fake_pagure.openFakePullRequest(
+            'org/project', 'master', 'A',
+            files={'random.txt': 'test', 'build-requires': 'test'})
+        self.fake_pagure.emitEvent(A.getPullRequestOpenedEvent())
+        self.waitUntilSettled()
+        self.assertEqual(1, len(self.history))
+
+        B = self.fake_pagure.openFakePullRequest('org/project', 'master', 'B',
+                                                 files={'random.txt': 'test2'})
+        self.fake_pagure.emitEvent(B.getPullRequestOpenedEvent())
+        self.waitUntilSettled()
+        self.assertEqual(1, len(self.history))
+
+        C = self.fake_pagure.openFakePullRequest(
+            'org/project', 'master', 'C',
+            files={'build-requires': 'test'})
+        self.fake_pagure.emitEvent(C.getPullRequestOpenedEvent())
+        self.waitUntilSettled()
+        self.assertEqual(2, len(self.history))
+
     @simple_layout('layouts/basic-pagure.yaml', driver='pagure')
     def test_tag_created(self):
 
@@ -732,7 +754,7 @@ class TestPagureToGerritCRD(ZuulTestCase):
         self.assertFalse(A.is_merged)
         self.assertEqual(B.data['status'], 'NEW')
 
-        for connection in self.connections.connections.values():
+        for connection in self.scheds.first.connections.connections.values():
             connection.maintainCache([])
 
         B.addApproval('Approved', 1)
@@ -817,7 +839,7 @@ class TestGerritToPagureCRD(ZuulTestCase):
         self.assertEqual(A.data['status'], 'NEW')
         self.assertFalse(B.is_merged)
 
-        for connection in self.connections.connections.values():
+        for connection in self.scheds.first.connections.connections.values():
             connection.maintainCache([])
 
         B.addFlag('success', 'https://url', 'Build passed')
@@ -894,7 +916,7 @@ class TestPagureToGithubCRD(ZuulTestCase):
         self.assertFalse(A.is_merged)
         self.assertFalse(B.is_merged)
 
-        for connection in self.connections.connections.values():
+        for connection in self.scheds.first.connections.connections.values():
             connection.maintainCache([])
 
         B.addLabel('approved')
@@ -963,7 +985,7 @@ class TestGithubToPagureCRD(ZuulTestCase):
         self.assertFalse(A.is_merged)
         self.assertFalse(B.is_merged)
 
-        for connection in self.connections.connections.values():
+        for connection in self.scheds.first.connections.connections.values():
             connection.maintainCache([])
 
         B.addFlag('success', 'https://url', 'Build passed')
@@ -1022,8 +1044,11 @@ class TestPagureWebhook(ZuulTestCase):
         super(TestPagureWebhook, self).setUp()
         # Start the web server
         self.web = self.useFixture(
-            ZuulWebFixture(self.gearman_server.port,
-                           self.config, self.test_root))
+            ZuulWebFixture(self.changes, self.config,
+                           self.additional_event_queues, self.upstream_root,
+                           self.rpcclient, self.poller_events,
+                           self.git_url_with_auth, self.addCleanup,
+                           self.test_root))
 
         host = '127.0.0.1'
         # Wait until web server is started
@@ -1067,8 +1092,11 @@ class TestPagureWebhookWhitelist(ZuulTestCase):
         super(TestPagureWebhookWhitelist, self).setUp()
         # Start the web server
         self.web = self.useFixture(
-            ZuulWebFixture(self.gearman_server.port,
-                           self.config, self.test_root))
+            ZuulWebFixture(self.changes, self.config,
+                           self.additional_event_queues, self.upstream_root,
+                           self.rpcclient, self.poller_events,
+                           self.git_url_with_auth, self.addCleanup,
+                           self.test_root))
 
         host = '127.0.0.1'
         # Wait until web server is started
