@@ -26,7 +26,7 @@ import time
 
 import zuul.web
 import zuul.lib.log_streamer
-import zuul.lib.fingergw
+from zuul.lib.fingergw import FingerGateway
 import tests.base
 from tests.base import iterate_timeout, ZuulWebFixture
 
@@ -250,8 +250,11 @@ class TestStreaming(tests.base.AnsibleZuulTestCase):
         '''
         # Start the web server
         web = self.useFixture(
-            ZuulWebFixture(self.gearman_server.port,
-                           self.config, self.test_root))
+            ZuulWebFixture(self.changes, self.config,
+                           self.additional_event_queues, self.upstream_root,
+                           self.rpcclient, self.poller_events,
+                           self.git_url_with_auth, self.addCleanup,
+                           self.test_root))
 
         # Start the finger streamer daemon
         streamer = zuul.lib.log_streamer.LogStreamer(
@@ -326,8 +329,11 @@ class TestStreaming(tests.base.AnsibleZuulTestCase):
     def test_websocket_streaming(self):
         # Start the web server
         web = self.useFixture(
-            ZuulWebFixture(self.gearman_server.port,
-                           self.config, self.test_root))
+            ZuulWebFixture(self.changes, self.config,
+                           self.additional_event_queues, self.upstream_root,
+                           self.rpcclient, self.poller_events,
+                           self.git_url_with_auth, self.addCleanup,
+                           self.test_root))
 
         # Start the finger streamer daemon
         streamer = zuul.lib.log_streamer.LogStreamer(
@@ -399,8 +405,11 @@ class TestStreaming(tests.base.AnsibleZuulTestCase):
     def test_websocket_hangup(self):
         # Start the web server
         web = self.useFixture(
-            ZuulWebFixture(self.gearman_server.port,
-                           self.config, self.test_root))
+            ZuulWebFixture(self.changes, self.config,
+                           self.additional_event_queues, self.upstream_root,
+                           self.rpcclient, self.poller_events,
+                           self.git_url_with_auth, self.addCleanup,
+                           self.test_root))
 
         # Start the finger streamer daemon
         streamer = zuul.lib.log_streamer.LogStreamer(
@@ -513,7 +522,8 @@ class TestStreaming(tests.base.AnsibleZuulTestCase):
         self.addCleanup(logfile.close)
 
         # Start the finger gateway daemon
-        gateway = zuul.lib.fingergw.FingerGateway(
+        gateway = FingerGateway(
+            self.config,
             ('127.0.0.1', self.gearman_server.port, None, None, None),
             (self.host, 0),
             user=None,
