@@ -27,8 +27,11 @@ class TestWebURLs(ZuulTestCase):
     def setUp(self):
         super(TestWebURLs, self).setUp()
         self.web = self.useFixture(
-            ZuulWebFixture(self.gearman_server.port,
-                           self.config, self.test_root))
+            ZuulWebFixture(self.changes, self.config,
+                           self.additional_event_queues, self.upstream_root,
+                           self.rpcclient, self.poller_events,
+                           self.git_url_with_auth, self.addCleanup,
+                           self.test_root))
 
     def _get(self, port, uri):
         url = "http://localhost:{}{}".format(port, uri)
