@@ -421,15 +421,15 @@ def zuul_run_command(self, args, zuul_log_id, check_rc=False, close_fds=True, ex
         #
         # # ZUUL: stderr follows stdout
         # rpipes = [cmd.stdout]
-        #
-        # if data:
-        #     if not binary_data:
-        #         data += '\n'
-        #     if isinstance(data, text_type):
-        #         data = to_bytes(data)
-        #     cmd.stdin.write(data)
-        #     cmd.stdin.close()
-        #
+
+        if data:
+            if not binary_data:
+                data += '\n'
+            if isinstance(data, text_type):
+                data = to_bytes(data)
+            cmd.stdin.write(data)
+            cmd.stdin.close()
+
         # while True:
         #     rfds, wfds, efds = select.select(rpipes, [], rpipes, 1)
         #     stdout += self._read_from_pipes(rpipes, rfds, cmd.stdout)
@@ -474,7 +474,7 @@ def zuul_run_command(self, args, zuul_log_id, check_rc=False, close_fds=True, ex
         if t:
             t.join(10)
             with Console(zuul_log_id) as console:
-                if t.isAlive():
+                if t.is_alive():
                     console.addLine("[Zuul] standard output/error still open "
                                     "after child exited")
             # ZUUL: stdout and stderr are in the console log file
@@ -495,7 +495,7 @@ def zuul_run_command(self, args, zuul_log_id, check_rc=False, close_fds=True, ex
     finally:
         if t:
             with Console(zuul_log_id) as console:
-                if t.isAlive():
+                if t.is_alive():
                     console.addLine("[Zuul] standard output/error still open "
                                     "after child exited")
                 if fail_json_kwargs:
