@@ -28,6 +28,7 @@ import sys
 import traceback
 import threading
 
+prometheus_client = extras.try_import('prometheus_client')
 yappi = extras.try_import('yappi')
 objgraph = extras.try_import('objgraph')
 
@@ -120,10 +121,7 @@ class ZuulApp(object):
         parser = self.createParser()
         self.args = parser.parse_args(args)
 
-        # The arguments debug and foreground both lead to nodaemon mode so
-        # set nodaemon if one of them is set.
-        if ((hasattr(self.args, 'debug') and self.args.debug) or
-                (hasattr(self.args, 'foreground') and self.args.foreground)):
+        if hasattr(self.args, 'foreground') and self.args.foreground:
             self.args.nodaemon = True
         else:
             self.args.nodaemon = False
@@ -164,22 +162,20 @@ class ZuulApp(object):
                 logging_config.setDebug()
         logging_config.apply()
 
-    def configure_connections(self, source_only=False, include_drivers=None):
+    def configure_connections(self, source_only=False, include_drivers=None,
+                              require_sql=False):
         self.connections = zuul.lib.connections.ConnectionRegistry()
-        self.connections.configure(self.config, source_only, include_drivers)
+        self.connections.configure(self.config, source_only, include_drivers,
+                                   require_sql)
 
 
 class ZuulDaemonApp(ZuulApp, metaclass=abc.ABCMeta):
     def createParser(self):
         parser = super(ZuulDaemonApp, self).createParser()
         parser.add_argument('-d', dest='debug', action='store_true',
-                            help='run in foreground with debug log. Note '
-                                 'that in future this will be changed to only '
-                                 'request debug logging. If you want to keep '
-                                 'running the process in the foreground '
-                                 'migrate/add the -f switch.')
+                            help='enable debug log')
         parser.add_argument('-f', dest='foreground', action='store_true',
-                            help='run in foreground with info log')
+                            help='run in foreground')
         return parser
 
     def getPidFile(self):
@@ -204,6 +200,15 @@ class ZuulDaemonApp(ZuulApp, metaclass=abc.ABCMeta):
             "Configured logging: {version}".format(
                 version=zuul_version_info.release_string()))
 
+    def setup_prometheus(self, section):
+        if self.config.has_option(section, 'prometheus_port'):
+            if not prometheus_client:
+                raise RuntimeError("prometheus_client library is missing.")
+            port = int(self.config.get(section, 'prometheus_port'))
+            addr = get_default(
+                self.config, section, 'prometheus_addr', '0.0.0.0')
+            prometheus_client.start_http_server(port, addr)
+
     def main(self):
         self.parseArguments()
         self.readConfig()
