@@ -36,12 +36,15 @@ class ZuulRESTClient(object):
     """Basic client for Zuul's REST API"""
     def __init__(self, url, verify=False, auth_token=None):
         self.url = url
+        if not self.url.endswith('/'):
+            self.url += '/'
         self.auth_token = auth_token
-        self.base_url = urllib.parse.urljoin(self.url, '/api/')
         self.verify = verify
-        self.session = requests.Session(
-            verify=self.verify,
-            headers={'Authorization': 'Bearer %s' % self.auth_token})
+        self.base_url = urllib.parse.urljoin(self.url, 'api/')
+        self.session = requests.Session()
+        self.session.verify = self.verify
+        self.session.headers = dict(
+            Authorization='Bearer %s' % self.auth_token)
 
     def _check_status(self, req):
         try:
@@ -79,17 +82,7 @@ class ZuulRESTClient(object):
             'tenant/%s/autohold' % tenant)
         req = requests.get(url, verify=self.verify)
         self._check_status(req)
-        resp = req.json()
-        # reformat the answer to match RPC format
-        ret = {}
-        for d in resp:
-            key = ','.join([d['tenant'],
-                            d['project'],
-                            d['job'],
-                            d['ref_filter']])
-            ret[key] = (d['count'], d['reason'], d['node_hold_expiration'])
-
-        return ret
+        return req.json()
 
     def enqueue(self, tenant, pipeline, project, trigger, change):
         if not self.auth_token:
@@ -137,9 +130,19 @@ class ZuulRESTClient(object):
         self._check_status(req)
         return req.json()
 
-    def promote(self, *args, **kwargs):
-        raise NotImplementedError(
-            'This action is unsupported by the REST API')
+    def promote(self, tenant, pipeline, change_ids):
+        if not self.auth_token:
+            raise Exception('Auth Token required')
+        args = {
+            "pipeline": pipeline,
+            "changes": change_ids,
+        }
+        url = urllib.parse.urljoin(
+            self.base_url,
+            'tenant/%s/promote' % tenant)
+        req = self.session.post(url, json=args)
+        self._check_status(req)
+        return req.json()
 
     def get_running_jobs(self, *args, **kwargs):
         raise NotImplementedError(
@@ -215,12 +218,13 @@ class Client(zuul.cmd.ZuulApp):
         cmd_autohold_list = subparsers.add_parser(
             'autohold-list', help='list autohold requests')
         cmd_autohold_list.add_argument('--tenant', help='tenant name',
-                                       required=False)
+                                       required=True)
         cmd_autohold_list.set_defaults(func=self.autohold_list)
 
         cmd_enqueue = subparsers.add_parser('enqueue', help='enqueue a change')
         cmd_enqueue.add_argument('--tenant', help='tenant name',
                                  required=True)
+        # TODO(mhu) remove in a few releases
         cmd_enqueue.add_argument('--trigger',
                                  help='trigger name (deprecated and ignored. '
                                       'Kept only for backward compatibility)',
@@ -480,7 +484,7 @@ class Client(zuul.cmd.ZuulApp):
         table = prettytable.PrettyTable(
             field_names=[
                 'ID', 'Tenant', 'Project', 'Job', 'Ref Filter',
-                'Max Count', 'Reason'
+                'Current Count', 'Max Count', 'Reason'
             ])
 
         for request in autohold_requests:
@@ -490,6 +494,7 @@ class Client(zuul.cmd.ZuulApp):
                 request['project'],
                 request['job'],
                 request['ref_filter'],
+                request['current_count'],
                 request['max_count'],
                 request['reason'],
             ])
@@ -567,7 +572,7 @@ class Client(zuul.cmd.ZuulApp):
         try:
             auth_token = jwt.encode(token,
                                     key=key,
-                                    algorithm=driver).decode('utf-8')
+                                    algorithm=driver)
             print("Bearer %s" % auth_token)
             err_code = 0
         except Exception as e:
@@ -712,15 +717,15 @@ class Client(zuul.cmd.ZuulApp):
     def validate(self):
         from zuul import scheduler
         from zuul import configloader
-        sched = scheduler.Scheduler(self.config, testonly=True)
         self.configure_connections(source_only=True)
-        sched.registerConnections(self.connections, load=False)
+        sched = scheduler.Scheduler(self.config, self.connections,
+                                    self, testonly=True)
         loader = configloader.ConfigLoader(
             sched.connections, sched, None, None)
         tenant_config, script = sched._checkTenantSourceConf(self.config)
         unparsed_abide = loader.readConfig(tenant_config, from_script=script)
         try:
-            for conf_tenant in unparsed_abide.tenants:
+            for conf_tenant in unparsed_abide.tenants.values():
                 loader.tenant_parser.getSchema()(conf_tenant)
             print("Tenants config validated with success")
             err_code = 0
