@@ -45,8 +45,6 @@ class Executor(zuul.cmd.ZuulDaemonApp):
 
     def exit_handler(self, signum, frame):
         self.executor.stop()
-        self.executor.join()
-        sys.exit(0)
 
     def start_log_streamer(self):
         pipe_read, pipe_write = os.pipe()
@@ -89,6 +87,7 @@ class Executor(zuul.cmd.ZuulDaemonApp):
                 os.mkdir(self.job_dir)
 
         self.setup_logging('executor', 'log_config')
+        self.setup_prometheus('executor')
         self.log = logging.getLogger("zuul.Executor")
 
         self.finger_port = int(
@@ -99,7 +98,8 @@ class Executor(zuul.cmd.ZuulDaemonApp):
         self.start_log_streamer()
 
         ExecutorServer = zuul.executor.server.ExecutorServer
-        self.executor = ExecutorServer(self.config, self.connections,
+        self.executor = ExecutorServer(self.config,
+                                       self.connections,
                                        jobdir_root=self.job_dir,
                                        keep_jobdir=self.args.keep_jobdir,
                                        log_streaming_port=self.finger_port)
@@ -107,14 +107,8 @@ class Executor(zuul.cmd.ZuulDaemonApp):
 
         if self.args.nodaemon:
             signal.signal(signal.SIGTERM, self.exit_handler)
-            while True:
-                try:
-                    signal.pause()
-                except KeyboardInterrupt:
-                    print("Ctrl + C: asking executor to exit nicely...\n")
-                    self.exit_handler(signal.SIGINT, None)
-        else:
-            self.executor.join()
+
+        self.executor.join()
 
 
 def main():
