@@ -15,11 +15,11 @@
 import logging
 import signal
 import sys
+from typing import Optional
 
 import zuul.cmd
-import zuul.lib.fingergw
-
 from zuul.lib.config import get_default
+from zuul.lib.fingergw import COMMANDS, FingerGateway
 
 
 class FingerGatewayApp(zuul.cmd.ZuulDaemonApp):
@@ -32,12 +32,12 @@ class FingerGatewayApp(zuul.cmd.ZuulDaemonApp):
 
     def __init__(self):
         super(FingerGatewayApp, self).__init__()
-        self.gateway = None
+        self.gateway: Optional[FingerGateway] = None
 
     def createParser(self):
         parser = super(FingerGatewayApp, self).createParser()
         parser.add_argument('command',
-                            choices=zuul.lib.fingergw.COMMANDS,
+                            choices=COMMANDS,
                             nargs='?')
         return parser
 
@@ -52,7 +52,7 @@ class FingerGatewayApp(zuul.cmd.ZuulDaemonApp):
 
         Called by the main() method of the parent class.
         '''
-        if self.args.command in zuul.lib.fingergw.COMMANDS:
+        if self.args.command in COMMANDS:
             self.send_command(self.args.command)
             sys.exit(0)
 
@@ -72,7 +72,8 @@ class FingerGatewayApp(zuul.cmd.ZuulDaemonApp):
         ssl_cert = get_default(self.config, 'gearman', 'ssl_cert')
         ssl_ca = get_default(self.config, 'gearman', 'ssl_ca')
 
-        self.gateway = zuul.lib.fingergw.FingerGateway(
+        self.gateway = FingerGateway(
+            self.config,
             (gear_server, gear_port, ssl_key, ssl_cert, ssl_ca),
             (host, port),
             user,
