@@ -19,14 +19,9 @@ import sys
 import signal
 
 import zuul.cmd
-import zuul.executor.client
-import zuul.merger.client
-import zuul.nodepool
-import zuul.scheduler
-import zuul.zk
-
 from zuul.lib.config import get_default
 from zuul.lib.statsd import get_statsd_config
+import zuul.scheduler
 
 
 class Scheduler(zuul.cmd.ZuulDaemonApp):
@@ -39,6 +34,14 @@ class Scheduler(zuul.cmd.ZuulDaemonApp):
 
     def createParser(self):
         parser = super(Scheduler, self).createParser()
+        parser.add_argument('--validate-tenants', dest='validate_tenants',
+                            metavar='TENANT', nargs='*',
+                            help='Load configuration of the listed tenants and'
+                                 ' exit afterwards, indicating success or '
+                                 'failure via the exit code. If no tenant is '
+                                 'listed, all tenants will be validated. '
+                                 'Note: this requires the gearman server and '
+                                 'will distribute work to mergers.')
         parser.add_argument('command',
                             choices=zuul.scheduler.COMMANDS,
                             nargs='?')
@@ -67,13 +70,8 @@ class Scheduler(zuul.cmd.ZuulDaemonApp):
         except Exception:
             self.log.exception("Reconfiguration failed:")
 
-    def reconfigure_handler(self, signum, frame):
-        signal.signal(signal.SIGHUP, signal.SIG_IGN)
-        self.fullReconfigure()
-        signal.signal(signal.SIGHUP, self.reconfigure_handler)
-
     def exit_handler(self, signum, frame):
-        self.sched.exit()
+        self.sched.stop()
         self.sched.join()
         self.stop_gear_server()
         sys.exit(0)
@@ -135,43 +133,19 @@ class Scheduler(zuul.cmd.ZuulDaemonApp):
             self.start_gear_server()
 
         self.setup_logging('scheduler', 'log_config')
+        self.setup_prometheus('scheduler')
         self.log = logging.getLogger("zuul.Scheduler")
 
-        self.sched = zuul.scheduler.Scheduler(self.config)
-
-        gearman = zuul.executor.client.ExecutorClient(self.config, self.sched)
-        self.sched.setZuulApp(self)
-        merger = zuul.merger.client.MergeClient(self.config, self.sched)
-        nodepool = zuul.nodepool.Nodepool(self.sched)
-
-        zookeeper = zuul.zk.ZooKeeper(enable_cache=True)
-        zookeeper_hosts = get_default(self.config, 'zookeeper', 'hosts', None)
-        if not zookeeper_hosts:
-            raise Exception("The zookeeper hosts config value is required")
-        zookeeper_tls_key = get_default(self.config, 'zookeeper', 'tls_key')
-        zookeeper_tls_cert = get_default(self.config, 'zookeeper', 'tls_cert')
-        zookeeper_tls_ca = get_default(self.config, 'zookeeper', 'tls_ca')
-        zookeeper_timeout = float(get_default(self.config, 'zookeeper',
-                                              'session_timeout', 10.0))
-        zookeeper.connect(
-            zookeeper_hosts,
-            timeout=zookeeper_timeout,
-            tls_cert=zookeeper_tls_cert,
-            tls_key=zookeeper_tls_key,
-            tls_ca=zookeeper_tls_ca)
-
-        self.configure_connections()
-        self.sched.setExecutor(gearman)
-        self.sched.setMerger(merger)
-        self.sched.setNodepool(nodepool)
-        self.sched.setZooKeeper(zookeeper)
+        self.configure_connections(require_sql=True)
+        self.sched = zuul.scheduler.Scheduler(self.config,
+                                              self.connections, self)
+        if self.args.validate_tenants is None:
+            self.connections.registerScheduler(self.sched)
 
         self.log.info('Starting scheduler')
         try:
             self.sched.start()
-            self.sched.registerConnections(self.connections)
-            self.sched.reconfigure(self.config)
-            self.sched.wakeUp()
+            self.sched.prime(self.config, self.args.validate_tenants)
         except Exception:
             self.log.exception("Error starting Zuul:")
             # TODO(jeblair): If we had all threads marked as daemon,
@@ -179,7 +153,9 @@ class Scheduler(zuul.cmd.ZuulDaemonApp):
             self.sched.stop()
             sys.exit(1)
 
-        signal.signal(signal.SIGHUP, self.reconfigure_handler)
+        if self.args.validate_tenants is not None:
+            self.sched.stop()
+            sys.exit(0)
 
         if self.args.nodaemon:
             signal.signal(signal.SIGTERM, self.exit_handler)
