@@ -23,8 +23,6 @@ import zuul.driver.sql
 import zuul.driver.github
 import zuul.lib.auth
 
-from zuul.lib.config import get_default
-
 
 class WebServer(zuul.cmd.ZuulDaemonApp):
     app_name = 'web'
@@ -48,31 +46,6 @@ class WebServer(zuul.cmd.ZuulDaemonApp):
     def _run(self):
         info = zuul.model.WebInfo.fromConfig(self.config)
 
-        params = dict()
-
-        params['info'] = info
-        params['listen_address'] = get_default(self.config,
-                                               'web', 'listen_address',
-                                               '127.0.0.1')
-        params['listen_port'] = get_default(self.config, 'web', 'port', 9000)
-        params['static_cache_expiry'] = get_default(self.config, 'web',
-                                                    'static_cache_expiry',
-                                                    3600)
-        params['static_path'] = get_default(self.config,
-                                            'web', 'static_path',
-                                            None)
-        params['gear_server'] = get_default(self.config, 'gearman', 'server')
-        params['gear_port'] = get_default(self.config, 'gearman', 'port', 4730)
-        params['ssl_key'] = get_default(self.config, 'gearman', 'ssl_key')
-        params['ssl_cert'] = get_default(self.config, 'gearman', 'ssl_cert')
-        params['ssl_ca'] = get_default(self.config, 'gearman', 'ssl_ca')
-
-        params['command_socket'] = get_default(
-            self.config, 'web', 'command_socket',
-            '/var/lib/zuul/web.socket')
-
-        params['connections'] = self.connections
-        params['authenticators'] = self.authenticators
         # Validate config here before we spin up the ZuulWeb object
         for conn_name, connection in self.connections.connections.items():
             try:
@@ -81,19 +54,13 @@ class WebServer(zuul.cmd.ZuulDaemonApp):
                 self.log.exception("Error validating config")
                 sys.exit(1)
 
-        params["zk_hosts"] = get_default(
-            self.config, 'zookeeper', 'hosts', None)
-        if not params["zk_hosts"]:
-            raise Exception("The zookeeper hosts config value is required")
-        params["zk_tls_key"] = get_default(self.config, 'zookeeper', 'tls_key')
-        params["zk_tls_cert"] = get_default(self.config,
-                                            'zookeeper', 'tls_cert')
-        params["zk_tls_ca"] = get_default(self.config, 'zookeeper', 'tls_ca')
-        params["zk_timeout"] = float(get_default(self.config, 'zookeeper',
-                                                 'session_timeout', 10.0))
-
         try:
-            self.web = zuul.web.ZuulWeb(**params)
+            self.web = zuul.web.ZuulWeb(
+                config=self.config,
+                info=info,
+                connections=self.connections,
+                authenticators=self.authenticators,
+            )
         except Exception:
             self.log.exception("Error creating ZuulWeb:")
             sys.exit(1)
@@ -117,6 +84,7 @@ class WebServer(zuul.cmd.ZuulDaemonApp):
             sys.exit(0)
 
         self.setup_logging('web', 'log_config')
+        self.setup_prometheus('web')
         self.log = logging.getLogger("zuul.WebServer")
 
         try:
@@ -124,7 +92,8 @@ class WebServer(zuul.cmd.ZuulDaemonApp):
                 include_drivers=[zuul.driver.sql.SQLDriver,
                                  zuul.driver.github.GithubDriver,
                                  zuul.driver.pagure.PagureDriver,
-                                 zuul.driver.gitlab.GitlabDriver])
+                                 zuul.driver.gitlab.GitlabDriver],
+                require_sql=True)
             self.configure_authenticators()
             self._run()
         except Exception:
