@@ -21,7 +21,6 @@ import math
 import logging
 import paramiko
 import pprint
-import queue
 import re
 import re2
 import requests
@@ -43,6 +42,7 @@ from zuul.driver.gerrit.gerritmodel import GerritChange, GerritTriggerEvent
 from zuul.driver.git.gitwatcher import GitWatcher
 from zuul.lib.logutil import get_annotated_logger
 from zuul.model import Ref, Tag, Branch, Project
+from zuul.zk.event_queues import ConnectionEventQueue, EventReceiverElection
 
 # HTTP timeout in seconds
 TIMEOUT = 30
@@ -119,6 +119,9 @@ class GerritEventConnector(threading.Thread):
 
     IGNORED_EVENTS = (
         'cache-eviction',  # evict-cache plugin
+        'ref-replicated',
+        'ref-replication-scheduled',
+        'ref-replication-done'
     )
 
     log = logging.getLogger("zuul.GerritEventConnector")
@@ -128,16 +131,45 @@ class GerritEventConnector(threading.Thread):
         super(GerritEventConnector, self).__init__()
         self.daemon = True
         self.connection = connection
+        self.event_queue = connection.event_queue
         self._stopped = False
+        self._connector_wake_event = threading.Event()
 
     def stop(self):
         self._stopped = True
-        self.connection.addEvent(None)
+        self._connector_wake_event.set()
+        self.event_queue.election.cancel()
+
+    def _onNewEvent(self):
+        self._connector_wake_event.set()
+        # Stop the data watch in case the connector was stopped
+        return not self._stopped
+
+    def run(self):
+        self.event_queue.registerEventWatch(self._onNewEvent)
+        while not self._stopped:
+            try:
+                self.event_queue.election.run(self._run)
+            except Exception:
+                self.log.exception("Exception moving Gerrit event:")
+                time.sleep(1)
+
+    def _run(self):
+        while not self._stopped:
+            for event in self.event_queue:
+                try:
+                    self._handleEvent(event)
+                finally:
+                    self.event_queue.ack(event)
+                if self._stopped:
+                    return
+            self._connector_wake_event.wait(10)
+            self._connector_wake_event.clear()
+
+    def _handleEvent(self, connection_event):
+        timestamp = connection_event["timestamp"]
+        data = connection_event["payload"]
 
-    def _handleEvent(self):
-        ts, data = self.connection.getEvent()
-        if self._stopped:
-            return
         # Gerrit can produce inconsistent data immediately after an
         # event, So ensure that we do not deliver the event to Zuul
         # until at least a certain amount of time has passed.  Note
@@ -145,9 +177,10 @@ class GerritEventConnector(threading.Thread):
         # only need to delay for the first event.  In essence, Zuul
         # should always be a constant number of seconds behind Gerrit.
         now = time.time()
-        time.sleep(max((ts + self.delay) - now, 0.0))
+        time.sleep(max((timestamp + self.delay) - now, 0.0))
         event = GerritTriggerEvent()
-        event.timestamp = ts
+        event.timestamp = timestamp
+        event.connection_name = self.connection.connection_name
 
         # Gerrit events don't have an event id that could be used to globally
         # identify this event in the system so we have to generate one.
@@ -186,6 +219,13 @@ class GerritEventConnector(threading.Thread):
                 event.ref = patchset.get('ref')
             event.approvals = data.get('approvals', [])
             event.comment = data.get('comment')
+            patchsetcomments = data.get('patchSetComments', {}).get(
+                "/PATCHSET_LEVEL")
+            if patchsetcomments:
+                event.patchsetcomments = []
+                for patchsetcomment in patchsetcomments:
+                    event.patchsetcomments.append(
+                        patchsetcomment.get('message'))
         refupdate = data.get('refUpdate')
         if refupdate:
             event.project_name = refupdate.get('project')
@@ -210,10 +250,8 @@ class GerritEventConnector(threading.Thread):
             'comment-added': 'author',
             'ref-updated': 'submitter',
             'reviewer-added': 'reviewer',  # Gerrit 2.5/2.6
-            'ref-replicated': None,
-            'ref-replication-done': None,
-            'ref-replication-scheduled': None,
             'topic-changed': 'changer',
+            'vote-deleted': 'deleter',
             'project-created': None,  # Gerrit 2.14
             'pending-check': None,  # Gerrit 3.0+
         }
@@ -230,23 +268,30 @@ class GerritEventConnector(threading.Thread):
         # This checks whether the event created or deleted a branch so
         # that Zuul may know to perform a reconfiguration on the
         # project.
+        branch_refs = 'refs/heads/'
         if (event.type == 'ref-updated' and
             ((not event.ref.startswith('refs/')) or
-             event.ref.startswith('refs/heads'))):
+             event.ref.startswith(branch_refs))):
+
+            if event.ref.startswith(branch_refs):
+                event.branch = event.ref[len(branch_refs):]
+            else:
+                event.branch = event.ref
+
             if event.oldrev == '0' * 40:
                 event.branch_created = True
-                event.branch = event.ref
                 project = self.connection.source.getProject(event.project_name)
                 self.connection._clearBranchCache(project)
             if event.newrev == '0' * 40:
                 event.branch_deleted = True
-                event.branch = event.ref
                 project = self.connection.source.getProject(event.project_name)
                 self.connection._clearBranchCache(project)
 
         self._getChange(event)
         self.connection.logEvent(event)
-        self.connection.sched.addEvent(event)
+        self.connection.sched.addTriggerEvent(
+            self.connection.driver_name, event
+        )
 
     def _getChange(self, event):
         # Grab the change if we are managing the project or if it exists in the
@@ -280,17 +325,6 @@ class GerritEventConnector(threading.Thread):
                                            event.patch_number,
                                            refresh=True, event=event)
 
-    def run(self):
-        while True:
-            if self._stopped:
-                return
-            try:
-                self._handleEvent()
-            except Exception:
-                self.log.exception("Exception moving Gerrit event:")
-            finally:
-                self.connection.eventDone()
-
 
 class GerritWatcher(threading.Thread):
     log = logging.getLogger("gerrit.GerritWatcher")
@@ -304,6 +338,14 @@ class GerritWatcher(threading.Thread):
         self.hostname = hostname
         self.port = port
         self.gerrit_connection = gerrit_connection
+        self._stop_event = threading.Event()
+        self.watcher_election = EventReceiverElection(
+            gerrit_connection.sched.zk_client,
+            gerrit_connection.connection_name,
+            "watcher")
+        self._connection_lost_event = threading.Event()
+        gerrit_connection.sched.zk_client.on_connection_lost_listeners.append(
+            self._connection_lost_event.set)
         self.keepalive = keepalive
         self._stopped = False
 
@@ -321,8 +363,10 @@ class GerritWatcher(threading.Thread):
     def _listen(self, stdout, stderr):
         poll = select.poll()
         poll.register(stdout.channel)
-        while not self._stopped:
+        while not (self._stopped or self._connection_lost_event.is_set()):
             ret = poll.poll(self.poll_timeout)
+            if self._connection_lost_event.is_set():
+                return
             for (fd, event) in ret:
                 if fd == stdout.channel.fileno():
                     if event == select.POLLIN:
@@ -361,10 +405,6 @@ class GerritWatcher(threading.Thread):
 
             if ret and ret not in [-1, 130]:
                 raise Exception("Gerrit error executing stream-events")
-        except Exception:
-            self.log.exception("Exception on ssh event stream with %s:",
-                               self.gerrit_connection.connection_name)
-            time.sleep(5)
         finally:
             # If we don't close on exceptions to connect we can leak the
             # connection and DoS Gerrit.
@@ -372,11 +412,19 @@ class GerritWatcher(threading.Thread):
 
     def run(self):
         while not self._stopped:
-            self._run()
+            try:
+                self.watcher_election.run(self._run)
+            except Exception:
+                self.log.exception("Exception on ssh event stream with %s:",
+                                   self.gerrit_connection.connection_name)
+                self._stop_event.wait(5)
+            self._connection_lost_event.clear()
 
     def stop(self):
         self.log.debug("Stopping watcher")
         self._stopped = True
+        self._stop_event.set()
+        self.watcher_election.cancel()
 
 
 class GerritPoller(threading.Thread):
@@ -388,8 +436,13 @@ class GerritPoller(threading.Thread):
         threading.Thread.__init__(self)
         self.connection = connection
         self.last_merged_poll = 0
+        self.poller_election = EventReceiverElection(
+            connection.sched.zk_client, connection.connection_name, "poller")
         self._stopped = False
         self._stop_event = threading.Event()
+        self._connection_lost_event = threading.Event()
+        connection.sched.zk_client.on_connection_lost_listeners.append(
+            self._connection_lost_event.set)
 
     def _makePendingCheckEvent(self, change, uuid, check):
         return {'type': 'pending-check',
@@ -418,56 +471,61 @@ class GerritPoller(threading.Thread):
                 }}
 
     def _poll_checkers(self):
-        try:
-            for checker in self.connection.watched_checkers:
-                changes = self.connection.get(
-                    'plugins/checks/checks.pending/?'
-                    'query=checker:%s+(state:NOT_STARTED)' % checker)
-                for change in changes:
-                    for uuid, check in change['pending_checks'].items():
-                        event = self._makePendingCheckEvent(
-                            change, uuid, check)
-                        self.connection.addEvent(event)
-        except Exception:
-            self.log.exception("Exception on Gerrit poll with %s:",
-                               self.connection.connection_name)
+        for checker in self.connection.watched_checkers:
+            changes = self.connection.get(
+                'plugins/checks/checks.pending/?'
+                'query=checker:%s+(state:NOT_STARTED)' % checker)
+            for change in changes:
+                for uuid, check in change['pending_checks'].items():
+                    event = self._makePendingCheckEvent(
+                        change, uuid, check)
+                    self.connection.addEvent(event)
 
     def _poll_merged_changes(self):
-        try:
-            now = datetime.datetime.utcnow()
-            age = self.last_merged_poll
-            if age:
-                # Allow an extra 4 seconds for request time
-                age = int(math.ceil((now - age).total_seconds())) + 4
-            changes = self.connection.simpleQueryHTTP(
-                "status:merged -age:%ss" % (age,))
-            self.last_merged_poll = now
-            for change in changes:
-                event = self._makeChangeMergedEvent(change)
-                self.connection.addEvent(event)
-        except Exception:
-            self.log.exception("Exception on Gerrit poll with %s:",
-                               self.connection.connection_name)
+        now = datetime.datetime.utcnow()
+        age = self.last_merged_poll
+        if age:
+            # Allow an extra 4 seconds for request time
+            age = int(math.ceil((now - age).total_seconds())) + 4
+        changes = self.connection.simpleQueryHTTP(
+            "status:merged -age:%ss" % (age,))
+        self.last_merged_poll = now
+        for change in changes:
+            event = self._makeChangeMergedEvent(change)
+            self.connection.addEvent(event)
 
-    def _run(self):
+    def _poll(self):
+        next_start = self._last_start + self.poll_interval
+        self._stop_event.wait(max(next_start - time.time(), 0))
+        if self._stopped or self._connection_lost_event.is_set():
+            return
+        self._last_start = time.time()
         self._poll_checkers()
         if not self.connection.enable_stream_events:
             self._poll_merged_changes()
 
+    def _run(self):
+        self._last_start = time.time()
+        while not (self._stopped or self._connection_lost_event.is_set()):
+            # during tests, a sub-class _poll method is used to send
+            # notifications
+            self._poll()
+
     def run(self):
-        last_start = time.time()
         while not self._stopped:
-            next_start = last_start + self.poll_interval
-            self._stop_event.wait(max(next_start - time.time(), 0))
-            if self._stopped:
-                break
-            last_start = time.time()
-            self._run()
+            try:
+                self.poller_election.run(self._run)
+            except Exception:
+                self.log.exception("Exception on Gerrit poll with %s:",
+                                   self.connection.connection_name)
+                time.sleep(1)
+            self._connection_lost_event.clear()
 
     def stop(self):
         self.log.debug("Stopping watcher")
         self._stopped = True
         self._stop_event.set()
+        self.poller_election.cancel()
 
 
 class GerritConnection(BaseConnection):
@@ -513,7 +571,6 @@ class GerritConnection(BaseConnection):
         self.watcher_thread = None
         self.poller_thread = None
         self.ref_watcher_thread = None
-        self.event_queue = queue.Queue()
         self.client = None
         self.watched_checkers = []
         self.project_checker_map = {}
@@ -664,7 +721,7 @@ class GerritConnection(BaseConnection):
     def addProject(self, project: Project) -> None:
         self.projects[project.name] = project
 
-    def clearBranchCache(self):
+    def clearCache(self):
         self._project_branch_cache = {}
 
     def _clearBranchCache(self, project):
@@ -867,33 +924,42 @@ class GerritConnection(BaseConnection):
         needed_by_changes = set()
         git_needed_by_changes = []
         for (dep_num, dep_ps) in data.needed_by:
-            log.debug("Updating %s: Getting git-needed change %s,%s",
-                      change, dep_num, dep_ps)
-            dep = self._getChange(dep_num, dep_ps, history=history,
-                                  event=event)
-            if (dep.open and dep.is_current_patchset and
-                dep not in needed_by_changes):
-                git_needed_by_changes.append(dep)
-                needed_by_changes.add(dep)
+            try:
+                log.debug("Updating %s: Getting git-needed change %s,%s",
+                          change, dep_num, dep_ps)
+                dep = self._getChange(dep_num, dep_ps, history=history,
+                                      event=event)
+                if (dep.open and dep.is_current_patchset and
+                    dep not in needed_by_changes):
+                    git_needed_by_changes.append(dep)
+                    needed_by_changes.add(dep)
+            except Exception:
+                log.exception("Failed to get git-needed change %s,%s",
+                              dep_num, dep_ps)
         change.git_needed_by_changes = git_needed_by_changes
 
         compat_needed_by_changes = []
         for (dep_num, dep_ps) in self._getNeededByFromCommit(
                 change.id, change, event):
-            log.debug("Updating %s: Getting commit-needed change %s,%s",
-                      change, dep_num, dep_ps)
-            # Because a commit needed-by may be a cross-repo
-            # dependency, cause that change to refresh so that it will
-            # reference the latest patchset of its Depends-On (this
-            # change). In case the dep is already in history we already
-            # refreshed this change so refresh is not needed in this case.
-            refresh = (dep_num, dep_ps) not in history
-            dep = self._getChange(
-                dep_num, dep_ps, refresh=refresh, history=history, event=event)
-            if (dep.open and dep.is_current_patchset
-                and dep not in needed_by_changes):
-                compat_needed_by_changes.append(dep)
-                needed_by_changes.add(dep)
+            try:
+                log.debug("Updating %s: Getting commit-needed change %s,%s",
+                          change, dep_num, dep_ps)
+                # Because a commit needed-by may be a cross-repo
+                # dependency, cause that change to refresh so that it will
+                # reference the latest patchset of its Depends-On (this
+                # change). In case the dep is already in history we already
+                # refreshed this change so refresh is not needed in this case.
+                refresh = (dep_num, dep_ps) not in history
+                dep = self._getChange(
+                    dep_num, dep_ps, refresh=refresh, history=history,
+                    event=event)
+                if (dep.open and dep.is_current_patchset
+                    and dep not in needed_by_changes):
+                    compat_needed_by_changes.append(dep)
+                    needed_by_changes.add(dep)
+            except Exception:
+                log.exception("Failed to get commit-needed change %s,%s",
+                              dep_num, dep_ps)
         change.compat_needed_by_changes = compat_needed_by_changes
 
     def isMerged(self, change, head=None):
@@ -917,6 +983,14 @@ class GerritConnection(BaseConnection):
 
         ref = 'refs/heads/' + change.branch
         self.log.debug("Waiting for %s to appear in git repo" % (change))
+        if not hasattr(change, '_ref_sha'):
+            self.log.error("Unable to confirm change %s in git repo: "
+                           "the change has not been reported; "
+                           "this pipeline may be misconfigured "
+                           "(check for multiple Gerrit connections)." %
+                           (change,))
+            return False
+
         if self._waitForRefSha(change.project, ref, change._ref_sha):
             self.log.debug("Change %s is in the git repo" %
                            (change))
@@ -953,6 +1027,8 @@ class GerritConnection(BaseConnection):
             # Good question.  It's probably ref-updated, which, ah,
             # means it's merged.
             return True
+        if change.wip:
+            return False
         if change.missing_labels <= set(allow_needs):
             return True
         return False
@@ -998,13 +1074,11 @@ class GerritConnection(BaseConnection):
         return heads
 
     def addEvent(self, data):
-        return self.event_queue.put((time.time(), data))
-
-    def getEvent(self):
-        return self.event_queue.get()
-
-    def eventDone(self):
-        self.event_queue.task_done()
+        event = {
+            "timestamp": time.time(),
+            "payload": data
+        }
+        self.event_queue.put(event)
 
     def review(self, item, message, submit, labels, checks_api,
                file_comments, zuul_event_id=None):
@@ -1034,6 +1108,8 @@ class GerritConnection(BaseConnection):
                 cmd += ' --%s' % key
             else:
                 cmd += ' --label %s=%s' % (key, val)
+        if self.version >= (2, 13, 0):
+            cmd += ' --tag autogenerated:zuul:%s' % (item.pipeline.name)
         changeid = '%s,%s' % (change.number, change.patchset)
         cmd += ' %s' % changeid
         out, err = self._ssh(cmd, zuul_event_id=zuul_event_id)
@@ -1063,9 +1139,9 @@ class GerritConnection(BaseConnection):
             checkinfo['started'] = fmt(item.enqueue_time)
             if item.report_time:
                 checkinfo['finished'] = fmt(item.report_time)
-                url = item.formatStatusUrl()
-                if url:
-                    checkinfo['url'] = url
+            url = item.formatStatusUrl()
+            if url:
+                checkinfo['url'] = url
         if checkinfo:
             for x in range(1, 4):
                 try:
@@ -1104,7 +1180,8 @@ class GerritConnection(BaseConnection):
                     data['robot_comments'] = file_comments
                 else:
                     data['comments'] = file_comments
-        data['tag'] = 'autogenerated:zuul:%s' % (item.pipeline.name)
+        if self.version >= (2, 13, 0):
+            data['tag'] = 'autogenerated:zuul:%s' % (item.pipeline.name)
         changeid = "%s~%s~%s" % (
             urllib.parse.quote(str(change.project), safe=''),
             urllib.parse.quote(str(change.branch), safe=''),
@@ -1143,7 +1220,7 @@ class GerritConnection(BaseConnection):
         args = '--all-approvals --comments --commit-message'
         args += ' --current-patch-set --dependencies --files'
         args += ' --patch-sets --submit-records'
-        cmd = 'gerrit query --format json %s %s' % (args, number)
+        cmd = 'gerrit query --format json %s change:%s' % (args, number)
         out, err = self._ssh(cmd)
         if not out:
             return False
@@ -1432,6 +1509,10 @@ class GerritConnection(BaseConnection):
         except Exception:
             self.log.exception("Unable to determine remote Gerrit version")
 
+        self.log.info("Creating Zookeeper event queue")
+        self.event_queue = ConnectionEventQueue(self.sched.zk_client,
+                                                self.connection_name)
+
         if self.enable_stream_events:
             self._start_watcher_thread()
         else:
@@ -1446,6 +1527,9 @@ class GerritConnection(BaseConnection):
         self._stop_ref_watcher_thread()
         self._stop_event_connector()
 
+    def getEventQueue(self):
+        return getattr(self, "event_queue", None)
+
     def _stop_watcher_thread(self):
         if self.watcher_thread:
             self.watcher_thread.stop()
@@ -1467,8 +1551,14 @@ class GerritConnection(BaseConnection):
             self.poller_thread.join()
 
     def _start_poller_thread(self):
-        self.poller_thread = self._poller_class(self)
-        self.poller_thread.start()
+        if self.session is not None:
+            self.poller_thread = self._poller_class(self)
+            self.poller_thread.start()
+        else:
+            self.log.info(
+                "%s: Gerrit Poller is disabled because no "
+                "HTTP authentication is defined",
+                self.connection_name)
 
     def _stop_ref_watcher_thread(self):
         if self.ref_watcher_thread:
@@ -1480,7 +1570,8 @@ class GerritConnection(BaseConnection):
             self,
             self.baseurl,
             self.ref_watcher_poll_interval,
-            self.refWatcherCallback)
+            self.refWatcherCallback,
+            election_name="ref-watcher")
         self.ref_watcher_thread.start()
 
     def _stop_event_connector(self):
