@@ -12,14 +12,18 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
-from zuul.driver import Driver, ConnectionInterface, SourceInterface
+from zuul.driver import (
+    Driver, ConnectionInterface, SourceInterface, TriggerInterface
+)
 from zuul.driver.git import gitconnection
+from zuul.driver.git import gitmodel
 from zuul.driver.git import gitsource
 from zuul.driver.git import gittrigger
 
 
-class GitDriver(Driver, ConnectionInterface, SourceInterface):
-    name = 'git'
+class GitDriver(Driver, ConnectionInterface, SourceInterface,
+                TriggerInterface):
+    name = "git"
 
     def getConnection(self, name, config):
         return gitconnection.GitConnection(self, name, config)
@@ -33,6 +37,9 @@ class GitDriver(Driver, ConnectionInterface, SourceInterface):
     def getTriggerSchema(self):
         return gittrigger.getSchema()
 
+    def getTriggerEventClass(self):
+        return gitmodel.GitTriggerEvent
+
     def getRequireSchema(self):
         return {}
 
