@@ -32,13 +32,13 @@ class GitSource(BaseSource):
     def isMerged(self, change, head=None):
         raise NotImplementedError()
 
-    def canMerge(self, change, allow_needs, event=None):
+    def canMerge(self, change, allow_needs, event=None, allow_refresh=False):
         raise NotImplementedError()
 
     def getChange(self, event, refresh=False):
         return self.connection.getChange(event, refresh)
 
-    def getChangeByURL(self, url):
+    def getChangeByURL(self, url, event):
         return None
 
     def getChangesDependingOn(self, change, projects, tenant):
