@@ -12,6 +12,7 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
+import json
 import logging
 import voluptuous as v
 import time
@@ -128,12 +129,7 @@ class GithubReporter(BaseReporter):
             sha = item.change.newrev
         state = self._commit_status
 
-        url_pattern = self.config.get('status-url')
-        if not url_pattern:
-            sched_config = self.connection.sched.config
-            if sched_config.has_option('web', 'status_url'):
-                url_pattern = sched_config.get('web', 'status_url')
-        url = item.formatUrlPattern(url_pattern) if url_pattern else ''
+        url = item.formatStatusUrl()
 
         description = '%s status: %s' % (item.pipeline.name,
                                          self._commit_status)
@@ -234,7 +230,23 @@ class GithubReporter(BaseReporter):
         # Check for inline comments that can be reported via checks API
         file_comments = self.getFileComments(item)
 
-        return self.connection.updateCheck(
+        # Github allows an external id to be added to a check run. We can use
+        # this to identify the check run in any custom actions we define.
+        # To uniquely identify the corresponding buildset in zuul, we need
+        # tenant, pipeline and change. The buildset's uuid cannot be used
+        # safely, as it might change e.g. during a gate reset. Fore more
+        # information, please see Jim's comment on
+        # https://review.opendev.org/#/c/666258/7
+        external_id = json.dumps(
+            {
+                "tenant": item.pipeline.tenant.name,
+                "pipeline": item.pipeline.name,
+                "change": item.change.number,
+            }
+        )
+
+        state = item.dynamic_state[self.connection.connection_name]
+        check_run_id, errors = self.connection.updateCheck(
             project,
             pr_number,
             sha,
@@ -244,9 +256,16 @@ class GithubReporter(BaseReporter):
             details_url,
             message,
             file_comments,
+            external_id,
             zuul_event_id=item.event,
+            check_run_id=state.get('check_run_id')
         )
 
+        if check_run_id:
+            state['check_run_id'] = check_run_id
+
+        return errors
+
     def setLabels(self, item):
         log = get_annotated_logger(self.log, item.event)
         project = item.change.project.name
@@ -265,30 +284,23 @@ class GithubReporter(BaseReporter):
                                         zuul_event_id=item.event)
 
     def _formatMergeMessage(self, change):
-        message = ''
-
+        message = []
         if change.title:
-            message += change.title
+            message.append(change.title)
+        if change.body_text:
+            message.append(change.body_text)
+        merge_message = "\n\n".join(message)
 
-        account = getattr(change.source_event, 'account', None)
-        if not account:
-            return message
+        if change.reviews:
+            review_users = []
+            for r in change.reviews:
+                name = r['by']['name']
+                email = r['by']['email']
+                review_users.append('Reviewed-by: {} <{}>'.format(name, email))
+            merge_message += '\n\n'
+            merge_message += '\n'.join(review_users)
 
-        name = account['name']
-        email = account['email']
-        message += '\n\nReviewed-by: '
-
-        if name:
-            message += name
-        if email:
-            if name:
-                message += ' '
-            message += '<' + email + '>'
-        if name or email:
-            message += '\n             '
-        message += account['html_url']
-
-        return message
+        return merge_message
 
     def getSubmitAllowNeeds(self):
         """Get a list of code review labels that are allowed to be
