@@ -15,73 +15,24 @@
 import logging
 import threading
 import json
-import queue
 import cherrypy
 import voluptuous as v
 import time
 import uuid
 import requests
+
+import dateutil.parser
+
 from urllib.parse import quote_plus
-from datetime import datetime
+from typing import List, Optional
 
-from zuul.connection import BaseConnection
+from zuul.connection import CachedBranchConnection
 from zuul.web.handler import BaseWebController
-from zuul.lib.gearworker import ZuulGearWorker
 from zuul.lib.logutil import get_annotated_logger
-
+from zuul.exceptions import MergeFailure
+from zuul.model import Branch, Project, Ref, Tag
 from zuul.driver.gitlab.gitlabmodel import GitlabTriggerEvent, MergeRequest
-
-
-class GitlabGearmanWorker(object):
-    """A thread that answers gearman requests"""
-    log = logging.getLogger("zuul.GitlabGearmanWorker")
-
-    def __init__(self, connection):
-        self.config = connection.sched.config
-        self.connection = connection
-        handler = "gitlab:%s:payload" % self.connection.connection_name
-        self.jobs = {
-            handler: self.handle_payload,
-        }
-        self.gearworker = ZuulGearWorker(
-            'Zuul Gitlab Worker',
-            'zuul.GitlabGearmanWorker',
-            'gitlab',
-            self.config,
-            self.jobs)
-
-    def handle_payload(self, job):
-        args = json.loads(job.arguments)
-        payload = args["payload"]
-
-        self.log.info(
-            "Gitlab Webhook Received event kind: %(object_kind)s" % payload)
-
-        try:
-            self.__dispatch_event(payload)
-            output = {'return_code': 200}
-        except Exception:
-            output = {'return_code': 503}
-            self.log.exception("Exception handling Gitlab event:")
-
-        job.sendWorkComplete(json.dumps(output))
-
-    def __dispatch_event(self, payload):
-        self.log.info(payload)
-        event = payload['object_kind']
-        try:
-            self.log.info("Dispatching event %s" % event)
-            self.connection.addEvent(payload, event)
-        except Exception as err:
-            message = 'Exception dispatching event: %s' % str(err)
-            self.log.exception(message)
-            raise Exception(message)
-
-    def start(self):
-        self.gearworker.start()
-
-    def stop(self):
-        self.gearworker.stop()
+from zuul.zk.event_queues import ConnectionEventQueue
 
 
 class GitlabEventConnector(threading.Thread):
@@ -93,25 +44,56 @@ class GitlabEventConnector(threading.Thread):
         super(GitlabEventConnector, self).__init__()
         self.daemon = True
         self.connection = connection
+        self.event_queue = connection.event_queue
         self._stopped = False
+        self._process_event = threading.Event()
         self.event_handler_mapping = {
             'merge_request': self._event_merge_request,
             'note': self._event_note,
+            'push': self._event_push,
+            'tag_push': self._event_tag_push,
         }
 
     def stop(self):
         self._stopped = True
-        self.connection.addEvent(None)
+        self._process_event.set()
+        self.event_queue.election.cancel()
+
+    def _onNewEvent(self):
+        self._process_event.set()
+        # Stop the data watch in case the connector was stopped
+        return not self._stopped
+
+    def run(self):
+        self.event_queue.registerEventWatch(self._onNewEvent)
+        while not self._stopped:
+            try:
+                self.event_queue.election.run(self._run)
+            except Exception:
+                self.log.exception("Exception handling Gitlab event:")
+
+    def _run(self):
+        while not self._stopped:
+            for event in self.event_queue:
+                try:
+                    self._handleEvent(event)
+                finally:
+                    self.event_queue.ack(event)
+                if self._stopped:
+                    return
+            self._process_event.wait(10)
+            self._process_event.clear()
 
     def _event_base(self, body):
         event = GitlabTriggerEvent()
-        attrs = body['object_attributes']
-        event.updated_at = int(datetime.strptime(
-            attrs['updated_at'], '%Y-%m-%d %H:%M:%S %Z').strftime('%s'))
-        event.created_at = int(datetime.strptime(
-            attrs['created_at'], '%Y-%m-%d %H:%M:%S %Z').strftime('%s'))
+        event.connection_name = self.connection.connection_name
+        attrs = body.get('object_attributes')
+        if attrs:
+            event.updated_at = int(dateutil.parser.parse(
+                attrs['updated_at']).timestamp())
+            event.created_at = int(dateutil.parser.parse(
+                attrs['created_at']).timestamp())
         event.project_name = body['project']['path_with_namespace']
-        event.ref = "refs/merge-requests/%s/head" % event.change_number
         return event
 
     # https://docs.gitlab.com/ee/user/project/integrations/webhooks.html#merge-request-events
@@ -120,14 +102,32 @@ class GitlabEventConnector(threading.Thread):
         attrs = body['object_attributes']
         event.title = attrs['title']
         event.change_number = attrs['iid']
+        event.ref = "refs/merge-requests/%s/head" % event.change_number
         event.branch = attrs['target_branch']
         event.patch_number = attrs['last_commit']['id']
-        event.change_url = self.connection.getPullUrl(event.project_name,
-                                                      event.change_number)
-        if event.created_at == event.updated_at:
+        event.change_url = self.connection.getMRUrl(event.project_name,
+                                                    event.change_number)
+        if attrs['action'] == 'open':
             event.action = 'opened'
-        else:
+        elif attrs['action'] == 'merge':
+            event.action = 'merged'
+        elif attrs['action'] == 'update' and "labels" not in body["changes"]:
             event.action = 'changed'
+        elif attrs['action'] == 'update' and "labels" in body["changes"]:
+            event.action = 'labeled'
+            previous_labels = [
+                label["title"] for
+                label in body["changes"]["labels"]["previous"]]
+            current_labels = [
+                label["title"] for
+                label in body["changes"]["labels"]["current"]]
+            new_labels = set(current_labels) - set(previous_labels)
+            event.labels = list(new_labels)
+        elif attrs['action'] in ('approved', 'unapproved'):
+            event.action = attrs['action']
+        else:
+            # Do not handle other merge_request action for now.
+            return None
         event.type = 'gl_merge_request'
         return event
 
@@ -138,20 +138,49 @@ class GitlabEventConnector(threading.Thread):
         mr = body['merge_request']
         event.title = mr['title']
         event.change_number = mr['iid']
-        event.branch = mr['target_branch']
+        # mr['last_commit']['id'] is the commit SHA
         event.patch_number = mr['last_commit']['id']
-        event.change_url = self.connection.getPullUrl(event.project_name,
-                                                      event.change_number)
+        event.ref = "refs/merge-requests/%s/head" % event.change_number
+        event.branch = mr['target_branch']
+        event.change_url = self.connection.getMRUrl(event.project_name,
+                                                    event.change_number)
         event.action = 'comment'
         event.type = 'gl_merge_request'
         return event
 
-    def _handleEvent(self):
-        ts, json_body, event_type = self.connection.getEvent()
+    # https://docs.gitlab.com/ee/user/project/integrations/webhooks.html#push-events
+    def _event_push(self, body):
+        event = self._event_base(body)
+        event.branch = body['ref'].replace('refs/heads/', '')
+        event.ref = body['ref']
+        event.newrev = body['after']
+        event.oldrev = body['before']
+        event.type = 'gl_push'
+
+        self.connection.clearConnectionCacheOnBranchEvent(event)
+
+        return event
+
+    # https://gitlab.com/help/user/project/integrations/webhooks#tag-events
+    def _event_tag_push(self, body):
+        event = self._event_base(body)
+        event.ref = body['ref']
+        event.newrev = body['after']
+        event.oldrev = None
+        event.tag = body['ref'].replace('refs/tags/', '')
+        event.type = 'gl_push'
+        return event
+
+    def _handleEvent(self, connection_event):
         if self._stopped:
             return
 
-        self.log.info("Received event: %s" % str(event_type))
+        timestamp = time.time()
+        json_body = connection_event["payload"]
+        self.log.info(json_body)
+
+        event_type = json_body['object_kind']
+        self.log.info("Received event: %s", event_type)
 
         if event_type not in self.event_handler_mapping:
             message = "Unhandled Gitlab event: %s" % event_type
@@ -170,7 +199,8 @@ class GitlabEventConnector(threading.Thread):
 
         if event:
             event.zuul_event_id = str(uuid.uuid4())
-            event.timestamp = ts
+            event.timestamp = timestamp
+            event.project_hostname = self.connection.canonical_hostname
             if event.change_number:
                 project = self.connection.source.getProject(event.project_name)
                 self.connection._getChange(project,
@@ -179,20 +209,17 @@ class GitlabEventConnector(threading.Thread):
                                            refresh=True,
                                            url=event.change_url,
                                            event=event)
-            event.project_hostname = self.connection.canonical_hostname
-            self.connection.logEvent(event)
-            self.connection.sched.addEvent(event)
 
-    def run(self):
-        while True:
-            if self._stopped:
-                return
-            try:
-                self._handleEvent()
-            except Exception:
-                self.log.exception("Exception moving Gitlab event:")
-            finally:
-                self.connection.eventDone()
+            # If this event references a branch and we're excluding
+            # unprotected branches, we might need to check whether the
+            # branch is now protected.
+            if hasattr(event, "branch") and event.branch:
+                self.connection.checkBranchCache(event.project_name, event)
+
+            self.connection.logEvent(event)
+            self.connection.sched.addTriggerEvent(
+                self.connection.driver_name, event
+            )
 
 
 class GitlabAPIClientException(Exception):
@@ -204,7 +231,7 @@ class GitlabAPIClient():
 
     def __init__(self, baseurl, api_token):
         self.session = requests.Session()
-        self.baseurl = '%s/api/v4/' % baseurl
+        self.baseurl = '%s/api/v4' % baseurl
         self.api_token = api_token
         self.headers = {'Authorization': 'Bearer %s' % (
             self.api_token)}
@@ -235,6 +262,15 @@ class GitlabAPIClient():
             ret.status_code, ret.text))
         return ret.json(), ret.status_code, ret.url, 'POST'
 
+    def put(self, url, params=None, zuul_event_id=None):
+        log = get_annotated_logger(self.log, zuul_event_id)
+        log.info(
+            "Put on resource %s, params (%s) ..." % (url, params))
+        ret = self.session.put(url, data=params, headers=self.headers)
+        log.debug("PUT returned (code: %s): %s" % (
+            ret.status_code, ret.text))
+        return ret.json(), ret.status_code, ret.url, 'PUT'
+
     # https://docs.gitlab.com/ee/api/merge_requests.html#get-single-mr
     def get_mr(self, project_name, number, zuul_event_id=None):
         path = "/projects/%s/merge_requests/%s" % (
@@ -244,12 +280,43 @@ class GitlabAPIClient():
         return resp[0]
 
     # https://docs.gitlab.com/ee/api/branches.html#list-repository-branches
-    def get_project_branches(self, project_name, zuul_event_id=None):
-        path = "/projects/%s/repository/branches" % (
-            quote_plus(project_name))
-        resp = self.get(self.baseurl + path, zuul_event_id=zuul_event_id)
-        self._manage_error(*resp, zuul_event_id=zuul_event_id)
-        return [branch['name'] for branch in resp[0]]
+    def get_project_branches(self, project_name, exclude_unprotected,
+                             zuul_event_id=None):
+        path = "/projects/{}/repository/branches?per_page=100&page={}"
+        page = 1
+        branches = []
+
+        # Handle pagination
+        while True:
+            url = self.baseurl + path.format(quote_plus(project_name), page)
+            resp = self.get(url, zuul_event_id=zuul_event_id)
+            if resp[0]:
+                self._manage_error(*resp, zuul_event_id=zuul_event_id)
+                branches.extend(resp[0])
+                page += 1
+            else:
+                break
+
+        if exclude_unprotected:
+            return [branch['name'] for branch
+                    in branches if branch['protected']]
+        else:
+            return [branch['name'] for branch in branches]
+
+    # https://docs.gitlab.com/ee/api/branches.html#get-single-repository-branch
+    def get_project_branch(self, project_name, branch_name,
+                           zuul_event_id=None):
+        path = "/projects/{}/repository/branches/{}"
+        path = path.format(quote_plus(project_name), quote_plus(branch_name))
+        url = self.baseurl + path
+        resp = self.get(url, zuul_event_id=zuul_event_id)
+        try:
+            self._manage_error(*resp, zuul_event_id=zuul_event_id)
+        except GitlabAPIClientException:
+            if resp[1] != 404:
+                raise
+            return {}
+        return resp[0]
 
     # https://docs.gitlab.com/ee/api/notes.html#create-new-merge-request-note
     def comment_mr(self, project_name, number, msg, zuul_event_id=None):
@@ -262,8 +329,55 @@ class GitlabAPIClient():
         self._manage_error(*resp, zuul_event_id=zuul_event_id)
         return resp[0]
 
+    # https://docs.gitlab.com/ee/api/merge_request_approvals.html#approve-merge-request
+    def approve_mr(self, project_name, number, patchset, approve=True,
+                   zuul_event_id=None):
+        approve = 'approve' if approve else 'unapprove'
+        path = "/projects/%s/merge_requests/%s/%s" % (
+            quote_plus(project_name), number, approve)
+        params = {'sha': patchset} if approve else {}
+        resp = self.post(
+            self.baseurl + path, params=params,
+            zuul_event_id=zuul_event_id)
+        try:
+            self._manage_error(*resp, zuul_event_id=zuul_event_id)
+        except GitlabAPIClientException:
+            # approve and unapprove endpoint could return code 401 whether the
+            # actual state of the Merge Request approval. Two call on approve
+            # endpoint the second call return 401.
+            # 409 is returned when current HEAD of the merge request doesn't
+            # match the 'sha' parameter.
+            if resp[1] not in (401, 409):
+                raise
+            elif approve == 'approve' and resp[1] == 409:
+                log = get_annotated_logger(self.log, zuul_event_id)
+                log.error('Fail to approve the merge request: %s' % resp[0])
+                return
+        return resp[0]
 
-class GitlabConnection(BaseConnection):
+    # https://docs.gitlab.com/ee/api/merge_request_approvals.html#get-configuration-1
+    def get_mr_approvals_status(self, project_name, number,
+                                zuul_event_id=None):
+        path = "/projects/%s/merge_requests/%s/approvals" % (
+            quote_plus(project_name), number)
+        resp = self.get(self.baseurl + path, zuul_event_id=zuul_event_id)
+        self._manage_error(*resp, zuul_event_id=zuul_event_id)
+        return resp[0]
+
+    # https://docs.gitlab.com/ee/api/merge_requests.html#accept-mr
+    def merge_mr(self, project_name, number,
+                 zuul_event_id=None):
+        path = "/projects/%s/merge_requests/%s/merge" % (
+            quote_plus(project_name), number)
+        resp = self.put(self.baseurl + path, zuul_event_id=zuul_event_id)
+        try:
+            self._manage_error(*resp, zuul_event_id=zuul_event_id)
+        except GitlabAPIClientException as e:
+            raise MergeFailure('Merge request merge failed: %s' % e)
+        return resp[0]
+
+
+class GitlabConnection(CachedBranchConnection):
     driver_name = 'gitlab'
     log = logging.getLogger("zuul.GitlabConnection")
     payload_path = 'payload'
@@ -272,7 +386,6 @@ class GitlabConnection(BaseConnection):
         super(GitlabConnection, self).__init__(
             driver, connection_name, connection_config)
         self.projects = {}
-        self.project_branch_cache = {}
         self._change_cache = {}
         self.server = self.connection_config.get('server', 'gitlab.com')
         self.baseurl = self.connection_config.get(
@@ -287,7 +400,6 @@ class GitlabConnection(BaseConnection):
             'api_token', '')
         self.gl_client = GitlabAPIClient(self.baseurl, self.api_token)
         self.sched = None
-        self.event_queue = queue.Queue()
         self.source = driver.getSource(self)
 
     def _start_event_connector(self):
@@ -300,50 +412,41 @@ class GitlabConnection(BaseConnection):
             self.gitlab_event_connector.join()
 
     def onLoad(self):
-        self.log.info('Starting Gitlab connection: %s' % self.connection_name)
-        self.gearman_worker = GitlabGearmanWorker(self)
+        self.log.info('Starting Gitlab connection: %s', self.connection_name)
+        self.log.info('Creating Zookeeper event queue')
+        self.event_queue = ConnectionEventQueue(
+            self.sched.zk_client, self.connection_name
+        )
         self.log.info('Starting event connector')
         self._start_event_connector()
-        self.log.info('Starting GearmanWorker')
-        self.gearman_worker.start()
 
     def onStop(self):
-        if hasattr(self, 'gearman_worker'):
-            self.gearman_worker.stop()
+        if hasattr(self, 'gitlab_event_connector'):
             self._stop_event_connector()
 
-    def addEvent(self, data, event=None):
-        return self.event_queue.put((time.time(), data, event))
-
-    def getEvent(self):
-        return self.event_queue.get()
-
-    def eventDone(self):
-        self.event_queue.task_done()
-
     def getWebController(self, zuul_web):
         return GitlabWebController(zuul_web, self)
 
+    def getEventQueue(self):
+        return getattr(self, "event_queue", None)
+
     def getProject(self, name):
         return self.projects.get(name)
 
     def addProject(self, project):
         self.projects[project.name] = project
 
-    def getProjectBranches(self, project, tenant):
-        branches = self.project_branch_cache.get(project.name)
-
-        if branches is not None:
-            return branches
-
-        branches = self.gl_client.get_project_branches(project.name)
-        self.project_branch_cache[project.name] = branches
-
-        self.log.info("Got branches for %s" % project.name)
+    def _fetchProjectBranches(self, project: Project,
+                              exclude_unprotected: bool) -> List[str]:
+        branches = self.gl_client.get_project_branches(project.name,
+                                                       exclude_unprotected)
         return branches
 
-    def clearBranchCache(self):
-        self.project_branch_cache = {}
+    def isBranchProtected(self, project_name: str, branch_name: str,
+                          zuul_event_id=None) -> Optional[bool]:
+        branch = self.gl_client.get_project_branch(project_name, branch_name,
+                                                   zuul_event_id)
+        return branch.get('protected')
 
     def getGitwebUrl(self, project, sha=None):
         url = '%s/%s' % (self.baseurl, project)
@@ -351,7 +454,7 @@ class GitlabConnection(BaseConnection):
             url += '/tree/%s' % sha
         return url
 
-    def getPullUrl(self, project, number):
+    def getMRUrl(self, project, number):
         return '%s/%s/merge_requests/%s' % (self.baseurl, project, number)
 
     def getGitUrl(self, project):
@@ -365,19 +468,33 @@ class GitlabConnection(BaseConnection):
             change = self._getChange(
                 project, event.change_number, event.patch_number,
                 refresh=refresh, event=event)
-            change.source_event = event
             change.is_current_patchset = (change.patchset ==
                                           event.patch_number)
         else:
             self.log.info("Getting change for %s ref:%s" % (
                 project, event.ref))
-            raise NotImplementedError
+            if event.ref and event.ref.startswith('refs/tags/'):
+                change = Tag(project)
+                change.tag = event.tag
+                change.branch = None
+            elif event.ref and event.ref.startswith('refs/heads/'):
+                change = Branch(project)
+                change.branch = event.branch
+            else:
+                change = Ref(project)
+                change.branch = None
+            change.ref = event.ref
+            change.oldrev = event.oldrev
+            change.newrev = event.newrev
+            change.url = self.getGitwebUrl(project, sha=event.newrev)
+
+            change.files = None
         return change
 
-    def _getChange(self, project, number, patchset=None,
+    def _getChange(self, project, number, patch_number=None,
                    refresh=False, url=None, event=None):
         log = get_annotated_logger(self.log, event)
-        key = (project.name, str(number), str(patchset))
+        key = (project.name, str(number), str(patch_number))
         change = self._change_cache.get(key)
         if change and not refresh:
             log.debug("Getting change from cache %s" % str(key))
@@ -386,9 +503,9 @@ class GitlabConnection(BaseConnection):
             change = MergeRequest(project.name)
             change.project = project
             change.number = number
-            # patchset is the tips commit of the PR
-            change.patchset = patchset
-            change.url = url or self.getPullUrl(project.name, number)
+            # patch_number is the tips commit SHA of the MR
+            change.patchset = patch_number
+            change.url = url or self.getMRUrl(project.name, number)
             change.uris = [change.url.split('://', 1)[-1]]  # remove scheme
         self._change_cache[key] = change
         try:
@@ -404,24 +521,27 @@ class GitlabConnection(BaseConnection):
     def _updateChange(self, change, event):
         log = get_annotated_logger(self.log, event)
         log.info("Updating change from Gitlab %s" % change)
-        change.mr = self.getPull(
+        change.mr = self.getMR(
             change.project.name, change.number, event=event)
         change.ref = "refs/merge-requests/%s/head" % change.number
         change.branch = change.mr['target_branch']
         change.patchset = change.mr['sha']
+        change.commit_id = change.mr['diff_refs'].get('head_sha')
+        change.owner = change.mr['author'].get('username')
         # Files changes are not part of the Merge Request data
         # See api/merge_requests.html#get-single-mr-changes
         # this endpoint includes file changes information
         change.files = None
         change.title = change.mr['title']
         change.open = change.mr['state'] == 'opened'
-        change.is_merged = change.mr['merged_at'] is not None
+        change.is_merged = change.mr['state'] == 'merged'
         # Can be "can_be_merged"
         change.merge_status = change.mr['merge_status']
+        change.approved = change.mr['approved']
         change.message = change.mr['description']
         change.labels = change.mr['labels']
-        change.updated_at = int(datetime.strptime(
-            change.mr['updated_at'], '%Y-%m-%dT%H:%M:%S.%fZ').strftime('%s'))
+        change.updated_at = int(dateutil.parser.parse(
+            change.mr['updated_at']).timestamp())
         log.info("Updated change from Gitlab %s" % change)
 
         if self.sched:
@@ -429,10 +549,26 @@ class GitlabConnection(BaseConnection):
 
         return change
 
-    def getPull(self, project_name, number, event=None):
+    def canMerge(self, change, allow_needs, event=None):
+        log = get_annotated_logger(self.log, event)
+        can_merge = True if change.merge_status == "can_be_merged" else False
+        log.info('Check MR %s#%s mergeability can_merge: %s',
+                 change.project.name, change.number, can_merge)
+        return can_merge
+
+    def getMR(self, project_name, number, event=None):
         log = get_annotated_logger(self.log, event)
         mr = self.gl_client.get_mr(project_name, number, zuul_event_id=event)
         log.info('Got MR %s#%s', project_name, number)
+        mr_approval_status = self.gl_client.get_mr_approvals_status(
+            project_name, number, zuul_event_id=event)
+        log.info('Got MR approval status %s#%s', project_name, number)
+        if 'approvals_left' in mr_approval_status:
+            # 'approvals_left' is not present when 'Required Merge Request
+            # Approvals' feature isn't available
+            mr['approved'] = mr_approval_status['approvals_left'] == 0
+        else:
+            mr['approved'] = mr_approval_status['approved']
         return mr
 
     def commentMR(self, project_name, number, message, event=None):
@@ -441,6 +577,28 @@ class GitlabConnection(BaseConnection):
             project_name, number, message, zuul_event_id=event)
         log.info("Commented on MR %s#%s", project_name, number)
 
+    def approveMR(self, project_name, number, patchset, approve, event=None):
+        log = get_annotated_logger(self.log, event)
+        result = self.gl_client.approve_mr(
+            project_name, number, patchset, approve, zuul_event_id=event)
+        if result:
+            log.info(
+                "Set approval: %s on MR %s#%s (%s)", approve,
+                project_name, number, patchset)
+
+    def getChangesDependingOn(self, change, projects, tenant):
+        """ Reverse lookup of MR depending on this one
+        """
+        # TODO(fbo): Further research to implement this lookup:
+        # https://docs.gitlab.com/ee/api/search.html#scope-merge_requests
+        # Will be done in a folloup commit
+        return []
+
+    def mergeMR(self, project_name, number, event=None):
+        log = get_annotated_logger(self.log, event)
+        self.gl_client.merge_mr(project_name, number, zuul_event_id=event)
+        log.info("Merged MR %s#%s", project_name, number)
+
 
 class GitlabWebController(BaseWebController):
 
@@ -449,6 +607,10 @@ class GitlabWebController(BaseWebController):
     def __init__(self, zuul_web, connection):
         self.connection = connection
         self.zuul_web = zuul_web
+        self.event_queue = ConnectionEventQueue(
+            self.zuul_web.zk_client,
+            self.connection.connection_name
+        )
 
     def _validate_token(self, headers):
         try:
@@ -477,11 +639,9 @@ class GitlabWebController(BaseWebController):
         self._validate_token(headers)
         json_payload = json.loads(body.decode('utf-8'))
 
-        job = self.zuul_web.rpc.submitJob(
-            'gitlab:%s:payload' % self.connection.connection_name,
-            {'payload': json_payload})
-
-        return json.loads(job.data[0])
+        data = {'payload': json_payload}
+        self.event_queue.put(data)
+        return data
 
 
 def getSchema():
