@@ -17,6 +17,7 @@ import time
 import voluptuous as v
 
 from zuul.lib.logutil import get_annotated_logger
+from zuul.lib.result_data import get_artifacts_from_result_data
 from zuul.reporter import BaseReporter
 
 
@@ -40,6 +41,8 @@ class MQTTReporter(BaseReporter):
             'change_url': item.change.url,
             'change': getattr(item.change, 'number', ''),
             'patchset': getattr(item.change, 'patchset', ''),
+            'commit_id': getattr(item.change, 'commit_id', ''),
+            'owner': getattr(item.change, 'owner', ''),
             'ref': getattr(item.change, 'ref', ''),
             'message': self._formatItemReport(
                 item, with_jobs=False),
@@ -48,7 +51,8 @@ class MQTTReporter(BaseReporter):
             'buildset': {
                 'uuid': item.current_build_set.uuid,
                 'result': item.current_build_set.result,
-                'builds': []
+                'builds': [],
+                'retry_builds': [],
             },
             'zuul_event_id': item.event.zuul_event_id,
         }
@@ -60,16 +64,38 @@ class MQTTReporter(BaseReporter):
             build = item.current_build_set.getBuild(job.name)
             if build:
                 # Report build data if available
-                (result, url) = item.formatJobResult(job)
+                (result, web_url) = item.formatJobResult(job)
                 job_informations.update({
                     'uuid': build.uuid,
                     'start_time': build.start_time,
                     'end_time': build.end_time,
                     'execute_time': build.execute_time,
-                    'log_url': url,
+                    'log_url': build.log_url,
+                    'web_url': web_url,
                     'result': result,
                     'dependencies': [j.name for j in job.dependencies],
+                    'artifacts': get_artifacts_from_result_data(
+                        build.result_data, logger=log)
                 })
+
+                # Report build data of retried builds if available
+                retry_builds = item.current_build_set.getRetryBuildsForJob(
+                    job.name)
+                for build in retry_builds:
+                    (result, url) = item.formatJobResult(job, build)
+                    retry_build_information = {
+                        'job_name': job.name,
+                        'voting': job.voting,
+                        'uuid': build.uuid,
+                        'start_time': build.start_time,
+                        'end_time': build.end_time,
+                        'execute_time': build.execute_time,
+                        'log_url': url,
+                        'result': result,
+                    }
+                    message['buildset']['retry_builds'].append(
+                        retry_build_information)
+
             message['buildset']['builds'].append(job_informations)
         topic = None
         try:
