@@ -13,6 +13,7 @@
 # under the License.
 
 import logging
+import time
 
 import alembic
 import alembic.command
@@ -60,7 +61,7 @@ class DatabaseSession(object):
                   change=None, branch=None, patchset=None, ref=None,
                   newrev=None, event_id=None, uuid=None, job_name=None,
                   voting=None, node_name=None, result=None, provides=None,
-                  limit=50, offset=0):
+                  final=None, held=None, limit=50, offset=0):
 
         build_table = self.connection.zuul_build_table
         buildset_table = self.connection.zuul_buildset_table
@@ -102,7 +103,9 @@ class DatabaseSession(object):
         q = self.listFilter(q, build_table.c.voting, voting)
         q = self.listFilter(q, build_table.c.node_name, node_name)
         q = self.listFilter(q, build_table.c.result, result)
+        q = self.listFilter(q, build_table.c.final, final)
         q = self.listFilter(q, provides_table.c.name, provides)
+        q = self.listFilter(q, build_table.c.held, held)
 
         q = q.order_by(build_table.c.id.desc()).\
             limit(limit).\
@@ -186,8 +189,9 @@ class SQLConnection(BaseConnection):
         self.dburi = None
         self.engine = None
         self.connection = None
-        self.tables_established = False
         self.table_prefix = self.connection_config.get('table_prefix', '')
+        self.log.info("Initializing SQL connection {} (prefix: {})".format(
+            connection_name, self.table_prefix))
 
         try:
             self.dburi = self.connection_config.get('dburi')
@@ -211,15 +215,10 @@ class SQLConnection(BaseConnection):
                                                     expire_on_commit=False,
                                                     autoflush=False)
             self.session = orm.scoped_session(self.session_factory)
-
         except sa.exc.NoSuchModuleError:
-            self.log.exception(
-                "The required module for the dburi dialect isn't available. "
-                "SQL connection %s will be unavailable." % connection_name)
-        except sa.exc.OperationalError:
-            self.log.exception(
-                "Unable to connect to the database or establish the required "
-                "tables. Reporter %s is disabled" % self)
+            self.log.error(
+                "The required module for the dburi dialect isn't available.")
+            raise
 
     def getSession(self):
         return DatabaseSession(self)
@@ -243,18 +242,15 @@ class SQLConnection(BaseConnection):
             alembic.command.upgrade(config, 'head', tag=tag)
 
     def onLoad(self):
-        try:
-            self._migrate()
-            self.tables_established = True
-        except sa.exc.NoSuchModuleError:
-            self.log.exception(
-                "The required module for the dburi dialect isn't available. "
-                "SQL connection %s will be unavailable." %
-                self.connection_name)
-        except sa.exc.OperationalError:
-            self.log.exception(
-                "Unable to connect to the database or establish the required "
-                "tables. Connection %s is disabled" % self)
+        while True:
+            try:
+                self._migrate()
+                break
+            except sa.exc.OperationalError:
+                self.log.error(
+                    "Unable to connect to the database or establish the "
+                    "required tables.")
+            time.sleep(10)
 
     def _setup_models(self):
         Base = declarative_base(metadata=sa.MetaData())
@@ -295,12 +291,14 @@ class SQLConnection(BaseConnection):
             uuid = sa.Column(sa.String(36))
             job_name = sa.Column(sa.String(255))
             result = sa.Column(sa.String(255))
+            held = sa.Column(sa.Boolean)
             start_time = sa.Column(sa.DateTime)
             end_time = sa.Column(sa.DateTime)
             voting = sa.Column(sa.Boolean)
             log_url = sa.Column(sa.String(255))
             node_name = sa.Column(sa.String(255))
             error_detail = sa.Column(sa.TEXT())
+            final = sa.Column(sa.Boolean)
             buildset = orm.relationship(BuildSetModel, backref="builds")
 
             def createArtifact(self, *args, **kw):
