@@ -23,6 +23,7 @@ from apscheduler.triggers.cron import CronTrigger
 
 from zuul.driver import Driver, TriggerInterface
 from zuul.driver.timer import timertrigger
+from zuul.driver.timer import timermodel
 from zuul.driver.timer.timermodel import TimerTriggerEvent
 from zuul.lib.logutil import get_annotated_logger
 
@@ -129,7 +130,7 @@ class TimerDriver(Driver, TriggerInterface):
                 event.timestamp = time.time()
                 log = get_annotated_logger(self.log, event)
                 log.debug("Adding event")
-                self.sched.addEvent(event)
+                self.sched.addTriggerEvent(self.name, event)
 
     def stop(self):
         if self.apsched:
@@ -141,3 +142,6 @@ class TimerDriver(Driver, TriggerInterface):
 
     def getTriggerSchema(self):
         return timertrigger.getSchema()
+
+    def getTriggerEventClass(self):
+        return timermodel.TimerTriggerEvent
