@@ -18,6 +18,7 @@ from uuid import uuid4
 
 from zuul.driver import Driver, TriggerInterface
 from zuul.driver.zuul.zuulmodel import ZuulTriggerEvent
+from zuul.driver.zuul import zuulmodel
 from zuul.driver.zuul import zuultrigger
 from zuul.lib.logutil import get_annotated_logger
 
@@ -86,6 +87,7 @@ class ZuulDriver(Driver, TriggerInterface):
         event = ZuulTriggerEvent()
         event.type = PROJECT_CHANGE_MERGED
         event.trigger_name = self.name
+        event.connection_name = "zuul"
         event.project_hostname = change.project.canonical_hostname
         event.project_name = change.project.name
         event.change_number = change.number
@@ -95,7 +97,7 @@ class ZuulDriver(Driver, TriggerInterface):
         event.ref = change.ref
         event.zuul_event_id = str(uuid4().hex)
         event.timestamp = time.time()
-        self.sched.addEvent(event)
+        self.sched.addTriggerEvent(self.name, event)
 
     def _createParentChangeEnqueuedEvents(self, change, pipeline, tenant,
                                           event):
@@ -122,6 +124,7 @@ class ZuulDriver(Driver, TriggerInterface):
     def _createParentChangeEnqueuedEvent(self, change, pipeline):
         event = ZuulTriggerEvent()
         event.type = PARENT_CHANGE_ENQUEUED
+        event.connection_name = "zuul"
         event.trigger_name = self.name
         event.pipeline_name = pipeline.name
         event.project_hostname = change.project.canonical_hostname
@@ -132,10 +135,13 @@ class ZuulDriver(Driver, TriggerInterface):
         event.patch_number = change.patchset
         event.ref = change.ref
         event.zuul_event_id = str(uuid4().hex)
-        self.sched.addEvent(event)
+        self.sched.addTriggerEvent(self.name, event)
 
     def getTrigger(self, connection_name, config=None):
         return zuultrigger.ZuulTrigger(self, config)
 
     def getTriggerSchema(self):
         return zuultrigger.getSchema()
+
+    def getTriggerEventClass(self):
+        return zuulmodel.ZuulTriggerEvent
