@@ -16,12 +16,14 @@ import functools
 import logging
 import socket
 import threading
+from configparser import ConfigParser
+from typing import Optional, Tuple
 
 import zuul.rpcclient
-
-from zuul.lib import commandsocket
 from zuul.lib import streamer_utils
-
+from zuul.lib.commandsocket import CommandSocket
+from zuul.zk import ZooKeeperClient
+from zuul.zk.components import FingerGatewayComponent
 
 COMMANDS = ['stop']
 
@@ -101,10 +103,19 @@ class FingerGateway(object):
 
     log = logging.getLogger("zuul.fingergw")
 
-    def __init__(self, gearman, address, user, command_socket, pid_file):
+    def __init__(
+        self,
+        config: ConfigParser,
+        gearman: Tuple,
+        address: Tuple,
+        user: Optional[str],
+        command_socket: Optional[str],
+        pid_file: Optional[str],
+    ):
         '''
         Initialize the finger gateway.
 
+        :param config: The parsed Zuul configuration.
         :param tuple gearman: Gearman connection information. This should
             include the server, port, SSL key, SSL cert, and SSL CA.
         :param tuple address: The address and port to bind to for our gateway.
@@ -128,12 +139,21 @@ class FingerGateway(object):
 
         self.command_thread = None
         self.command_running = False
-        self.command_socket = command_socket
+        self.command_socket_path = command_socket
+        self.command_socket = None
 
         self.command_map = dict(
             stop=self.stop,
         )
 
+        self.zk_client = ZooKeeperClient.fromConfig(config)
+        self.zk_client.connect()
+        self.hostname = socket.getfqdn()
+        self.component_info = FingerGatewayComponent(
+            self.zk_client, self.hostname
+        )
+        self.component_info.register()
+
     def _runCommand(self):
         while self.command_running:
             try:
@@ -168,10 +188,9 @@ class FingerGateway(object):
             pid_file=self.pid_file)
 
         # Start the command processor after the server and privilege drop
-        if self.command_socket:
+        if self.command_socket_path:
             self.log.debug("Starting command processor")
-            self.command_socket = commandsocket.CommandSocket(
-                self.command_socket)
+            self.command_socket = CommandSocket(self.command_socket_path)
             self.command_socket.start()
             self.command_running = True
             self.command_thread = threading.Thread(
@@ -184,9 +203,11 @@ class FingerGateway(object):
         self.server_thread = threading.Thread(target=self._run)
         self.server_thread.daemon = True
         self.server_thread.start()
+        self.component_info.state = self.component_info.RUNNING
         self.log.info("Finger gateway is started")
 
     def stop(self):
+        self.component_info.state = self.component_info.STOPPED
         if self.server:
             try:
                 self.server.shutdown()
@@ -210,6 +231,8 @@ class FingerGateway(object):
             except Exception:
                 self.log.exception("Error stopping command socket:")
 
+        self.zk_client.disconnect()
+
         self.log.info("Finger gateway is stopped")
 
     def wait(self):
@@ -217,3 +240,6 @@ class FingerGateway(object):
         Wait on the gateway to shutdown.
         '''
         self.server_thread.join()
+
+        if self.command_thread:
+            self.command_thread.join()
