@@ -50,13 +50,30 @@ class DependentPipelineManager(SharedQueuePipelineManager):
         source = change.project.source
         if not source.canMerge(change, self.getSubmitAllowNeeds(),
                                event=event):
-            log.debug("Change %s can not merge, ignoring", change)
+            log.debug("Change %s can not merge", change)
             return False
         return True
 
+    def canMergeCycle(self, bundle):
+        """Check if the cycle still fulfills the pipeline's ready criteria."""
+        for item in bundle.items:
+            source = item.change.project.source
+            if not source.canMerge(
+                item.change,
+                self.getSubmitAllowNeeds(),
+                event=item.event,
+                allow_refresh=True,
+            ):
+                log = get_annotated_logger(self.log, item.event)
+                log.debug("Change %s can no longer be merged", item.change)
+                return False
+        return True
+
     def enqueueChangesBehind(self, change, event, quiet, ignore_requirements,
-                             change_queue):
+                             change_queue, history=None,
+                             dependency_graph=None):
         log = get_annotated_logger(self.log, event)
+        history = history if history is not None else []
 
         log.debug("Checking for changes needing %s:" % change)
         if not hasattr(change, 'needed_by_changes'):
@@ -65,15 +82,17 @@ class DependentPipelineManager(SharedQueuePipelineManager):
 
         # for project in change_queue, project.source get changes, then dedup.
         sources = set()
-        for project in change_queue.projects:
+        for project, _ in change_queue.project_branches:
             sources.add(project.source)
 
         seen = set(change.needed_by_changes)
         needed_by_changes = change.needed_by_changes[:]
         for source in sources:
             log.debug("  Checking source: %s", source)
+            projects = [project_branch[0]
+                        for project_branch in change_queue.project_branches]
             for c in source.getChangesDependingOn(change,
-                                                  change_queue.projects,
+                                                  projects,
                                                   self.pipeline.tenant):
                 if c not in seen:
                     seen.add(c)
@@ -82,7 +101,17 @@ class DependentPipelineManager(SharedQueuePipelineManager):
         log.debug("  Following changes: %s", needed_by_changes)
 
         to_enqueue = []
+        change_dependencies = dependency_graph.get(change, [])
         for other_change in needed_by_changes:
+            if other_change in change_dependencies:
+                # Only consider the change if it is not part of a cycle, as
+                # cycle changes will otherwise be partially enqueued without
+                # any error handling
+                self.log.debug(
+                    "    Skipping change %s due to dependency cycle"
+                )
+                continue
+
             with self.getChangeQueue(other_change,
                                      event) as other_change_queue:
                 if other_change_queue != change_queue:
@@ -92,7 +121,8 @@ class DependentPipelineManager(SharedQueuePipelineManager):
                                change_queue))
                     continue
             source = other_change.project.source
-            if source.canMerge(other_change, self.getSubmitAllowNeeds()):
+            if source.canMerge(other_change, self.getSubmitAllowNeeds(),
+                               event=event):
                 log.debug("  Change %s needs %s and is ready to merge",
                           other_change, change)
                 to_enqueue.append(other_change)
@@ -103,32 +133,41 @@ class DependentPipelineManager(SharedQueuePipelineManager):
         for other_change in to_enqueue:
             self.addChange(other_change, event, quiet=quiet,
                            ignore_requirements=ignore_requirements,
-                           change_queue=change_queue)
+                           change_queue=change_queue, history=history,
+                           dependency_graph=dependency_graph)
 
     def enqueueChangesAhead(self, change, event, quiet, ignore_requirements,
-                            change_queue, history=None):
+                            change_queue, history=None, dependency_graph=None):
         log = get_annotated_logger(self.log, event)
 
+        history = history if history is not None else []
         if hasattr(change, 'number'):
-            history = history or []
-            history = history + [change]
+            history.append(change)
         else:
             # Don't enqueue dependencies ahead of a non-change ref.
             return True
 
-        ret = self.checkForChangesNeededBy(change, change_queue, event)
+        ret = self.checkForChangesNeededBy(change, change_queue, event,
+                                           dependency_graph=dependency_graph)
         if ret in [True, False]:
             return ret
         log.debug("  Changes %s must be merged ahead of %s", ret, change)
         for needed_change in ret:
-            r = self.addChange(needed_change, event, quiet=quiet,
-                               ignore_requirements=ignore_requirements,
-                               change_queue=change_queue, history=history)
-            if not r:
-                return False
+            # If the change is already in the history, but the change also has
+            # a git level dependency, we need to enqueue it before the current
+            # change.
+            if (needed_change not in history or
+                needed_change in change.git_needs_changes):
+                r = self.addChange(needed_change, event, quiet=quiet,
+                                   ignore_requirements=ignore_requirements,
+                                   change_queue=change_queue, history=history,
+                                   dependency_graph=dependency_graph)
+                if not r:
+                    return False
         return True
 
-    def checkForChangesNeededBy(self, change, change_queue, event):
+    def checkForChangesNeededBy(self, change, change_queue, event,
+                                dependency_graph=None):
         log = get_annotated_logger(self.log, event)
 
         # Return true if okay to proceed enqueing this change,
@@ -152,6 +191,13 @@ class DependentPipelineManager(SharedQueuePipelineManager):
                 if needed_change.is_merged:
                     log.debug("  Needed change is merged")
                     continue
+
+                if dependency_graph is not None:
+                    log.debug("  Adding change %s to dependency graph for "
+                              "change %s", needed_change, change)
+                    node = dependency_graph.setdefault(change, [])
+                    node.append(needed_change)
+
                 with self.getChangeQueue(needed_change,
                                          event) as needed_change_queue:
                     if needed_change_queue != change_queue:
@@ -168,7 +214,8 @@ class DependentPipelineManager(SharedQueuePipelineManager):
                     log.debug("  Needed change is already ahead in the queue")
                     continue
                 if needed_change.project.source.canMerge(
-                        needed_change, self.getSubmitAllowNeeds()):
+                        needed_change, self.getSubmitAllowNeeds(),
+                        event=event):
                     log.debug("  Change %s is needed", needed_change)
                     if needed_change not in changes_needed:
                         changes_needed.append(needed_change)
@@ -193,6 +240,9 @@ class DependentPipelineManager(SharedQueuePipelineManager):
                 continue
             if needed_item.current_build_set.failing_reasons:
                 failing_items.add(needed_item)
+        if item.isBundleFailing():
+            failing_items.update(item.bundle.items)
+            failing_items.remove(item)
         if failing_items:
             return failing_items
         return None
