@@ -14,7 +14,10 @@
 # under the License.
 
 from contextlib import contextmanager
+from logging import Logger
+from typing import Optional
 from urllib.parse import urlsplit, urlunsplit, urlparse
+import hashlib
 import logging
 import os
 import re
@@ -24,6 +27,8 @@ import time
 import git
 import gitdb
 import paramiko
+from zuul.zk import ZooKeeperClient
+from zuul.lib import strings
 
 import zuul.model
 
@@ -32,21 +37,6 @@ from zuul.lib.logutil import get_annotated_logger
 NULL_REF = '0000000000000000000000000000000000000000'
 
 
-def reset_repo_to_head(repo):
-    # This lets us reset the repo even if there is a file in the root
-    # directory named 'HEAD'.  Currently, GitPython does not allow us
-    # to instruct it to always include the '--' to disambiguate.  This
-    # should no longer be necessary if this PR merges:
-    #   https://github.com/gitpython-developers/GitPython/pull/319
-    try:
-        repo.git.reset('--hard', 'HEAD', '--')
-    except git.GitCommandError as e:
-        # git nowadays may use 1 as status to indicate there are still unstaged
-        # modifications after the reset
-        if e.status != 1:
-            raise
-
-
 def redact_url(url):
     parsed = urlsplit(url)
     if parsed.password is None:
@@ -183,12 +173,50 @@ class Repo(object):
             for ref in origin.refs:
                 if ref.remote_head == 'HEAD':
                     continue
-                repo.create_head(ref.remote_head, ref, force=True)
+                repo.create_head('refs/heads/' + ref.remote_head,
+                                 ref.commit,
+                                 force=True)
         with repo.config_writer() as config_writer:
             if self.email:
                 config_writer.set_value('user', 'email', self.email)
             if self.username:
                 config_writer.set_value('user', 'name', self.username)
+
+            # By default automatic garbage collection in git runs
+            # asynchronously in the background. This can lead to broken repos
+            # caused by a race in the following scenario:
+            #  1. git fetch (eventually triggers async gc)
+            #  2. zuul deletes all refs as part of reset
+            #  3. git gc looks for unreachable objects
+            #  4. zuul re-creates all refs as part of reset
+            #  5. git gc deletes unreachable objects it found
+            # Result is a repo with refs pointing to not existing objects.
+            # To prevent this race autoDetach can be disabled so git fetch
+            # returns after the gc finished.
+            config_writer.set_value('gc', 'autoDetach', 'false')
+
+            # Lower the threshold of how many loose objects can trigger
+            # automatic garbage collection. With the default value of 6700
+            # we observed that with some repos automatic garbage collection
+            # simply refused to do its job because it refuses to prune if the
+            # number of unreachable objects it needs to prune exceeds a certain
+            # threshold. Thus lower the threshold to trigger automatic garbage
+            # collection more often.
+            config_writer.set_value('gc', 'auto', '512')
+
+            # By default garbage collection keeps unreachable objects for two
+            # weeks. However we don't need to carry around any unreachable
+            # objects so just prune them all when gc kicks in.
+            config_writer.set_value('gc', 'pruneExpire', 'now')
+
+            # By default git keeps a reflog of each branch for 90 days. Objects
+            # that are reachable from a reflog entry are not considered
+            # unrechable and thus won't be pruned for 90 days. This can blow up
+            # the repo significantly over time. Since the reflog is only really
+            # useful for humans working with repos we can just drop all the
+            # reflog when gc kicks in.
+            config_writer.set_value('gc', 'reflogExpire', 'now')
+
             config_writer.write()
         if rewrite_url:
             self._git_set_remote_url(repo, self.remote_url)
@@ -220,7 +248,10 @@ class Repo(object):
         for attempt in range(1, self.retry_attempts + 1):
             try:
                 with timeout_handler(self.local_path):
-                    repo.git.fetch(remote, ref,
+                    ref_to_fetch = ref
+                    if ref_to_fetch:
+                        ref_to_fetch += ':refs/zuul/fetch'
+                    repo.git.fetch(remote, ref_to_fetch,
                                    kill_after_timeout=self.git_timeout, f=True,
                                    **kwargs)
                 break
@@ -228,11 +259,26 @@ class Repo(object):
                 if attempt < self.retry_attempts:
                     if 'fatal: bad config' in e.stderr.lower():
                         # This error can be introduced by a merge conflict
+                        # or someone committing faulty configuration
                         # in the .gitmodules which was left by the last
                         # merge operation. In this case reset and clean
                         # the repo and try again immediately.
-                        reset_repo_to_head(repo)
-                        repo.git.clean('-x', '-f', '-d')
+                        reset_ref = 'HEAD'
+                        try:
+                            if not repo.is_dirty():
+                                reset_ref = "{}^".format(repo.git.log(
+                                    '--diff-filter=A',
+                                    '-n', '1',
+                                    '--pretty=format:%H',
+                                    '--', '.gitmodules'))
+                            repo.head.reset(reset_ref, working_tree=True)
+                            repo.git.clean('-x', '-f', '-d')
+                        except Exception:
+                            # If we get here there probably isn't
+                            # a valid commit we can easily find so
+                            # delete the repo to make sure it doesn't
+                            # get stuck in a broken state.
+                            shutil.rmtree(self.local_path)
                     elif 'fatal: not a git repository' in e.stderr.lower():
                         # If we get here the git.Repo object was happy with its
                         # lightweight way of checking if this is a valid git
@@ -281,6 +327,12 @@ class Repo(object):
                     messages.append("Cleaning empty ref dir %s" % root)
                 os.rmdir(root)
 
+    @staticmethod
+    def refNameToZuulRef(ref_name: str) -> str:
+        return "refs/zuul/{}".format(
+            hashlib.sha1(ref_name.encode("utf-8")).hexdigest()
+        )
+
     @staticmethod
     def _reset(local_path, env, log=None):
         messages = []
@@ -303,6 +355,9 @@ class Repo(object):
             raise Exception("Couldn't detach HEAD to any existing commit")
 
         # Delete local heads that no longer exist on the remote end
+        zuul_refs_to_keep = [
+            "refs/zuul/fetch",  # ref to last FETCH_HEAD
+        ]
         remote_heads = {r.remote_head for r in origin.refs}
         for ref in repo.heads:
             if ref.name not in remote_heads:
@@ -311,6 +366,18 @@ class Repo(object):
                 else:
                     messages.append("Delete stale local ref %s" % ref)
                 repo.delete_head(ref, force=True)
+            else:
+                zuul_refs_to_keep.append(Repo.refNameToZuulRef(ref.name))
+
+        # Delete local zuul refs when the related branch no longer exists
+        for ref in (r for r in repo.refs if r.path.startswith("refs/zuul/")):
+            if ref.path in zuul_refs_to_keep:
+                continue
+            if log:
+                log.debug("Delete stale Zuul ref %s", ref)
+            else:
+                messages.append("Delete stale Zuul ref {}".format(ref))
+            Repo._deleteRef(ref.path, repo)
 
         # Note: Before git 2.13 deleting a a ref foo/bar leaves an empty
         # directory foo behind that will block creating the reference foo
@@ -323,13 +390,14 @@ class Repo(object):
         for ref in origin.refs:
             if ref.remote_head == 'HEAD':
                 continue
-            repo.create_head(ref.remote_head, ref, force=True)
+            repo.create_head('refs/heads/' + ref.remote_head,
+                             ref.commit,
+                             force=True)
         return messages
 
     def reset(self, zuul_event_id=None, build=None, process_worker=None):
         log = get_annotated_logger(self.log, zuul_event_id, build=build)
         log.debug("Resetting repository %s", self.local_path)
-        self.update(zuul_event_id=zuul_event_id, build=build)
         self.createRepoObject(zuul_event_id, build=build)
 
         if process_worker is None:
@@ -345,6 +413,11 @@ class Repo(object):
         branch_head = repo.heads[branch]
         return branch_head.commit
 
+    def setBranchHead(self, branch, head_ref, repo=None, zuul_event_id=None):
+        if repo is None:
+            repo = self.createRepoObject(zuul_event_id)
+        repo.create_head(branch, head_ref, force=True)
+
     def hasBranch(self, branch, zuul_event_id=None):
         repo = self.createRepoObject(zuul_event_id)
         origin = repo.remotes.origin
@@ -368,9 +441,10 @@ class Repo(object):
         return repo.refs
 
     def setRef(self, path, hexsha, repo=None, zuul_event_id=None):
-        log = get_annotated_logger(self.log, zuul_event_id)
-        log.debug("Create reference %s at %s in %s",
-                  path, hexsha, self.local_path)
+        ref_log = get_annotated_logger(
+            logging.getLogger("zuul.Repo.Ref"), zuul_event_id)
+        ref_log.debug("Create reference %s at %s in %s",
+                      path, hexsha, self.local_path)
         if repo is None:
             repo = self.createRepoObject(zuul_event_id)
         self._setRef(path, hexsha, repo)
@@ -385,8 +459,9 @@ class Repo(object):
 
     def setRefs(self, refs, keep_remotes=False, zuul_event_id=None):
         repo = self.createRepoObject(zuul_event_id)
-        log = get_annotated_logger(self.log, zuul_event_id)
-        self._setRefs(repo, refs, keep_remotes=keep_remotes, log=log)
+        ref_log = get_annotated_logger(
+            logging.getLogger("zuul.Repo.Ref"), zuul_event_id)
+        self._setRefs(repo, refs, keep_remotes=keep_remotes, log=ref_log)
 
     @staticmethod
     def setRefsAsync(local_path, refs, keep_remotes=False):
@@ -430,10 +505,11 @@ class Repo(object):
         origin_ref.commit = rev
 
     def deleteRef(self, path, repo=None, zuul_event_id=None):
-        log = get_annotated_logger(self.log, zuul_event_id)
+        ref_log = get_annotated_logger(
+            logging.getLogger("zuul.Repo.Ref"), zuul_event_id)
         if repo is None:
             repo = self.createRepoObject(zuul_event_id)
-        log.debug("Delete reference %s", path)
+        ref_log.debug("Delete reference %s", path)
         Repo._deleteRef(path, repo)
 
     @staticmethod
@@ -454,7 +530,7 @@ class Repo(object):
             # that we clean up anything that might be left over from a merge
             # while still only preparing the working copy once.
             repo.head.reference = ref
-            reset_repo_to_head(repo)
+            repo.head.reset(working_tree=True)
             repo.git.clean('-x', '-f', '-d')
             repo.git.checkout(ref)
 
@@ -528,12 +604,13 @@ class Repo(object):
             # --tags' is all that is necessary.  See
             # https://github.com/git/git/blob/master/Documentation/RelNotes/1.9.0.txt#L18-L20
             self._git_fetch(repo, 'origin', zuul_event_id)
-        self._git_fetch(repo, 'origin', zuul_event_id, tags=True, prune=True)
+        self._git_fetch(repo, 'origin', zuul_event_id, tags=True,
+                        prune=True, prune_tags=True)
 
-    def isUpdateNeeded(self, repo_state, zuul_event_id=None):
+    def isUpdateNeeded(self, project_repo_state, zuul_event_id=None):
         repo = self.createRepoObject(zuul_event_id)
         refs = [x.path for x in repo.refs]
-        for ref, rev in repo_state.items():
+        for ref, rev in project_repo_state.items():
             # Check that each ref exists and that each commit exists
             if ref not in refs:
                 return True
@@ -566,8 +643,24 @@ class Repo(object):
             for dn in dirs:
                 if dn not in tree:
                     continue
+
+                # Some people like to keep playbooks, etc. grouped
+                # under their zuul config dirs; record the leading
+                # directories of any .zuul.ignore files and prune them
+                # from the config read.
+                to_ignore = []
                 for blob in tree[dn].traverse():
-                    if blob.path.endswith(".yaml"):
+                    if blob.path.endswith(".zuul.ignore"):
+                        to_ignore.append(os.path.split(blob.path)[0])
+
+                def _ignored(blob):
+                    for prefix in to_ignore:
+                        if blob.path.startswith(prefix):
+                            return True
+                    return False
+
+                for blob in tree[dn].traverse():
+                    if not _ignored(blob) and blob.path.endswith(".yaml"):
                         ret[blob.path] = blob.data_stream.read().decode(
                             'utf-8')
         return ret
@@ -634,38 +727,117 @@ class Repo(object):
         return branches
 
 
+class MergerTree:
+    """
+    A tree structure for quickly determining if a repo collides with
+    another in the same merger workspace.
+
+    Each node is a dictionary represents a path element.  The keys are
+    child path elements and their values are either another dictionary
+    for another node, or None if the child node is a git repo.
+    """
+
+    def __init__(self):
+        self.root = {}
+
+    def add(self, path):
+        parts = path.split('/')
+        root = self.root
+        for i, part in enumerate(parts[:-1]):
+            root = root.setdefault(part, {})
+            if root is None:
+                other = '/'.join(parts[:i])
+                raise Exception(f"Repo {path} collides with {other}")
+        last = parts[-1]
+        if last in root:
+            raise Exception(f"Repo {path} collides with "
+                            "an existing repo with a longer path")
+        root[last] = None
+
+
 class Merger(object):
-    def __init__(self, working_root, connections, email, username,
-                 speed_limit, speed_time, cache_root=None, logger=None,
-                 execution_context=False, git_timeout=300):
+
+    def __init__(
+        self,
+        working_root: str,
+        connections,
+        zk_client: ZooKeeperClient,
+        email: str,
+        username: str,
+        speed_limit: str,
+        speed_time: str,
+        cache_root: Optional[str] = None,
+        logger: Optional[Logger] = None,
+        execution_context: bool = False,
+        git_timeout: int = 300,
+        scheme: str = None,
+        cache_scheme: str = None,
+    ):
         self.logger = logger
         if logger is None:
             self.log = logging.getLogger("zuul.Merger")
         else:
             self.log = logger
-        self.repos = {}
+        self.repos = {}  # type: ignore
         self.working_root = working_root
         os.makedirs(working_root, exist_ok=True)
         self.connections = connections
+        self.zk_client = zk_client
         self.email = email
         self.username = username
         self.speed_limit = speed_limit
         self.speed_time = speed_time
         self.git_timeout = git_timeout
         self.cache_root = cache_root
+        self.scheme = scheme or zuul.model.SCHEME_UNIQUE
+        self.cache_scheme = cache_scheme or zuul.model.SCHEME_UNIQUE
         # Flag to determine if the merger is used for preparing repositories
         # for job execution. This flag can be used to enable executor specific
         # behavior e.g. to keep the 'origin' remote intact.
         self.execution_context = execution_context
+        # A tree of repos in our working root for detecting collisions
+        self.repo_roots = MergerTree()
+        # If we're not in an execution context, then check to see if
+        # our working root needs a "schema" upgrade.
+        if not execution_context:
+            self._upgradeRootScheme()
 
-    def _addProject(self, hostname, project_name, url, sshkey, zuul_event_id):
+    def _upgradeRootScheme(self):
+        # If the scheme for the root directory has changed, delete all
+        # the repos so they can be re-cloned.
+        try:
+            with open(os.path.join(self.working_root,
+                                   '.zuul_merger_scheme')) as f:
+                scheme = f.read().strip()
+        except FileNotFoundError:
+            # The previous default was golang
+            scheme = zuul.model.SCHEME_GOLANG
+        if scheme == self.scheme:
+            return
+        if os.listdir(self.working_root):
+            self.log.info(f"Existing merger scheme {scheme} does not match "
+                          f"requested scheme {self.scheme}, deleting merger "
+                          "root (repositories will be re-cloned).")
+            shutil.rmtree(self.working_root)
+            os.makedirs(self.working_root)
+        with open(os.path.join(self.working_root,
+                               '.zuul_merger_scheme'), 'w') as f:
+            f.write(self.scheme)
+
+    def _addProject(self, hostname, connection_name, project_name, url, sshkey,
+                    zuul_event_id, process_worker=None):
         repo = None
         key = '/'.join([hostname, project_name])
         try:
-            path = os.path.join(self.working_root, hostname, project_name)
+            path = os.path.join(self.working_root,
+                                strings.workspace_project_path(
+                                    hostname, project_name, self.scheme))
+            self.repo_roots.add(path)
             if self.cache_root:
-                cache_path = os.path.join(self.cache_root, hostname,
-                                          project_name)
+                cache_path = os.path.join(
+                    self.cache_root,
+                    strings.workspace_project_path(
+                        hostname, project_name, self.cache_scheme))
             else:
                 cache_path = None
             repo = Repo(
@@ -681,7 +853,8 @@ class Merger(object):
                           hostname, project_name)
         return repo
 
-    def getRepo(self, connection_name, project_name, zuul_event_id=None):
+    def getRepo(self, connection_name, project_name,
+                zuul_event_id=None, process_worker=None):
         source = self.connections.getSource(connection_name)
         project = source.getProject(project_name)
         hostname = project.canonical_hostname
@@ -697,38 +870,66 @@ class Merger(object):
             raise Exception("Unable to set up repo for project %s/%s"
                             " without a url" %
                             (connection_name, project_name,))
-        return self._addProject(hostname, project_name, url, sshkey,
-                                zuul_event_id)
+        return self._addProject(hostname, connection_name, project_name, url,
+                                sshkey, zuul_event_id,
+                                process_worker=process_worker)
 
     def updateRepo(self, connection_name, project_name, repo_state=None,
                    zuul_event_id=None,
                    build=None, process_worker=None):
+        """Fetch from origin if needed
+
+        If repo_state is None, then this will always git fetch.
+        If repo_state is provided, then this may no-op if
+        the shas specified by repo_state are already present.
+        """
+
         log = get_annotated_logger(self.log, zuul_event_id, build=build)
         repo = self.getRepo(connection_name, project_name,
                             zuul_event_id=zuul_event_id)
-        try:
+        if repo_state:
+            projects = repo_state.get(connection_name, {})
+            project_repo_state = projects.get(project_name, None)
+        else:
+            project_repo_state = None
 
-            # Check if we need an update if we got a repo_state
-            if repo_state and not repo.isUpdateNeeded(
-                    repo_state, zuul_event_id=zuul_event_id):
+        try:
+            # Check if we need an update if we got a repo_state and
+            # our project appears in it (otherwise we always update).
+            if project_repo_state and not repo.isUpdateNeeded(
+                    project_repo_state, zuul_event_id=zuul_event_id):
                 log.info("Skipping updating local repository %s/%s",
                          connection_name, project_name)
             else:
                 log.info("Updating local repository %s/%s",
                          connection_name, project_name)
-                repo.reset(zuul_event_id=zuul_event_id, build=build,
-                           process_worker=process_worker)
+                repo.update(zuul_event_id=zuul_event_id, build=build)
         except Exception:
             log.exception("Unable to update %s/%s",
                           connection_name, project_name)
+            raise
 
     def checkoutBranch(self, connection_name, project_name, branch,
-                       zuul_event_id=None):
+                       repo_state=None, zuul_event_id=None,
+                       process_worker=None):
+        """Check out a branch
+
+        Call Merger.updateRepo() first.  This does not reset the repo,
+        and is expected to be called only after a fresh clone.
+
+        """
         log = get_annotated_logger(self.log, zuul_event_id)
         log.info("Checking out %s/%s branch %s",
                  connection_name, project_name, branch)
         repo = self.getRepo(connection_name, project_name,
+                            process_worker=process_worker,
                             zuul_event_id=zuul_event_id)
+        # We don't need to reset because this is only called by the
+        # executor after a clone.
+        if repo_state:
+            self._restoreRepoState(connection_name, project_name, repo,
+                                   repo_state, zuul_event_id,
+                                   process_worker=process_worker)
         repo.checkout(branch, zuul_event_id=zuul_event_id)
 
     def _saveRepoState(self, connection_name, project_name, repo,
@@ -778,8 +979,10 @@ class Merger(object):
                 Repo.setRefsAsync, repo.local_path, project,
                 keep_remotes=self.execution_context)
             messages = job.result()
+            ref_log = get_annotated_logger(
+                logging.getLogger("zuul.Repo.Ref"), zuul_event_id)
             for message in messages:
-                log.debug(message)
+                ref_log.debug(message)
 
     def _mergeChange(self, item, ref, zuul_event_id):
         log = get_annotated_logger(self.log, zuul_event_id)
@@ -836,7 +1039,8 @@ class Merger(object):
             # we need to reset here in order to call getBranchHead
             log.debug("No base commit found for %s" % (key,))
             try:
-                repo.reset(zuul_event_id=zuul_event_id)
+                repo.reset(zuul_event_id=zuul_event_id,
+                           process_worker=process_worker)
             except Exception:
                 log.exception("Unable to reset repo %s" % repo)
                 return None, None
@@ -865,11 +1069,27 @@ class Merger(object):
             return None, None
         # Store this commit as the most recent for this project-branch
         recent[key] = commit
+
+        # Make sure to have a local ref that points to the  most recent
+        # (intermediate) speculative state of a branch, so commits are not
+        # garbage collected. The branch name is hashed to not cause any
+        # problems with empty directories in case of branch names containing
+        # slashes. In order to prevent issues with Git garbage collection
+        # between merger and executor jobs, we create refs in "refs/zuul"
+        # instead of updating local branch heads.
+        repo.setRef(Repo.refNameToZuulRef(item["branch"]),
+                    commit.hexsha, zuul_event_id=zuul_event_id)
+
         return orig_commit, commit
 
     def mergeChanges(self, items, files=None, dirs=None, repo_state=None,
                      repo_locks=None, branches=None, zuul_event_id=None,
                      process_worker=None):
+        """Merge changes
+
+        Call Merger.updateRepo() first.
+        """
+        # _mergeItem calls reset as necessary.
         log = get_annotated_logger(self.log, zuul_event_id)
         # connection+project+branch -> commit
         recent = {}
@@ -907,48 +1127,51 @@ class Merger(object):
             ret_recent[k] = v.hexsha
         return commit.hexsha, read_files, repo_state, ret_recent, orig_commit
 
-    def setRepoState(self, items, repo_state, zuul_event_id=None,
-                     process_worker=None):
-        # Sets the repo state for the items
-        seen = set()
-        for item in items:
-            repo = self.getRepo(item['connection'], item['project'],
-                                zuul_event_id=zuul_event_id)
-            key = (item['connection'], item['project'], item['branch'])
+    def setRepoState(self, connection_name, project_name, repo_state,
+                     zuul_event_id=None, process_worker=None):
+        """Set the repo state
 
-            if key in seen:
-                continue
+        Call Merger.updateRepo() first.
+        """
+        repo = self.getRepo(connection_name, project_name,
+                            zuul_event_id=zuul_event_id)
 
-            repo.reset(zuul_event_id=zuul_event_id,
-                       process_worker=process_worker)
-            self._restoreRepoState(item['connection'], item['project'], repo,
-                                   repo_state, zuul_event_id)
+        # TODO: why is reset required here?
+        repo.reset(zuul_event_id=zuul_event_id,
+                   process_worker=process_worker)
+
+        self._restoreRepoState(connection_name, project_name, repo,
+                               repo_state, zuul_event_id)
+
+    def getRepoState(self, items, repo_locks, branches=None):
+        """Gets the repo state for items.
+
+        This will perform repo updates as needed, so there is no need
+        to call Merger.updateRepo() first.
+
+        """
+        # Generally this will be called in any non-change pipeline.  We
+        # will return the repo state for each item, but manipulated with
+        # any information in the item (eg, if it creates a ref, that
+        # will be in the repo state regardless of the actual state).
 
-    def getRepoState(self, items, branches=None, repo_locks=None):
-        # Gets the repo state for items.  Generally this will be
-        # called in any non-change pipeline.  We will return the repo
-        # state for each item, but manipulated with any information in
-        # the item (eg, if it creates a ref, that will be in the repo
-        # state regardless of the actual state).
         seen = set()
         recent = {}
         repo_state = {}
         # A list of branch names the last item appears in.
         item_in_branches = []
         for item in items:
-            # If we're in the executor context we have the repo_locks object
-            # and perform per repo locking.
-            if repo_locks is not None:
-                lock = repo_locks.getRepoLock(
-                    item['connection'], item['project'])
-            else:
-                lock = nullcontext()
+            # If we're in the executor context we need to lock the repo.
+            # If not repo_locks will give us a fake lock.
+            lock = repo_locks.getRepoLock(item['connection'], item['project'])
             with lock:
                 repo = self.getRepo(item['connection'], item['project'])
-                key = (item['connection'], item['project'], item['branch'])
+                key = (item['connection'], item['project'])
                 if key not in seen:
                     try:
+                        repo.update()
                         repo.reset()
+                        seen.add(key)
                     except Exception:
                         self.log.exception("Unable to reset repo %s" % repo)
                         return (False, {}, [])
@@ -964,15 +1187,42 @@ class Merger(object):
                         item['ref'], item['newrev'])
         item = items[-1]
         repo = self.getRepo(item['connection'], item['project'])
-        item_in_branches = repo.contains(item['newrev'])
+        item_in_branches = False
+        if item.get('newrev'):
+            item_in_branches = repo.contains(item['newrev'])
         return (True, repo_state, item_in_branches)
 
     def getFiles(self, connection_name, project_name, branch, files, dirs=[]):
+        """Get file contents on branch.
+
+        Call Merger.updateRepo() first to make sure the repo is up to
+        date.
+
+        """
+        # We don't update the repo so that it can happen outside the
+        # lock.
         repo = self.getRepo(connection_name, project_name)
+        # TODO: why is reset required here?
+        repo.reset()
+        # This does not fetch, update, or reset, it operates on the
+        # working state.
         return repo.getFiles(files, dirs, branch=branch)
 
     def getFilesChanges(self, connection_name, project_name, branch,
                         tosha=None, zuul_event_id=None):
+        """Get a list of files changed in one or more commits
+
+        Gets files changed between tosha and branch (or just the
+        commit on branch if tosha is not specified).
+
+        Call Merger.updateRepo() first to make sure the repo is up to
+        date.
+        """
+        # Note, the arguments to this method should be reworked.  We
+        # fetch branch, and therefore it is typically actually a
+        # change ref.  tosha is typically the branch name.
         repo = self.getRepo(connection_name, project_name,
                             zuul_event_id=zuul_event_id)
+        # This performs a fetch, and therefore update/reset are not
+        # required.
         return repo.getFilesChanges(branch, tosha, zuul_event_id=zuul_event_id)
