@@ -14,14 +14,20 @@
 
 import json
 import logging
+import os
+import socket
 import threading
 from abc import ABCMeta
+from configparser import ConfigParser
+
+from zuul.zk import ZooKeeperClient
 
 from zuul.lib import commandsocket
 from zuul.lib.config import get_default
 from zuul.lib.gearworker import ZuulGearWorker
 from zuul.merger import merger
 from zuul.merger.merger import nullcontext
+from zuul.zk.components import MergerComponent
 
 COMMANDS = ['stop', 'pause', 'unpause']
 
@@ -48,10 +54,17 @@ class BaseMergeServer(metaclass=ABCMeta):
 
     _repo_locks_class = BaseRepoLocks
 
-    def __init__(self, config, component, connections=None):
-        self.connections = connections or {}
-        self.merge_email = get_default(config, 'merger', 'git_user_email')
-        self.merge_name = get_default(config, 'merger', 'git_user_name')
+    def __init__(
+        self,
+        config: ConfigParser,
+        component: str,
+        connections,
+    ):
+        self.connections = connections
+        self.merge_email = get_default(config, 'merger', 'git_user_email',
+                                       'zuul.merger.default@example.com')
+        self.merge_name = get_default(config, 'merger', 'git_user_name',
+                                      'Zuul Merger Default')
         self.merge_speed_limit = get_default(
             config, 'merger', 'git_http_low_speed_limit', '1000')
         self.merge_speed_time = get_default(
@@ -61,13 +74,17 @@ class BaseMergeServer(metaclass=ABCMeta):
         self.merge_root = get_default(config, component, 'git_dir',
                                       '/var/lib/zuul/{}-git'.format(component))
 
+        self.config = config
+
+        self.zk_client = ZooKeeperClient.fromConfig(self.config)
+        self.zk_client.connect()
+
         # This merger and its git repos are used to maintain
         # up-to-date copies of all the repos that are used by jobs, as
         # well as to support the merger:cat functon to supply
         # configuration information to Zuul when it starts.
-        self.merger = self._getMerger(self.merge_root, None)
-
-        self.config = config
+        self.merger = self._getMerger(self.merge_root, None,
+                                      execution_context=False)
 
         # Repo locking is needed on the executor
         self.repo_locks = self._repo_locks_class()
@@ -85,27 +102,61 @@ class BaseMergeServer(metaclass=ABCMeta):
             self.config,
             self.merger_jobs)
 
-    def _getMerger(self, root, cache_root, logger=None):
+    def _getMerger(self, root, cache_root, logger=None,
+                   execution_context=True, scheme=None,
+                   cache_scheme=None):
         return merger.Merger(
-            root, self.connections, self.merge_email, self.merge_name,
-            self.merge_speed_limit, self.merge_speed_time, cache_root, logger,
-            execution_context=True, git_timeout=self.git_timeout)
+            root,
+            self.connections,
+            self.zk_client,
+            self.merge_email,
+            self.merge_name,
+            self.merge_speed_limit,
+            self.merge_speed_time,
+            cache_root,
+            logger,
+            execution_context=execution_context,
+            git_timeout=self.git_timeout,
+            scheme=scheme,
+            cache_scheme=cache_scheme,
+        )
 
     def _repoLock(self, connection_name, project_name):
         # The merger does not need locking so return a null lock.
         return nullcontext()
 
     def _update(self, connection_name, project_name, zuul_event_id=None):
+        # The executor overrides _update so it can do the update
+        # asynchronously.
         self.merger.updateRepo(connection_name, project_name,
                                zuul_event_id=zuul_event_id)
 
     def start(self):
         self.log.debug('Starting merger worker')
+        self.log.debug('Cleaning any stale git index.lock files')
+        for (dirpath, dirnames, filenames) in os.walk(self.merge_root):
+            if '.git' in dirnames:
+                # Only recurse into .git dirs
+                dirnames.clear()
+                dirnames.append('.git')
+            elif dirpath.endswith('/.git'):
+                # Recurse no further
+                dirnames.clear()
+                if 'index.lock' in filenames:
+                    fp = os.path.join(dirpath, 'index.lock')
+                    try:
+                        os.unlink(fp)
+                        self.log.debug('Removed stale git lock: %s' % fp)
+                    except Exception:
+                        self.log.exception(
+                            'Unable to remove stale git lock: '
+                            '%s this may result in failed merges' % fp)
         self.merger_gearworker.start()
 
     def stop(self):
         self.log.debug('Stopping merger worker')
         self.merger_gearworker.stop()
+        self.zk_client.disconnect()
 
     def join(self):
         self.merger_gearworker.join()
@@ -124,15 +175,19 @@ class BaseMergeServer(metaclass=ABCMeta):
 
         connection_name = args['connection']
         project_name = args['project']
-        self._update(connection_name, project_name)
 
         lock = self.repo_locks.getRepoLock(connection_name, project_name)
-        with lock:
-            files = self.merger.getFiles(connection_name, project_name,
-                                         args['branch'], args['files'],
-                                         args.get('dirs'))
-        result = dict(updated=True,
-                      files=files)
+        try:
+            self._update(connection_name, project_name)
+            with lock:
+                files = self.merger.getFiles(connection_name, project_name,
+                                             args['branch'], args['files'],
+                                             args.get('dirs'))
+        except Exception:
+            result = dict(update=False)
+        else:
+            result = dict(updated=True, files=files)
+
         job.sendWorkComplete(json.dumps(result))
 
     def merge(self, job):
@@ -140,6 +195,8 @@ class BaseMergeServer(metaclass=ABCMeta):
         args = json.loads(job.arguments)
         zuul_event_id = args.get('zuul_event_id')
 
+        for item in args['items']:
+            self._update(item['connection'], item['project'])
         ret = self.merger.mergeChanges(
             args['items'], args.get('files'),
             args.get('dirs', []),
@@ -163,8 +220,7 @@ class BaseMergeServer(metaclass=ABCMeta):
         zuul_event_id = args.get('zuul_event_id')
         success, repo_state, item_in_branches = \
             self.merger.getRepoState(
-                args['items'], branches=args.get('branches'),
-                repo_locks=self.repo_locks)
+                args['items'], self.repo_locks, branches=args.get('branches'))
         result = dict(updated=success,
                       repo_state=repo_state,
                       item_in_branches=item_in_branches)
@@ -178,16 +234,21 @@ class BaseMergeServer(metaclass=ABCMeta):
 
         connection_name = args['connection']
         project_name = args['project']
-        self._update(connection_name, project_name,
-                     zuul_event_id=zuul_event_id)
 
         lock = self.repo_locks.getRepoLock(connection_name, project_name)
-        with lock:
-            files = self.merger.getFilesChanges(
-                connection_name, project_name, args['branch'], args['tosha'],
-                zuul_event_id=zuul_event_id)
-        result = dict(updated=True,
-                      files=files)
+        try:
+            self._update(connection_name, project_name,
+                         zuul_event_id=zuul_event_id)
+            with lock:
+                files = self.merger.getFilesChanges(
+                    connection_name, project_name,
+                    args['branch'], args['tosha'],
+                    zuul_event_id=zuul_event_id)
+        except Exception:
+            result = dict(update=False)
+        else:
+            result = dict(updated=True, files=files)
+
         result['zuul_event_id'] = zuul_event_id
         job.sendWorkComplete(json.dumps(result))
 
@@ -195,8 +256,15 @@ class BaseMergeServer(metaclass=ABCMeta):
 class MergeServer(BaseMergeServer):
     log = logging.getLogger("zuul.MergeServer")
 
-    def __init__(self, config, connections=None):
+    def __init__(
+        self,
+        config: ConfigParser,
+        connections,
+    ):
         super().__init__(config, 'merger', connections)
+        self.hostname = socket.getfqdn()
+        self.component_info = MergerComponent(self.zk_client, self.hostname)
+        self.component_info.register()
 
         self.command_map = dict(
             stop=self.stop,
@@ -219,9 +287,11 @@ class MergeServer(BaseMergeServer):
             target=self.runCommand, name='command')
         self.command_thread.daemon = True
         self.command_thread.start()
+        self.component_info.state = self.component_info.RUNNING
 
     def stop(self):
         self.log.debug("Stopping")
+        self.component_info.state = self.component_info.STOPPED
         super().stop()
         self._command_running = False
         self.command_socket.stop()
@@ -232,11 +302,13 @@ class MergeServer(BaseMergeServer):
 
     def pause(self):
         self.log.debug('Pausing')
+        self.component_info.state = self.component_info.PAUSED
         super().pause()
 
     def unpause(self):
         self.log.debug('Resuming')
         super().unpause()
+        self.component_info.state = self.component_info.RUNNING
 
     def runCommand(self):
         while self._command_running:
