@@ -175,8 +175,16 @@ class BaseReporter(object, metaclass=abc.ABCMeta):
         return msg
 
     def _formatItemReportFailure(self, item, with_jobs=True):
-        if item.dequeued_needing_change:
+        if item.cannotMergeBundle():
+            msg = 'This change is part of a bundle that failed to merge.\n'
+        elif item.dequeued_needing_change:
             msg = 'This change depends on a change that failed to merge.\n'
+        elif item.isBundleFailing():
+            msg = 'This change is part of a bundle that failed.\n'
+            if with_jobs:
+                msg = '{}\n\n{}'.format(msg, self._formatItemReportJobs(item))
+            msg = "{}\n\n{}".format(
+                msg, self._formatItemReportOtherBundleItems(item))
         elif item.didMergerFail():
             msg = item.pipeline.merge_failure_message
         elif item.getConfigErrors():
@@ -215,6 +223,10 @@ class BaseReporter(object, metaclass=abc.ABCMeta):
             msg += '\n\n' + self._formatItemReportJobs(item)
         return msg
 
+    def _formatItemReportOtherBundleItems(self, item):
+        return "Related changes:\n{}".format("\n".join(
+            c.url for c in item.change.needs_changes if c is not item.change))
+
     def _getItemReportJobsFields(self, item):
         # Extract the report elements from an item
         config = self.connection.sched.config
@@ -251,6 +263,12 @@ class BaseReporter(object, metaclass=abc.ABCMeta):
             else:
                 error = ''
             name = job.name + ' '
+            # TODO(mordred) The gerrit consumption interface depends on
+            # something existing in the url field and don't have a great
+            # behavior defined for url being none/missing. Put name into
+            # the url field to match old behavior until we can deal with
+            # the gerrit-side piece as well
+            url = url or job.name
             jobs_fields.append((name, url, result, error, elapsed, voting))
         return jobs_fields
 
