@@ -16,6 +16,10 @@
 import json
 import logging
 import time
+from abc import ABCMeta
+from typing import List
+
+import zuul.executor.common
 
 from zuul import model
 from zuul.connection import BaseConnection
@@ -24,48 +28,24 @@ from zuul.lib.gearworker import ZuulGearWorker
 from zuul.lib.jsonutil import ZuulJSONEncoder
 
 
-class RPCListener(object):
-    log = logging.getLogger("zuul.RPCListener")
+class RPCListenerBase(metaclass=ABCMeta):
+    log = logging.getLogger("zuul.RPCListenerBase")
+    thread_name = 'zuul-rpc-gearman-worker'
+    functions = []  # type: List[str]
 
     def __init__(self, config, sched):
         self.config = config
         self.sched = sched
 
         self.jobs = {}
-        functions = [
-            'autohold',
-            'autohold_delete',
-            'autohold_info',
-            'autohold_list',
-            'allowed_labels_get',
-            'dequeue',
-            'enqueue',
-            'enqueue_ref',
-            'promote',
-            'get_admin_tenants',
-            'get_running_jobs',
-            'get_job_log_stream_address',
-            'tenant_list',
-            'tenant_sql_connection',
-            'status_get',
-            'job_get',
-            'job_list',
-            'project_get',
-            'project_list',
-            'project_freeze_jobs',
-            'pipeline_list',
-            'key_get',
-            'config_errors_list',
-            'connection_list',
-            'authorize_user',
-        ]
-        for func in functions:
+
+        for func in self.functions:
             f = getattr(self, 'handle_%s' % func)
             self.jobs['zuul:%s' % func] = f
         self.gearworker = ZuulGearWorker(
             'Zuul RPC Listener',
-            'zuul.RPCListener',
-            'zuul-rpc-gearman-worker',
+            self.log.name,
+            self.thread_name,
             self.config,
             self.jobs)
 
@@ -80,6 +60,17 @@ class RPCListener(object):
     def join(self):
         self.gearworker.join()
 
+
+class RPCListenerSlow(RPCListenerBase):
+    log = logging.getLogger("zuul.RPCListenerSlow")
+    thread_name = 'zuul-rpc-slow-gearman-worker'
+    functions = [
+        'dequeue',
+        'enqueue',
+        'enqueue_ref',
+        'promote',
+    ]
+
     def handle_dequeue(self, job):
         args = json.loads(job.arguments)
         tenant_name = args['tenant']
@@ -95,6 +86,97 @@ class RPCListener(object):
             return
         job.sendWorkComplete()
 
+    def _common_enqueue(self, job, args):
+        tenant_name = args['tenant']
+        pipeline_name = args['pipeline']
+        project_name = args['project']
+        change = args.get('change')
+        ref = args.get('ref')
+        oldrev = args.get('oldrev')
+        newrev = args.get('newrev')
+        try:
+            self.sched.enqueue(tenant_name, pipeline_name, project_name,
+                               change, ref, oldrev, newrev)
+        except Exception as e:
+            job.sendWorkException(str(e).encode('utf8'))
+            return
+
+        job.sendWorkComplete()
+
+    def handle_enqueue(self, job):
+        args = json.loads(job.arguments)
+        self._common_enqueue(job, args)
+
+    def handle_enqueue_ref(self, job):
+        args = json.loads(job.arguments)
+        oldrev = args['oldrev']
+        newrev = args['newrev']
+        errors = ''
+        try:
+            int(oldrev, 16)
+            if len(oldrev) != 40:
+                errors += f'Old rev must be 40 character sha1: {oldrev}\n'
+        except Exception:
+            errors += f'Old rev must be base16 hash: {oldrev}\n'
+        try:
+            int(newrev, 16)
+            if len(newrev) != 40:
+                errors += f'New rev must be 40 character sha1: {newrev}\n'
+        except Exception:
+            errors += f'New rev must be base16 hash: {newrev}\n'
+
+        if errors:
+            job.sendWorkException(errors.encode('utf8'))
+        else:
+            self._common_enqueue(job, args)
+
+    def handle_promote(self, job):
+        args = json.loads(job.arguments)
+        tenant_name = args['tenant']
+        pipeline_name = args['pipeline']
+        change_ids = args['change_ids']
+        self.sched.promote(tenant_name, pipeline_name, change_ids)
+        job.sendWorkComplete()
+
+
+class RPCListener(RPCListenerBase):
+    log = logging.getLogger("zuul.RPCListener")
+    thread_name = 'zuul-rpc-gearman-worker'
+    functions = [
+        'autohold',
+        'autohold_delete',
+        'autohold_info',
+        'autohold_list',
+        'allowed_labels_get',
+        'get_admin_tenants',
+        'get_running_jobs',
+        'get_job_log_stream_address',
+        'tenant_list',
+        'status_get',
+        'job_get',
+        'job_list',
+        'project_get',
+        'project_list',
+        'project_freeze_jobs',
+        'project_freeze_job',
+        'pipeline_list',
+        'key_get',
+        'config_errors_list',
+        'connection_list',
+        'authorize_user',
+    ]
+
+    def start(self):
+        self.gearworker.start()
+
+    def stop(self):
+        self.log.debug("Stopping")
+        self.gearworker.stop()
+        self.log.debug("Stopped")
+
+    def join(self):
+        self.gearworker.join()
+
     def handle_autohold_info(self, job):
         args = json.loads(job.arguments)
         request_id = args['request_id']
@@ -166,93 +248,6 @@ class RPCListener(object):
         self.sched.autohold(**params)
         job.sendWorkComplete()
 
-    def _common_enqueue(self, job):
-        args = json.loads(job.arguments)
-        event = model.TriggerEvent()
-        event.timestamp = time.time()
-        errors = ''
-        tenant = None
-        project = None
-        pipeline = None
-
-        tenant = self.sched.abide.tenants.get(args['tenant'])
-        if tenant:
-            event.tenant_name = args['tenant']
-
-            (trusted, project) = tenant.getProject(args['project'])
-            if project:
-                event.project_hostname = project.canonical_hostname
-                event.project_name = project.name
-            else:
-                errors += 'Invalid project: %s\n' % (args['project'],)
-
-            pipeline = tenant.layout.pipelines.get(args['pipeline'])
-            if pipeline:
-                event.forced_pipeline = args['pipeline']
-            else:
-                errors += 'Invalid pipeline: %s\n' % (args['pipeline'],)
-        else:
-            errors += 'Invalid tenant: %s\n' % (args['tenant'],)
-
-        return (args, event, errors, project)
-
-    def handle_enqueue(self, job):
-        (args, event, errors, project) = self._common_enqueue(job)
-
-        if not errors:
-            event.change_number, event.patch_number = args['change'].split(',')
-            try:
-                ch = project.source.getChange(event, refresh=True)
-                if ch.project.name != project.name:
-                    errors += ('Change %s does not belong to project "%s", '
-                               % (args['change'], project.name))
-            except Exception:
-                errors += 'Invalid change: %s\n' % (args['change'],)
-
-        if errors:
-            job.sendWorkException(errors.encode('utf8'))
-        else:
-            self.sched.enqueue(event)
-            job.sendWorkComplete()
-
-    def handle_enqueue_ref(self, job):
-        (args, event, errors, project) = self._common_enqueue(job)
-
-        if not errors:
-            event.ref = args['ref']
-            event.oldrev = args['oldrev']
-            event.newrev = args['newrev']
-            try:
-                int(event.oldrev, 16)
-                if len(event.oldrev) != 40:
-                    errors += 'Old rev must be 40 character sha1: ' \
-                              '%s\n' % event.oldrev
-            except Exception:
-                errors += 'Old rev must be base16 hash: ' \
-                          '%s\n' % event.oldrev
-            try:
-                int(event.newrev, 16)
-                if len(event.newrev) != 40:
-                    errors += 'New rev must be 40 character sha1: ' \
-                              '%s\n' % event.newrev
-            except Exception:
-                errors += 'New rev must be base16 hash: ' \
-                          '%s\n' % event.newrev
-
-        if errors:
-            job.sendWorkException(errors.encode('utf8'))
-        else:
-            self.sched.enqueue(event)
-            job.sendWorkComplete()
-
-    def handle_promote(self, job):
-        args = json.loads(job.arguments)
-        tenant_name = args['tenant']
-        pipeline_name = args['pipeline']
-        change_ids = args['change_ids']
-        self.sched.promote(tenant_name, pipeline_name, change_ids)
-        job.sendWorkComplete()
-
     def handle_get_running_jobs(self, job):
         # args = json.loads(job.arguments)
         # TODO: use args to filter by pipeline etc
@@ -348,19 +343,14 @@ class RPCListener(object):
                            'queue': queue_size})
         job.sendWorkComplete(json.dumps(output))
 
-    def handle_tenant_sql_connection(self, job):
-        args = json.loads(job.arguments)
-        sql_driver = self.sched.connections.drivers['sql']
-        conn = sql_driver.tenant_connections.get(args['tenant'])
-        if conn:
-            name = conn.connection_name
-        else:
-            name = ''
-        job.sendWorkComplete(json.dumps(name))
-
     def handle_status_get(self, job):
         args = json.loads(job.arguments)
+        start = time.monotonic()
         output = self.sched.formatStatusJSON(args.get("tenant"))
+        end = time.monotonic()
+        self.log.debug('Formatting tenant %s status took %.3f seconds for '
+                       '%d bytes', args.get("tenant"), end - start,
+                       len(output))
         job.sendWorkComplete(output)
 
     def handle_job_get(self, gear_job):
@@ -479,8 +469,7 @@ class RPCListener(object):
         change.branch = args.get("branch", "master")
         queue = model.ChangeQueue(pipeline)
         item = model.QueueItem(queue, change, None)
-        item.layout = tenant.layout
-        item.freezeJobGraph(skip_file_matcher=True)
+        item.freezeJobGraph(tenant.layout, skip_file_matcher=True)
 
         output = []
 
@@ -494,6 +483,37 @@ class RPCListener(object):
 
         gear_job.sendWorkComplete(json.dumps(output))
 
+    def handle_project_freeze_job(self, gear_job):
+        args = json.loads(gear_job.arguments)
+        tenant = self.sched.abide.tenants.get(args.get("tenant"))
+        project = None
+        pipeline = None
+        if tenant:
+            (trusted, project) = tenant.getProject(args.get("project"))
+            pipeline = tenant.layout.pipelines.get(args.get("pipeline"))
+        if not project or not pipeline:
+            gear_job.sendWorkComplete(json.dumps(None))
+            return
+
+        change = model.Branch(project)
+        change.branch = args.get("branch", "master")
+        queue = model.ChangeQueue(pipeline)
+        item = model.QueueItem(queue, change, None)
+        item.freezeJobGraph(tenant.layout, skip_file_matcher=True)
+
+        job = item.job_graph.jobs.get(args.get("job"))
+        if not job:
+            gear_job.sendWorkComplete(json.dumps(None))
+            return
+        # TODO: check if this is frozen?
+        nodeset = job.nodeset
+        job.setBase(tenant.layout)
+        uuid = '0' * 32
+        params = zuul.executor.common.construct_gearman_params(
+            uuid, self.sched, nodeset,
+            job, item, pipeline)
+        gear_job.sendWorkComplete(json.dumps(params, cls=ZuulJSONEncoder))
+
     def handle_allowed_labels_get(self, job):
         args = json.loads(job.arguments)
         tenant = self.sched.abide.tenants.get(args.get("tenant"))
