@@ -12,10 +12,10 @@
 # implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
-
-
 import cherrypy
 import socket
+
+from cachetools.func import ttl_cache
 from ws4py.server.cherrypyserver import WebSocketPlugin, WebSocketTool
 from ws4py.websocket import WebSocket
 import codecs
@@ -28,13 +28,17 @@ import time
 import select
 import threading
 
+from zuul import exceptions
 import zuul.lib.repl
+from zuul.lib import commandsocket
 from zuul.lib.re2util import filter_allowed_disallowed
 import zuul.model
-from zuul import exceptions
 import zuul.rpcclient
-import zuul.zk
-from zuul.lib import commandsocket
+from zuul.zk import ZooKeeperClient
+from zuul.zk.components import WebComponent
+from zuul.zk.nodepool import ZooKeeperNodepool
+from zuul.lib.auth import AuthenticatorRegistry
+from zuul.lib.config import get_default
 
 STATIC_DIR = os.path.join(os.path.dirname(__file__), 'static')
 cherrypy.tools.websocket = WebSocketTool()
@@ -68,6 +72,30 @@ class SaveParamsTool(cherrypy.Tool):
 cherrypy.tools.save_params = SaveParamsTool()
 
 
+def handle_options(allowed_methods=None):
+    if cherrypy.request.method == 'OPTIONS':
+        methods = allowed_methods or ['GET', 'OPTIONS']
+        if allowed_methods and 'OPTIONS' not in allowed_methods:
+            methods = methods + ['OPTIONS']
+        # discard decorated handler
+        request = cherrypy.serving.request
+        request.handler = None
+        # Set CORS response headers
+        resp = cherrypy.response
+        resp.headers['Access-Control-Allow-Origin'] = '*'
+        resp.headers['Access-Control-Allow-Headers'] =\
+            ', '.join(['Authorization', 'Content-Type'])
+        resp.headers['Access-Control-Allow-Methods'] =\
+            ', '.join(methods)
+        # Allow caching of the preflight response
+        resp.headers['Access-Control-Max-Age'] = 86400
+        resp.status = 204
+
+
+cherrypy.tools.handle_options = cherrypy.Tool('on_start_resource',
+                                              handle_options)
+
+
 class ChangeFilter(object):
     def __init__(self, desired):
         self.desired = desired
@@ -203,7 +231,8 @@ class ZuulWebAPI(object):
 
     def __init__(self, zuulweb):
         self.rpc = zuulweb.rpc
-        self.zk = zuulweb.zk
+        self.zk_client = zuulweb.zk_client
+        self.zk_nodepool = ZooKeeperNodepool(self.zk_client)
         self.zuulweb = zuulweb
         self.cache = {}
         self.cache_time = {}
@@ -237,9 +266,25 @@ class ZuulWebAPI(object):
                 'error': e,
                 'realm': self.zuulweb.authenticators.default_realm}
 
+    def _auth_token_check(self):
+        rawToken = \
+            cherrypy.request.headers['Authorization'][len('Bearer '):]
+        try:
+            claims = self.zuulweb.authenticators.authenticate(rawToken)
+        except exceptions.AuthTokenException as e:
+            for header, contents in e.getAdditionalHeaders().items():
+                cherrypy.response.headers[header] = contents
+            cherrypy.response.status = e.HTTPError
+            return ({},
+                    {'description': e.error_description,
+                     'error': e.error,
+                     'realm': e.realm})
+        return (claims, None)
+
     @cherrypy.expose
     @cherrypy.tools.json_in()
     @cherrypy.tools.json_out(content_type='application/json; charset=utf-8')
+    @cherrypy.tools.handle_options(allowed_methods=['POST', ])
     def dequeue(self, tenant, project):
         basic_error = self._basic_auth_header_check()
         if basic_error is not None:
@@ -247,16 +292,9 @@ class ZuulWebAPI(object):
         if cherrypy.request.method != 'POST':
             raise cherrypy.HTTPError(405)
         # AuthN/AuthZ
-        rawToken = cherrypy.request.headers['Authorization'][len('Bearer '):]
-        try:
-            claims = self.zuulweb.authenticators.authenticate(rawToken)
-        except exceptions.AuthTokenException as e:
-            for header, contents in e.getAdditionalHeaders().items():
-                cherrypy.response.headers[header] = contents
-            cherrypy.response.status = e.HTTPError
-            return {'description': e.error_description,
-                    'error': e.error,
-                    'realm': e.realm}
+        claims, token_error = self._auth_token_check()
+        if token_error is not None:
+            return token_error
         self.is_authorized(claims, tenant)
         msg = 'User "%s" requesting "%s" on %s/%s'
         self.log.info(
@@ -284,6 +322,7 @@ class ZuulWebAPI(object):
     @cherrypy.expose
     @cherrypy.tools.json_in()
     @cherrypy.tools.json_out(content_type='application/json; charset=utf-8')
+    @cherrypy.tools.handle_options(allowed_methods=['POST', ])
     def enqueue(self, tenant, project):
         basic_error = self._basic_auth_header_check()
         if basic_error is not None:
@@ -291,16 +330,9 @@ class ZuulWebAPI(object):
         if cherrypy.request.method != 'POST':
             raise cherrypy.HTTPError(405)
         # AuthN/AuthZ
-        rawToken = cherrypy.request.headers['Authorization'][len('Bearer '):]
-        try:
-            claims = self.zuulweb.authenticators.authenticate(rawToken)
-        except exceptions.AuthTokenException as e:
-            for header, contents in e.getAdditionalHeaders().items():
-                cherrypy.response.headers[header] = contents
-            cherrypy.response.status = e.HTTPError
-            return {'description': e.error_description,
-                    'error': e.error,
-                    'realm': e.realm}
+        claims, token_error = self._auth_token_check()
+        if token_error is not None:
+            return token_error
         self.is_authorized(claims, tenant)
         msg = 'User "%s" requesting "%s" on %s/%s'
         self.log.info(
@@ -317,25 +349,23 @@ class ZuulWebAPI(object):
             raise cherrypy.HTTPError(400,
                                      'Invalid request body')
 
-    def _enqueue(self, tenant, project, trigger, change, pipeline, **kwargs):
+    def _enqueue(self, tenant, project, change, pipeline, **kwargs):
         job = self.rpc.submitJob('zuul:enqueue',
                                  {'tenant': tenant,
                                   'pipeline': pipeline,
                                   'project': project,
-                                  'trigger': trigger,
                                   'change': change, })
         result = not job.failure
         resp = cherrypy.response
         resp.headers['Access-Control-Allow-Origin'] = '*'
         return result
 
-    def _enqueue_ref(self, tenant, project, trigger, ref,
+    def _enqueue_ref(self, tenant, project, ref,
                      oldrev, newrev, pipeline, **kwargs):
         job = self.rpc.submitJob('zuul:enqueue_ref',
                                  {'tenant': tenant,
                                   'pipeline': pipeline,
                                   'project': project,
-                                  'trigger': trigger,
                                   'ref': ref,
                                   'oldrev': oldrev,
                                   'newrev': newrev, })
@@ -344,6 +374,42 @@ class ZuulWebAPI(object):
         resp.headers['Access-Control-Allow-Origin'] = '*'
         return result
 
+    @cherrypy.expose
+    @cherrypy.tools.json_in()
+    @cherrypy.tools.json_out(content_type='application/json; charset=utf-8')
+    @cherrypy.tools.handle_options(allowed_methods=['POST', ])
+    def promote(self, tenant):
+        basic_error = self._basic_auth_header_check()
+        if basic_error is not None:
+            return basic_error
+        if cherrypy.request.method != 'POST':
+            raise cherrypy.HTTPError(405)
+        # AuthN/AuthZ
+        claims, token_error = self._auth_token_check()
+        if token_error is not None:
+            return token_error
+        self.is_authorized(claims, tenant)
+
+        body = cherrypy.request.json
+        pipeline = body.get('pipeline')
+        changes = body.get('changes')
+
+        msg = 'User "%s" requesting "%s" on %s/%s'
+        self.log.info(
+            msg % (claims['__zuul_uid_claim'], 'promote',
+                   tenant, pipeline))
+
+        job = self.rpc.submitJob('zuul:promote',
+                                 {
+                                     'tenant': tenant,
+                                     'pipeline': pipeline,
+                                     'change_ids': changes,
+                                 })
+        result = not job.failure
+        resp = cherrypy.response
+        resp.headers['Access-Control-Allow-Origin'] = '*'
+        return result
+
     @cherrypy.expose
     @cherrypy.tools.json_out(content_type='application/json; charset=utf-8')
     def autohold_list(self, tenant, *args, **kwargs):
@@ -356,6 +422,7 @@ class ZuulWebAPI(object):
 
     @cherrypy.expose
     @cherrypy.tools.json_out(content_type='application/json; charset=utf-8')
+    @cherrypy.tools.handle_options(allowed_methods=['GET', 'POST', ])
     def autohold(self, tenant, project=None):
         # we don't use json_in because a payload is not mandatory with GET
         # Note: GET handling is redundant with autohold_list
@@ -367,17 +434,9 @@ class ZuulWebAPI(object):
             if basic_error is not None:
                 return basic_error
             # AuthN/AuthZ
-            rawToken = \
-                cherrypy.request.headers['Authorization'][len('Bearer '):]
-            try:
-                claims = self.zuulweb.authenticators.authenticate(rawToken)
-            except exceptions.AuthTokenException as e:
-                for header, contents in e.getAdditionalHeaders().items():
-                    cherrypy.response.headers[header] = contents
-                cherrypy.response.status = e.HTTPError
-                return {'description': e.error_description,
-                        'error': e.error,
-                        'realm': e.realm}
+            claims, token_error = self._auth_token_check()
+            if token_error is not None:
+                return token_error
             self.is_authorized(claims, tenant)
             msg = 'User "%s" requesting "%s" on %s/%s'
             self.log.info(
@@ -433,9 +492,12 @@ class ZuulWebAPI(object):
                              'project': request['project'],
                              'job': request['job'],
                              'ref_filter': request['ref_filter'],
-                             'count': request['max_count'],
+                             'max_count': request['max_count'],
+                             'current_count': request['current_count'],
                              'reason': request['reason'],
-                             'node_hold_expiration': request['node_expiration']
+                             'node_expiration': request['node_expiration'],
+                             'expired': request['expired'],
+                             'nodes': request['nodes']
                             })
             resp = cherrypy.response
             resp.headers['Access-Control-Allow-Origin'] = '*'
@@ -443,15 +505,16 @@ class ZuulWebAPI(object):
 
     @cherrypy.expose
     @cherrypy.tools.json_out(content_type='application/json; charset=utf-8')
+    @cherrypy.tools.handle_options(allowed_methods=['GET', 'DELETE', ])
     def autohold_by_request_id(self, tenant, request_id):
         if cherrypy.request.method == 'GET':
-            return self._autohold_info(request_id)
+            return self._autohold_info(tenant, request_id)
         elif cherrypy.request.method == 'DELETE':
-            return self._autohold_delete(request_id)
+            return self._autohold_delete(tenant, request_id)
         else:
             raise cherrypy.HTTPError(405)
 
-    def _autohold_info(self, request_id):
+    def _autohold_info(self, tenant, request_id):
         job = self.rpc.submitJob('zuul:autohold_info',
                                  {'request_id': request_id})
         if job.failure:
@@ -460,7 +523,11 @@ class ZuulWebAPI(object):
             request = json.loads(job.data[0])
             if not request:
                 raise cherrypy.HTTPError(
-                    404, 'Hold request %s does not exist.' % request_id)
+                    404, 'Hold request %s not found.' % request_id)
+            if tenant != request['tenant']:
+                # return 404 rather than 403 to avoid leaking tenant info
+                raise cherrypy.HTTPError(
+                    404, 'Hold request %s not found.' % request_id)
             resp = cherrypy.response
             resp.headers['Access-Control-Allow-Origin'] = '*'
             return {
@@ -469,29 +536,24 @@ class ZuulWebAPI(object):
                 'project': request['project'],
                 'job': request['job'],
                 'ref_filter': request['ref_filter'],
-                'count': request['max_count'],
+                'max_count': request['max_count'],
+                'current_count': request['current_count'],
                 'reason': request['reason'],
-                'node_hold_expiration': request['node_expiration']
+                'node_expiration': request['node_expiration'],
+                'expired': request['expired'],
+                'nodes': request['nodes']
             }
 
-    def _autohold_delete(self, request_id):
+    def _autohold_delete(self, tenant, request_id):
         # We need tenant info from the request for authz
-        request = self._autohold_info(request_id)
-
+        request = self._autohold_info(tenant, request_id)
         basic_error = self._basic_auth_header_check()
         if basic_error is not None:
             return basic_error
         # AuthN/AuthZ
-        rawToken = cherrypy.request.headers['Authorization'][len('Bearer '):]
-        try:
-            claims = self.zuulweb.authenticators.authenticate(rawToken)
-        except exceptions.AuthTokenException as e:
-            for header, contents in e.getAdditionalHeaders().items():
-                cherrypy.response.headers[header] = contents
-            cherrypy.response.status = e.HTTPError
-            return {'description': e.error_description,
-                    'error': e.error,
-                    'realm': e.realm}
+        claims, token_error = self._auth_token_check()
+        if token_error is not None:
+            return token_error
         self.is_authorized(claims, request['tenant'])
         msg = 'User "%s" requesting "%s" on %s/%s'
         self.log.info(
@@ -535,14 +597,19 @@ class ZuulWebAPI(object):
             'buildsets': '/api/tenant/{tenant}/buildsets',
             'buildset': '/api/tenant/{tenant}/buildset/{uuid}',
             'config_errors': '/api/tenant/{tenant}/config-errors',
+            # TODO(mhu) remove after next release
             'authorizations': '/api/user/authorizations',
+            'tenant_authorizations': ('/api/tenant/{tenant}'
+                                      '/authorizations'),
             'autohold': '/api/tenant/{tenant}/project/{project:.*}/autohold',
             'autohold_list': '/api/tenant/{tenant}/autohold',
-            'autohold_by_request_id': '/api/tenant/{tenant}/'
-                                      'autohold/{request_id}',
-            'autohold_delete': '/api/tenant/{tenant}/autohold/{request_id}',
+            'autohold_by_request_id': ('/api/tenant/{tenant}'
+                                       '/autohold/{request_id}'),
+            'autohold_delete': ('/api/tenant/{tenant}'
+                                '/autohold/{request_id}'),
             'enqueue': '/api/tenant/{tenant}/project/{project:.*}/enqueue',
             'dequeue': '/api/tenant/{tenant}/project/{project:.*}/dequeue',
+            'promote': '/api/tenant/{tenant}/promote',
         }
 
     @cherrypy.expose
@@ -583,13 +650,20 @@ class ZuulWebAPI(object):
             raise cherrypy.HTTPError(403)
         return user_authz
 
-    # TODO good candidate for caching
+    # TODO(mhu) deprecated, remove next version
     @cherrypy.expose
     @cherrypy.tools.json_out(content_type='application/json; charset=utf-8')
-    def user_authorizations(self):
-        rawToken = cherrypy.request.headers['Authorization'][len('Bearer '):]
+    @cherrypy.tools.handle_options(allowed_methods=['GET', ])
+    def authorizations(self):
+        basic_error = self._basic_auth_header_check()
+        if basic_error is not None:
+            return basic_error
+        # AuthN/AuthZ
+        claims, token_error = self._auth_token_check()
+        if token_error is not None:
+            return token_error
         try:
-            claims = self.zuulweb.authenticators.authenticate(rawToken)
+            admin_tenants = self._authorizations()
         except exceptions.AuthTokenException as e:
             for header, contents in e.getAdditionalHeaders().items():
                 cherrypy.response.headers[header] = contents
@@ -597,18 +671,51 @@ class ZuulWebAPI(object):
             return {'description': e.error_description,
                     'error': e.error,
                     'realm': e.realm}
+        return {'zuul': {'admin': admin_tenants}, }
+
+    @cherrypy.expose
+    @cherrypy.tools.json_out(content_type='application/json; charset=utf-8')
+    @cherrypy.tools.handle_options(allowed_methods=['GET', ])
+    def tenant_authorizations(self, tenant):
+        basic_error = self._basic_auth_header_check()
+        if basic_error is not None:
+            return basic_error
+        # AuthN/AuthZ
+        claims, token_error = self._auth_token_check()
+        if token_error is not None:
+            return token_error
+        try:
+            admin_tenants = self._authorizations()
+        except exceptions.AuthTokenException as e:
+            for header, contents in e.getAdditionalHeaders().items():
+                cherrypy.response.headers[header] = contents
+            cherrypy.response.status = e.HTTPError
+            return {'description': e.error_description,
+                    'error': e.error,
+                    'realm': e.realm}
+        return {'zuul': {'admin': tenant in admin_tenants,
+                         'scope': [tenant, ]}, }
+
+    # TODO good candidate for caching
+    def _authorizations(self):
+        rawToken = cherrypy.request.headers['Authorization'][len('Bearer '):]
+        claims = self.zuulweb.authenticators.authenticate(rawToken)
         if 'zuul' in claims and 'admin' in claims.get('zuul', {}):
             return {'zuul': {'admin': claims['zuul']['admin']}, }
         job = self.rpc.submitJob('zuul:get_admin_tenants',
                                  {'claims': claims})
         admin_tenants = json.loads(job.data[0])
-        return {'zuul': {'admin': admin_tenants}, }
+        return admin_tenants
+
+    @ttl_cache(ttl=60)
+    def _tenants(self):
+        job = self.rpc.submitJob('zuul:tenant_list', {})
+        return json.loads(job.data[0])
 
     @cherrypy.expose
     @cherrypy.tools.json_out(content_type='application/json; charset=utf-8')
     def tenants(self):
-        job = self.rpc.submitJob('zuul:tenant_list', {})
-        ret = json.loads(job.data[0])
+        ret = self._tenants()
         resp = cherrypy.response
         resp.headers['Access-Control-Allow-Origin'] = '*'
         return ret
@@ -751,7 +858,7 @@ class ZuulWebAPI(object):
             allowed_labels = data['allowed_labels']
             disallowed_labels = data['disallowed_labels']
         labels = set()
-        for launcher in self.zk.getRegisteredLaunchers():
+        for launcher in self.zk_nodepool.getRegisteredLaunchers():
             labels.update(filter_allowed_disallowed(
                 launcher.supported_labels,
                 allowed_labels, disallowed_labels))
@@ -765,7 +872,7 @@ class ZuulWebAPI(object):
     @cherrypy.tools.json_out(content_type='application/json; charset=utf-8')
     def nodes(self, tenant):
         ret = []
-        for node in self.zk.nodeIterator():
+        for node in self.zk_nodepool.nodeIterator():
             node_data = {}
             for key in ("id", "type", "connection_type", "external_id",
                         "provider", "state", "state_time", "comment"):
@@ -822,6 +929,7 @@ class ZuulWebAPI(object):
             'uuid': build.uuid,
             'job_name': build.job_name,
             'result': build.result,
+            'held': build.held,
             'start_time': start_time,
             'end_time': end_time,
             'duration': duration,
@@ -829,6 +937,7 @@ class ZuulWebAPI(object):
             'log_url': build.log_url,
             'node_name': build.node_name,
             'error_detail': build.error_detail,
+            'final': build.final,
             'artifacts': [],
             'provides': [],
         }
@@ -863,16 +972,8 @@ class ZuulWebAPI(object):
             })
         return ret
 
-    def _get_connection(self, tenant):
-        # Ask the scheduler which sql connection to use for this tenant
-        job = self.rpc.submitJob('zuul:tenant_sql_connection',
-                                 {'tenant': tenant})
-        connection_name = json.loads(job.data[0])
-
-        if not connection_name:
-            raise cherrypy.HTTPError(404, 'Tenant %s does not exist.' % tenant)
-
-        return self.zuulweb.connections.connections[connection_name]
+    def _get_connection(self):
+        return self.zuulweb.connections.connections['database']
 
     @cherrypy.expose
     @cherrypy.tools.save_params()
@@ -880,14 +981,21 @@ class ZuulWebAPI(object):
     def builds(self, tenant, project=None, pipeline=None, change=None,
                branch=None, patchset=None, ref=None, newrev=None,
                uuid=None, job_name=None, voting=None, node_name=None,
-               result=None, limit=50, skip=0):
-        connection = self._get_connection(tenant)
+               result=None, final=None, held=None, limit=50, skip=0):
+        connection = self._get_connection()
+
+        if tenant not in [x['name'] for x in self._tenants()]:
+            raise cherrypy.HTTPError(404, 'Tenant %s does not exist.' % tenant)
+
+        # If final is None, we return all builds, both final and non-final
+        if final is not None:
+            final = final.lower() == "true"
 
         builds = connection.getBuilds(
             tenant=tenant, project=project, pipeline=pipeline, change=change,
             branch=branch, patchset=patchset, ref=ref, newrev=newrev,
             uuid=uuid, job_name=job_name, voting=voting, node_name=node_name,
-            result=result, limit=limit, offset=skip)
+            result=result, final=final, held=held, limit=limit, offset=skip)
 
         resp = cherrypy.response
         resp.headers['Access-Control-Allow-Origin'] = '*'
@@ -897,7 +1005,7 @@ class ZuulWebAPI(object):
     @cherrypy.tools.save_params()
     @cherrypy.tools.json_out(content_type='application/json; charset=utf-8')
     def build(self, tenant, uuid):
-        connection = self._get_connection(tenant)
+        connection = self._get_connection()
 
         data = connection.getBuilds(tenant=tenant, uuid=uuid, limit=1)
         if not data:
@@ -924,14 +1032,19 @@ class ZuulWebAPI(object):
         }
         if builds:
             ret['builds'] = []
+            ret['retry_builds'] = []
         for build in builds:
-            ret['builds'].append(self.buildToDict(build))
+            # Put all non-final (retry) builds under a different key
+            if not build.final:
+                ret['retry_builds'].append(self.buildToDict(build))
+            else:
+                ret['builds'].append(self.buildToDict(build))
         return ret
 
     @cherrypy.expose
     @cherrypy.tools.save_params()
     def badge(self, tenant, project=None, pipeline=None, branch=None):
-        connection = self._get_connection(tenant)
+        connection = self._get_connection()
 
         buildsets = connection.getBuildsets(
             tenant=tenant, project=project, pipeline=pipeline,
@@ -945,6 +1058,9 @@ class ZuulWebAPI(object):
             file = 'failing.svg'
         path = os.path.join(self.zuulweb.static_path, file)
 
+        # Ensure the badge are not cached
+        cherrypy.response.headers['Cache-Control'] = "no-cache"
+
         return cherrypy.lib.static.serve_file(
             path=path, content_type="image/svg+xml")
 
@@ -954,7 +1070,7 @@ class ZuulWebAPI(object):
     def buildsets(self, tenant, project=None, pipeline=None, change=None,
                   branch=None, patchset=None, ref=None, newrev=None,
                   uuid=None, result=None, limit=50, skip=0):
-        connection = self._get_connection(tenant)
+        connection = self._get_connection()
 
         buildsets = connection.getBuildsets(
             tenant=tenant, project=project, pipeline=pipeline, change=change,
@@ -970,7 +1086,7 @@ class ZuulWebAPI(object):
     @cherrypy.tools.save_params()
     @cherrypy.tools.json_out(content_type='application/json; charset=utf-8')
     def buildset(self, tenant, uuid):
-        connection = self._get_connection(tenant)
+        connection = self._get_connection()
 
         data = connection.getBuildset(tenant, uuid)
         if not data:
@@ -1006,6 +1122,30 @@ class ZuulWebAPI(object):
         resp.headers['Access-Control-Allow-Origin'] = '*'
         return ret
 
+    @cherrypy.expose
+    @cherrypy.tools.save_params()
+    @cherrypy.tools.json_out(content_type='application/json; charset=utf-8')
+    def project_freeze_job(self, tenant, pipeline, project, branch, job):
+        # TODO(jhesketh): Allow a canonical change/item to be passed in which
+        # would return the job with any in-change modifications.
+
+        job = self.rpc.submitJob(
+            'zuul:project_freeze_job',
+            {
+                'tenant': tenant,
+                'project': project,
+                'pipeline': pipeline,
+                'branch': branch,
+                'job': job
+            }
+        )
+        ret = json.loads(job.data[0])
+        if not ret:
+            raise cherrypy.HTTPError(404)
+        resp = cherrypy.response
+        resp.headers['Access-Control-Allow-Origin'] = '*'
+        return ret
+
 
 class StaticHandler(object):
     def __init__(self, root):
@@ -1093,36 +1233,51 @@ class StreamManager(object):
 class ZuulWeb(object):
     log = logging.getLogger("zuul.web.ZuulWeb")
 
-    def __init__(self, listen_address, listen_port, gear_server,
-                 gear_port, ssl_key=None, ssl_cert=None, ssl_ca=None,
-                 static_cache_expiry=3600, connections=None,
-                 info=None, static_path=None, zk_hosts=None,
-                 zk_timeout=None, zk_tls_cert=None, zk_tls_key=None,
-                 zk_tls_ca=None, authenticators=None,
-                 command_socket=None):
+    def __init__(self,
+                 config,
+                 connections,
+                 authenticators: AuthenticatorRegistry,
+                 info: zuul.model.WebInfo = None):
         self.start_time = time.time()
-        self.listen_address = listen_address
-        self.listen_port = listen_port
-        self.event_loop = None
-        self.term = None
+        self.config = config
+        self.listen_address = get_default(self.config,
+                                          'web', 'listen_address',
+                                          '127.0.0.1')
+        self.listen_port = get_default(self.config, 'web', 'port', 9000)
         self.server = None
-        self.static_cache_expiry = static_cache_expiry
+        self.static_cache_expiry = get_default(self.config, 'web',
+                                               'static_cache_expiry',
+                                               3600)
         self.info = info
-        self.static_path = os.path.abspath(static_path or STATIC_DIR)
+        self.static_path = os.path.abspath(
+            get_default(self.config, 'web', 'static_path', STATIC_DIR)
+        )
+        self.hostname = socket.getfqdn()
+
+        gear_server = get_default(self.config, 'gearman', 'server')
+        gear_port = get_default(self.config, 'gearman', 'port', 4730)
+        ssl_key = get_default(self.config, 'gearman', 'ssl_key')
+        ssl_cert = get_default(self.config, 'gearman', 'ssl_cert')
+        ssl_ca = get_default(self.config, 'gearman', 'ssl_ca')
+
         # instanciate handlers
         self.rpc = zuul.rpcclient.RPCClient(gear_server, gear_port,
                                             ssl_key, ssl_cert, ssl_ca,
                                             client_id='Zuul Web Server')
-        self.zk = zuul.zk.ZooKeeper(enable_cache=True)
-        if zk_hosts:
-            self.zk.connect(hosts=zk_hosts, read_only=True,
-                            timeout=zk_timeout, tls_cert=zk_tls_cert,
-                            tls_key=zk_tls_key, tls_ca=zk_tls_ca)
+        self.zk_client = ZooKeeperClient.fromConfig(self.config)
+        self.zk_client.connect()
+        self.component_info = WebComponent(self.zk_client, self.hostname)
+        self.component_info.register()
 
         self.connections = connections
         self.authenticators = authenticators
         self.stream_manager = StreamManager()
 
+        command_socket = get_default(
+            self.config, 'web', 'command_socket',
+            '/var/lib/zuul/web.socket'
+        )
+
         self.command_socket = commandsocket.CommandSocket(command_socket)
 
         self.repl = None
@@ -1158,7 +1313,12 @@ class ZuulWeb(object):
             # route order is important, put project actions before the more
             # generic tenant/{tenant}/project/{project} route
             route_map.connect('api', '/api/user/authorizations',
-                              controller=api, action='user_authorizations')
+                              controller=api, action='authorizations')
+            route_map.connect('api', '/api/tenant/{tenant}/authorizations',
+                              controller=api,
+                              action='tenant_authorizations')
+            route_map.connect('api', '/api/tenant/{tenant}/promote',
+                              controller=api, action='promote')
             route_map.connect(
                 'api',
                 '/api/tenant/{tenant}/project/{project:.*}/autohold',
@@ -1172,7 +1332,8 @@ class ZuulWeb(object):
                 '/api/tenant/{tenant}/project/{project:.*}/dequeue',
                 controller=api, action='dequeue')
         route_map.connect('api', '/api/tenant/{tenant}/autohold/{request_id}',
-                          controller=api, action='autohold_by_request_id')
+                          controller=api,
+                          action='autohold_by_request_id')
         route_map.connect('api', '/api/tenant/{tenant}/autohold',
                           controller=api, action='autohold_list')
         route_map.connect('api', '/api/tenant/{tenant}/projects',
@@ -1185,6 +1346,12 @@ class ZuulWeb(object):
             '/project/{project:.*}/branch/{branch:.*}/freeze-jobs',
             controller=api, action='project_freeze_jobs'
         )
+        route_map.connect(
+            'api',
+            '/api/tenant/{tenant}/pipeline/{pipeline}'
+            '/project/{project:.*}/branch/{branch:.*}/freeze-job/{job}',
+            controller=api, action='project_freeze_job'
+        )
         route_map.connect('api', '/api/tenant/{tenant}/pipelines',
                           controller=api, action='pipelines')
         route_map.connect('api', '/api/tenant/{tenant}/labels',
@@ -1232,8 +1399,8 @@ class ZuulWeb(object):
         cherrypy.config.update({
             'global': {
                 'environment': 'production',
-                'server.socket_host': listen_address,
-                'server.socket_port': int(listen_port),
+                'server.socket_host': self.listen_address,
+                'server.socket_port': int(self.listen_port),
             },
         })
 
@@ -1257,9 +1424,11 @@ class ZuulWeb(object):
                                                name='command')
         self.command_thread.daemon = True
         self.command_thread.start()
+        self.component_info.state = self.component_info.RUNNING
 
     def stop(self):
         self.log.debug("ZuulWeb stopping")
+        self.component_info.state = self.component_info.STOPPED
         self.rpc.shutdown()
         cherrypy.engine.exit()
         # Not strictly necessary, but without this, if the server is
@@ -1268,7 +1437,7 @@ class ZuulWeb(object):
         cherrypy.server.httpserver = None
         self.wsplugin.unsubscribe()
         self.stream_manager.stop()
-        self.zk.disconnect()
+        self.zk_client.disconnect()
         self.stop_repl()
         self._command_running = False
         self.command_socket.stop()
